/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 3.0
 *
 * The contents of this file are subject to the General Public License
 * 3.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * -- Exchange 2007/2010 Calendar and Tasks Provider.
 * -- For Thunderbird with the Lightning add-on.
 *
 * This work is a combination of the Storage calendar, part of the default Lightning add-on, and 
 * the "Exchange Data Provider for Lightning" add-on currently, october 2011, maintained by Simon Schubert.
 * Primarily made because the "Exchange Data Provider for Lightning" add-on is a continuation 
 * of old code and this one is build up from the ground. It still uses some parts from the 
 * "Exchange Data Provider for Lightning" project.
 *
 * Author: Michel Verbraak (info@1st-setup.nl)
 * Website: http://www.1st-setup.nl/wordpress/?page_id=133
 * email: exchangecalendar@extensions.1st-setup.nl
 *
 *
 * This code uses parts of the Microsoft Exchange Calendar Provider code on which the
 * "Exchange Data Provider for Lightning" was based.
 * The Initial Developer of the Microsoft Exchange Calendar Provider Code is
 *   Andrea Bittau <a.bittau@cs.ucl.ac.uk>, University College London
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * ***** BEGIN LICENSE BLOCK *****/

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;


Cu.import("resource://calendar/modules/calUtils.jsm");

function checkRequired()
{
    let canAdvance = true;
    let vbox = document.getElementById('ec-exchange-settings');
    if (vbox) {
        let eList = vbox.getElementsByAttribute('required', 'true');
        for (let i = 0; i < eList.length && canAdvance; ++i) {
            canAdvance = (eList[i].value != "");
        }

        if (canAdvance) {
		document.getElementById("ecExchangeSettings").buttons = "accept,cancel";
	}
	else {
		document.getElementById("ecExchangeSettings").buttons = "cancel";
	}
    }

	window.sizeToContent() ;

}

function safeGetBoolPref(aBranch, aName, aDefaultValue)
{
	try {
		return aBranch.getBoolPref(aName);
	}
	catch(err) {
		return aDefaultValue;
	}
}

function safeGetIntPref(aBranch, aName, aDefaultValue)
{
	try {
		return aBranch.getIntPref(aName);
	}
	catch(err) {
		return aDefaultValue;
	}
}

function safeGetCharPref(aBranch, aName, aDefaultValue)
{
	try {
		return aBranch.getCharPref(aName);
	}
	catch(err) {
		return aDefaultValue;
	}
}

function exchWebServicesOnLoad()
{
	var calId = window.arguments[0].calendar.id;
	document.getElementById("ecExchangeSettings-title").value = window.arguments[0].calendar.name;
	exchWebServicesLoadExchangeSettingsByCalId(calId);

	// Load meeting request settings.
	var exchWebServicesCalPrefs = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService)
		    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl."+calId+".");

	document.getElementById("ec-poll-inbox").checked = safeGetBoolPref(exchWebServicesCalPrefs, "ecPollInbox", true);
	document.getElementById("ec-poll-inbox-interval").value = safeGetIntPref(exchWebServicesCalPrefs, "ecPollInboxInterval", 3);
	document.getElementById("ec-autorespond-meetingrequest").checked = safeGetBoolPref(exchWebServicesCalPrefs, "ecAutoRespondMeetingRequest", false); 
	document.getElementById("ec-autorespond-answer").value = safeGetCharPref(exchWebServicesCalPrefs, "ecAutoRespondAnswer", "TENTATIVE"); 

	document.getElementById("ec-autoremove-invitation_cancellation1").checked = safeGetBoolPref(exchWebServicesCalPrefs, "ecAutoRemoveInvitationCancellation1", false); 
	document.getElementById("ec-autoremove-invitation_cancellation2").checked = safeGetBoolPref(exchWebServicesCalPrefs, "ecAutoRemoveInvitationCancellation2", false); 

	document.getElementById("ec-autoremove-invitation_response1").checked = safeGetBoolPref(exchWebServicesCalPrefs, "ecAutoRemoveInvitationResponse1", true); 

	document.getElementById("ec-doautorespond-meetingrequest-message").checked = safeGetBoolPref(exchWebServicesCalPrefs, "ecSendAutoRespondMeetingRequestMessage", false); 
	document.getElementById("ec-autorespond-meetingrequest-message").value = safeGetCharPref(exchWebServicesCalPrefs, "ecAutoRespondMeetingRequestMessage", ""); 

	exchWebServicesCheckFolderSettings();
}

function exchWebServicesOnSave()
{
	var calId = window.arguments[0].calendar.id;
	var aCalendar = window.arguments[0].calendar;
	exchWebServicesSaveExchangeSettingsByCalId(calId);

	// Save meeting request settings.
	var exchWebServicesCalPrefs = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService)
		    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl."+calId+".");

	exchWebServicesCheckFolderSettings();

	exchWebServicesCalPrefs.setBoolPref("ecPollInbox", document.getElementById("ec-poll-inbox").checked); 
	exchWebServicesCalPrefs.setIntPref("ecPollInboxInterval", document.getElementById("ec-poll-inbox-interval").value); 
	exchWebServicesCalPrefs.setBoolPref("ecAutoRespondMeetingRequest", document.getElementById("ec-autorespond-meetingrequest").checked); 
	exchWebServicesCalPrefs.setCharPref("ecAutoRespondAnswer", document.getElementById("ec-autorespond-answer").value); 

	exchWebServicesCalPrefs.setBoolPref("ecAutoRemoveInvitationCancellation1", document.getElementById("ec-autoremove-invitation_cancellation1").checked); 
	exchWebServicesCalPrefs.setBoolPref("ecAutoRemoveInvitationCancellation2", document.getElementById("ec-autoremove-invitation_cancellation2").checked); 

	exchWebServicesCalPrefs.setBoolPref("ecAutoRemoveInvitationResponse1", document.getElementById("ec-autoremove-invitation_response1").checked); 

	exchWebServicesCalPrefs.setBoolPref("ecSendAutoRespondMeetingRequestMessage", document.getElementById("ec-doautorespond-meetingrequest-message").checked); 
	exchWebServicesCalPrefs.setCharPref("ecAutoRespondMeetingRequestMessage", document.getElementById("ec-autorespond-meetingrequest-message").value); 

	window.arguments[0].answer = "saved";

	var observerService = Cc["@mozilla.org/observer-service;1"]  
	                          .getService(Ci.nsIObserverService);  
	observerService.notifyObservers(this, "onCalReset", calId);  

	return true;
}

function exchWebServicesCheckFolderSettings()
{
	if ((document.getElementById("ecfolderbase").value != "calendar") ||
	    (document.getElementById("ecfolderpath").value != "/")) {
		// Disable inbox polling.
		document.getElementById("ec-poll-inbox").checked = false;
		document.getElementById("vbox-ec-meetingrequestsettings").hidden = true;
		document.getElementById("vbox-ec-nomeetingrequestsettings").hidden = false;
	}
	else {
		document.getElementById("vbox-ec-meetingrequestsettings").hidden = false;
		document.getElementById("vbox-ec-nomeetingrequestsettings").hidden = true;
	}

	document.getElementById("ec-poll-inbox-interval").disabled = (!document.getElementById("ec-poll-inbox").checked);

	document.getElementById("ec-autorespond-answer").disabled = (!document.getElementById("ec-autorespond-meetingrequest").checked);
	document.getElementById("ec-doautorespond-meetingrequest-message").disabled = (!document.getElementById("ec-autorespond-meetingrequest").checked);
	document.getElementById("ec-autorespond-meetingrequest-message").disabled = (!document.getElementById("ec-autorespond-meetingrequest").checked);

	window.sizeToContent() ;
}

function exchWebServicesDoTabChanged(aTabs)
{
	exchWebServicesCheckFolderSettings();
}

function exchWebServicesDoPollInboxChanged(aCheckbox)
{
	exchWebServicesCheckFolderSettings();
}

function exchWebServicesDoAutoRespondChanged(aCheckbox)
{
	exchWebServicesCheckFolderSettings();
}
