/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 3.0
 *
 * The contents of this file are subject to the General Public License
 * 3.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * -- Exchange 2007/2010 Calendar and Tasks Provider.
 * -- For Thunderbird with the Lightning add-on.
 *
 * This work is a combination of the Storage calendar, part of the default Lightning add-on, and 
 * the "Exchange Data Provider for Lightning" add-on currently, october 2011, maintained by Simon Schubert.
 * Primarily made because the "Exchange Data Provider for Lightning" add-on is a continuation 
 * of old code and this one is build up from the ground. It still uses some parts from the 
 * "Exchange Data Provider for Lightning" project.
 *
 * Author: Michel Verbraak (info@1st-setup.nl)
 * Website: http://www.1st-setup.nl/wordpress/?page_id=133
 * email: exchangecalendar@extensions.1st-setup.nl
 *
 *
 * ***** BEGIN LICENSE BLOCK *****/

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;


Cu.import("resource://calendar/modules/calUtils.jsm");
Cu.import("resource://exchangecalendar/ecFunctions.js");

var exchWebServicesProgressPanel = {

	queueSizeTotal: 0,
	calendarQueues: {},
	runningJobs: 0,
	changeQueue: [],
	busy: false,
	isLoaded: false,
	imageCounter: 1,
	imageList: {    image1: "chrome://exchangecalendar/content/arrow-circle.png",
			image2: "chrome://exchangecalendar/content/arrow-circle-315.png",
			image3: "chrome://exchangecalendar/content/arrow-circle-225.png",
			image4: "chrome://exchangecalendar/content/arrow-circle-135.png"
			},

	notify: function ecProgressPanel_notify() 
	{
		if (this.isLoaded) {
			this.imageCounter = Number(this.imageCounter) + Number(1);
			if (this.imageCounter > 4) {
				this.imageCounter = 1;
			}
			if (document) {
				document.getElementById("exchange-progress-image").style.listStyleImage = "url('"+this.imageList["image"+this.imageCounter]+"')";
			}
		}
	},

	observe: function ecProgressPanel_observe(subject, topic, data) 
	{
		if (topic == "onExchangeProgressChange") {
			this.changeQueue.push({ jobchange: Number(data), calendar: subject});
		}

		if (!this.busy) {
			this.busy = true;
			while (this.changeQueue.length > 0) {
				if (this.changeQueue[0].jobchange != 0) {

					this.queueSizeTotal = this.queueSizeTotal + this.changeQueue[0].jobchange;

					if (!this.calendarQueues[this.changeQueue[0].calendar.id]) {
						this.calendarQueues[this.changeQueue[0].calendar.id] = {};
					}
					this.calendarQueues[this.changeQueue[0].calendar.id].name = this.changeQueue[0].calendar.name;
					if (this.calendarQueues[this.changeQueue[0].calendar.id].queueSize) {
						this.calendarQueues[this.changeQueue[0].calendar.id].queueSize = this.calendarQueues[this.changeQueue[0].calendar.id].queueSize + this.changeQueue[0].jobchange; 
					}
					else {
						this.calendarQueues[this.changeQueue[0].calendar.id].queueSize = this.changeQueue[0].jobchange; 
					}

					if (this.calendarQueues[this.changeQueue[0].calendar.id].queueSize == 0) {
						if (document.getElementById(this.changeQueue[0].calendar.id)) {
							document.getElementById("ewsprogressvbox").deleteChild(document.getElementById(this.changeQueue[0].calendar.id));
						}
						delete this.calendarQueues[this.changeQueue[0].calendar.id];
					}

				}
				this.changeQueue.shift();
			}

			if (this.isLoaded) {
				if (this.queueSizeTotal == 0) {
					document.getElementById("exchange-progress-panel").hidden = true;
					this.timer.cancel();
				}
				else {
					if (document.getElementById("exchange-progress-panel").hidden) {
						if (this.queueSizeTotal > 1) {
							document.getElementById("exchange-progress-panel").hidden = false;
							this.timer.initWithCallback(this, 200, this.timer.TYPE_REPEATING_SLACK);
						}
					}
					var tmpStr = this.queueSizeTotal + " job";
					if (this.queueSizeTotal > 1) {
						tmpStr = tmpStr + "s";
					}
					document.getElementById("exchange-progress-label").value = tmpStr;
				}

				// Update the tooltip
				var counter = 1;
				for (var calendarQueue in this.calendarQueues) {
					document.getElementById("ewsprogress"+counter).value = this.calendarQueues[calendarQueue].name+": "+this.calendarQueues[calendarQueue].queueSize;
					document.getElementById("ewsprogress"+counter).hidden = false;
					counter = counter + 1;
				}
				while (counter <= 10) {
					document.getElementById("ewsprogress"+counter).hidden = true;
					counter = counter + 1;
				}
			}

			this.busy = false;
		}

        },

	init: function ecProgressPanel_init()
	{
		this.observerService = Cc["@mozilla.org/observer-service;1"]  
                          .getService(Ci.nsIObserverService);

		this.observerService.addObserver(this, "onExchangeProgressChange", false);

		this.timer = Cc["@mozilla.org/timer;1"]
				.createInstance(Ci.nsITimer);

	},

	destroy: function ecProgressPanel_destroy()
	{
		this.observerService.removeObserver(this, "onExchangeProgressChange");
	},

	loaded: function ecProgressPanel_loaded()
	{
		this.isLoaded = true;
	},

}

function exchWebServicesOnLoad(event) {

	// nuke the onload, or we get called every time there's
	// any load that occurs
	document.removeEventListener("load", exchWebServicesOnLoad, true);

	// Do init
	exchWebServicesProgressPanel.loaded();

	// Add an unload function to the window so we don't leak any listeners
	window.addEventListener("unload", exchWebServicesOnFinish, false);


}

function exchWebServicesOnFinish() {
	// Do some cleanup
	exchWebServicesProgressPanel.destroy();
}

function exchWebServicesOpenProgressDialog()
{
}

exchWebServicesProgressPanel.init();
document.addEventListener("load", exchWebServicesOnLoad, true);
