/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 3.0
 *
 * The contents of this file are subject to the General Public License
 * 3.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.gnu.org/licenses/gpl.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * -- Exchange 2007/2010 Calendar and Tasks Provider.
 * -- For Thunderbird with the Lightning add-on.
 *
 * This work is a combination of the Storage calendar, part of the default Lightning add-on, and 
 * the "Exchange Data Provider for Lightning" add-on currently, october 2011, maintained by Simon Schubert.
 * Primarily made because the "Exchange Data Provider for Lightning" add-on is a continuation 
 * of old code and this one is build up from the ground. It still uses some parts from the 
 * "Exchange Data Provider for Lightning" project.
 *
 * Author: Michel Verbraak (info@1st-setup.nl)
 * Website: http://www.1st-setup.nl/wordpress/?page_id=133
 * email: exchangecalendar@extensions.1st-setup.nl
 *
 *
 * This code uses parts of the Microsoft Exchange Calendar Provider code on which the
 * "Exchange Data Provider for Lightning" was based.
 * The Initial Developer of the Microsoft Exchange Calendar Provider Code is
 *   Andrea Bittau <a.bittau@cs.ucl.ac.uk>, University College London
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * ***** BEGIN LICENSE BLOCK *****/

var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;
var Cr = Components.results;
var components = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

Cu.import("resource://calendar/modules/calUtils.jsm");
Cu.import("resource://calendar/modules/calAlarmUtils.jsm");
Cu.import("resource://calendar/modules/calProviderUtils.jsm");
Cu.import("resource://calendar/modules/calAuthUtils.jsm");

Cu.import("resource://exchangecalendar/ecFunctions.js");
Cu.import("resource://exchangecalendar/soapFunctions.js");
Cu.import("resource://exchangecalendar/ecExchangeRequest.js");
Cu.import("resource://exchangecalendar/erFindFolder.js");
Cu.import("resource://exchangecalendar/erGetFolder.js");
Cu.import("resource://exchangecalendar/erFindCalendarItems.js");
Cu.import("resource://exchangecalendar/erFindTaskItems.js");
Cu.import("resource://exchangecalendar/erGetItems.js");
Cu.import("resource://exchangecalendar/erGetMeetingRequestByUID.js");
Cu.import("resource://exchangecalendar/erGetOccurrenceIndex.js");
Cu.import("resource://exchangecalendar/erDeleteItem.js");
Cu.import("resource://exchangecalendar/erFindMasterOccurrences.js");
Cu.import("resource://exchangecalendar/erSyncFolderItems.js");
Cu.import("resource://exchangecalendar/erFindOccurrences.js");
Cu.import("resource://exchangecalendar/erUpdateItem.js");
Cu.import("resource://exchangecalendar/erGetUserAvailability.js");
Cu.import("resource://exchangecalendar/erCreateItem.js");
Cu.import("resource://exchangecalendar/erSendMeetingRespons.js");
Cu.import("resource://exchangecalendar/erSyncInbox.js");
Cu.import("resource://exchangecalendar/erGetTimeZones.js");
Cu.import("resource://exchangecalendar/ecEWSTimeZoneDefinitions.js");


var tmpActivityManager = Cc["@mozilla.org/activity-manager;1"];

if (tmpActivityManager) {
	ews.LOG("-- ActivityManger available. Enabeling it.");
	const nsIAP = Ci.nsIActivityProcess;  
	const nsIAE = Ci.nsIActivityEvent;  
	const nsIAM = Ci.nsIActivityManager;

	var gActivityManager = Cc["@mozilla.org/activity-manager;1"].getService(nsIAM);  
}
else {
	ews.LOG("-- ActivityManger not available.");
}

const fieldPathMap = {
	'Body'			: 'item',
	'CalendarItemType'	: 'calendar',
	'Categories'		: 'item',
	'DateTimeReceived'	: 'item',
	'DeletedOccurrences'	: 'calendar',
	'End'			: 'calendar',
	'FirstOccurrence'	: 'calendar',
	'IsAllDayEvent'		: 'calendar',
	'IsCancelled'		: 'calendar',
	'LastOccurrence'	: 'calendar',
	'Location'		: 'calendar',
	'MeetingTimeZone'	: 'calendar',
	'ModifiedOccurrences'	: 'calendar',
	'MyResponseType'	: 'calendar',
	'OptionalAttendees'	: 'calendar',
	'Organizer'		: 'calendar',
	'Recurrence'		: 'calendar',
	'RecurrenceId'		: 'calendar',
	'ReminderDueBy'		: 'item',
	'ReminderIsSet'		: 'item',
	'ReminderMinutesBeforeStart' : 'item',
	'RequiredAttendees'	: 'calendar',
	'Start'			: 'calendar',
	'Subject'		: 'item',
	'UID'			: 'calendar',
	'Importance'		: 'item',
	'Sensitivity'		: 'item',
	'LegacyFreeBusyStatus'  : 'calendar',
	'Status'		: 'task',
	'StartDate'		: 'task',
	'DueDate'		: 'task',
	'PercentComplete'	: 'task',
	'CompleteDate'		: 'task',
	'DateTimeCreated'	: 'item',
	'LastModifiedTime'	: 'item'
};

const dayRevMap = {
	'MO' : 'Monday',
	'TU' : 'Tuesday',
	'WE' : 'Wednesday',
	'TH' : 'Thursday',
	'FR' : 'Friday',
	'SA' : 'Saturday',
	'SU' : 'Sunday'
};

const dayIdxMap = ['SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA'];

const weekRevMap = {
	'1' : 'First',
	'2' : 'Second',
	'3' : 'Third',
	'4' : 'Fourth',
	'-1': 'Last'
};

const monthIdxMap = ['January', 'February', 'March', 'April', 'May', 'June',
		     'July', 'August', 'September', 'October', 'November', 'December'];

const participationMap = {
	"Unknown"	: "NEEDS-ACTION",
	"NoResponseReceived" : "NEEDS-ACTION",
	"Tentative"	: "TENTATIVE",
	"Accept"	: "ACCEPTED",
	"Decline"	: "DECLINED",
	"Organizer"	: "ACCEPTED"
};

const dayMap = {
	'Monday'	: 'MO',
	'Tuesday'	: 'TU',
	'Wednesday'	: 'WE',
	'Thursday'	: 'TH',
	'Friday'	: 'FR',
	'Saturday'	: 'SA',
	'Sunday'	: 'SU',
	'Weekday'	: ['MO', 'TU', 'WE', 'TH', 'FR'],
	'WeekendDay'	: ['SA', 'SO'],
	'Day'		: ['MO', 'TU', 'WE', 'TH', 'FR', 'SA', 'SO']
};

const weekMap = {
	'First'		: 1,
	'Second'	: 2,
	'Third'		: 3,
	'Fourth'	: 4,
	'Last'		: -1
};

const monthMap = {
	'January'	: 1,
	'February'	: 2,
	'March'		: 3,
	'April'		: 4,
	'May'		: 5,
	'June'		: 6,
	'July'		: 7,
	'August'	: 8,
	'September'	: 9,
	'October'	: 10,
	'November'	: 11,
	'December'	: 12
};

//
// calExchangeCalendar
//

var EXPORTED_SYMBOLS = ["calExchangeCalendar", "fixPrefBug", "removeOldPrefs"];

function calExchangeCalendar() {
	this.initProviderBase();

	this.folderPathStatus = 1;
	this.firstrun = true;
	this.mUri = "";
	this.mid = null;

//	this.initialized = false;

	this.prefs = null;

	this.myAvailable = false;

	this.itemCache = new Array;
	this.recurringMasterCache = new Array;
	this.newMasters = new Array;

	this.startDate = null;
	this.endDate = null;

	this.syncState = null;
	this.syncInboxState = null;
	this.syncBusy = false;
	this.weAreSyncing = false;
	this.firstSyncDone = false;

	this.meetingRequestsCache = [];
	this.meetingCancelationsCache = [];
	this.meetingrequestAnswered = [];
	this.meetingResponsesCache = [];

	this.queue = new Array;
	this.tmpJobs = new Array;
	this.getItemSyncQueue = [];
	this.processItemSyncQueueBusy = false;
	this.timers = [];

	this.doReset = false;

	this.haveTimeZones = false;
	this.EWSTimeZones = null;

	this.shutdown = false;

	this.inboxPoller = Cc["@mozilla.org/timer;1"]
			.createInstance(Ci.nsITimer);

	this.observerService = Cc["@mozilla.org/observer-service;1"]  
	                          .getService(Ci.nsIObserverService);  

	this.calendarPoller = null;

        this.mObserver = new ecObserver(this);

	this.supportsTasks = true;
	this.supportsEvents = true;
}

var calExchangeCalendarGUID = "720a458e-b6cd-4883-8a4d-5be27ec454d8";

calExchangeCalendar.prototype = {
	__proto__: cal.ProviderBase.prototype,

// Begin nsIClassInfo
	classID: components.ID("{"+calExchangeCalendarGUID+"}"),
	contractID: "@mozilla.org/calendar/calendar;1?type=exchangecalendar",
	classDescription: "Exchange 2007/2010 Calendar and Tasks Provider",

	// void getInterfaces(out PRUint32 count, [array, size_is(count), retval] out nsIIDPtr array);
	getInterfaces: function _getInterfaces(count) 
	{
		var ifaces = [Ci.calICalendar,
			Ci.calICalendarProvider,
			Ci.calIFreeBusyService,
			Ci.calISchedulingSupport,
			Ci.calICalendarProvider,
			Ci.nsIClassInfo,
			Ci.nsISupports];
		count.value = ifaces.length;
		return ifaces;
	},

	// nsISupports getHelperForLanguage(in PRUint32 language);
	getHelperForLanguage: function (language) {
		return null;
	},

	implementationLanguage: Ci.nsIProgrammingLanguage.JAVASCRIPT,
	flags: 0,
// End nsIClassInfo

// Begin nsISupports
	// void QueryInterface(in nsIIDRef uuid, [iid_is(uuid),retval] out nsQIResult result);
	QueryInterface: function _QueryInterface(aIID) {
		return cal.doQueryInterface(this, calExchangeCalendar.prototype, aIID,null, this);
	},
// End nsISupports

// Begin calICalendarProvider
	// readonly attribute nsIURI prefChromeOverlay;
	get prefChromeOverlay() {
		this.logInfo("get prefChromeOverlay()");
		return null;
	},

	get displayName() {
		this.logInfo("get displayName()");
		return calGetString("calExchangeCalendar", "displayName", null, "exchangecalendar");
	},

	//  void createCalendar(in AUTF8String aName, in nsIURI aURL,
        //	            in calIProviderListener aListener);
	createCalendar: function calWcapCalendar_createCalendar(name, url, listener) {
		this.logInfo("createCalendar");
		throw NS_ERROR_NOT_IMPLEMENTED;
	},

	//  void deleteCalendar(in calICalendar aCalendar,
        //  	            in calIProviderListener aListener);
	deleteCalendar: function calWcapCalendar_deleteCalendar(calendar, listener) {
		this.logInfo("deleteCalendar");
		throw NS_ERROR_NOT_IMPLEMENTED;
	},

	//  calICalendar getCalendar(in nsIURI aURL);
	getCalendar: function calWcapCalendar_getCalendar(url) {
		this.logInfo("getCalendar");
		throw NS_ERROR_NOT_IMPLEMENTED;
	},
// End calICalendarProvider

// Begin calICalendar

	//  attribute AUTF8String id;

	//   attribute AUTF8String name;

	//  readonly attribute AUTF8String type;
	get type()
	{
//		this.logInfo("get type()");
		return "exchangecalendar";
	},

	//  readonly attribute AString providerID;
	get providerID()
	{
		return "exchangecalendar@extensions.1st-setup.nl";
	},

	//  attribute calICalendar superCalendar;

	//  attribute nsIURI uri;
	get uri()
	{
	        return this.mUri;
	},

	set uri(aUri)
	{
	        this.mUri = aUri;
	        return this.uri;
	},

	//  attribute boolean readOnly;
	get readOnly() {
		return false;
	},

	//  readonly attribute boolean canRefresh;
	get canRefresh() {
		return true;
	},

	//  attribute boolean transientProperties;

	//  nsIVariant getProperty(in AUTF8String aName);
	getProperty: function _getProperty(aName) 
	{
		//LOG("1 getProperty("+aName+")");
	//	if (!this.isInitialized) {
	//		return;
	//	}

		//LOG("2 getProperty("+aName+")");
		switch (aName) {
		case "capabilities.tasks.supported" :
			return this.supportsTasks;
			break;
		case "capabilities.events.supported" :
			return this.supportsEvents;
			break;
		case "auto-enabled":
			return true;
		case "organizerId":
			//this.logInfo("getProperty: organizerId");
			return "exchangecalendar:" + this.mailbox;
	                break;
		case "organizerCN":
			return this.userDisplayName;
	                break;
		case "cache.supported":
			return false;
		case "requiresNetwork":
                	return true;
		case "disabled":
//			return !this.myAvailable;
			return ((!this.isInitialized) && (this.folderPathStatus == 0));
		case "itip.notify-replies":
			return true;
		case "itip.transport":
			this.logInfo("getProperty: itip.transport");
			return Cc["@mozilla.org/calendar/itip-transport;1?type=exchangecalendar"]
                        		.getService(Ci.calIItipTransport);
            	case "capabilities.autoschedule.supported":
			this.logInfo("capabilities.autoschedule.supported");
                	return true;
	        }
		// itip.disableRevisionChecks

		// capabilities.events.supported
		// capabilities.tasks.supported

		//LOG("getProperty("+aName+")");
	        return this.__proto__.__proto__.getProperty.apply(this, arguments);
	},

	//  void setProperty(in AUTF8String aName, in nsIVariant aValue);

	//  void deleteProperty(in AUTF8String aName);

	//  void addObserver( in calIObserver observer );
	//  void removeObserver( in calIObserver observer );

	//  calIOperation addItem(in calIItemBase aItem,
        //                in calIOperationListener aListener);
	addItem: function _addItem(aItem, aListener)
	{
		this.logInfo("addItem id="+aItem.id);

	        //let newItem = aItem.clone();
	        let newItem = aItem;
	
		// We check if we not allready have this item in Cache. If so we modify.
		// This will happen when some pressed the accept,decline or tentative buttons
		// in the itip status bar on the header of an email message.
		if (this.itemCache[newItem.id]) {
			return this.modifyItem(newItem, this.itemCache[newItem.id]);
		}

	        return this.adoptItem(newItem, aListener);
	},


	//  calIOperation adoptItem(in calIItemBase aItem,
        //                  in calIOperationListener aListener);
	adoptItem: function _adoptItem(aItem, aListener)
	{
		this.logInfo("adoptItem()");
		if (this.readOnly) {
			this.notifyOperationComplete(aListener,
                                         Ci.calIErrors.CAL_IS_READONLY,
                                         Ci.calIOperationListener.ADD,
                                         null,
                                         "Calendar is readonly");
			return;
		}

		if (isEvent(aItem)) {	
			if (aItem.id) {
				// This is and item create through an iTIP response.
				var cachedItem = null;
				for (var index in this.meetingRequestsCache) {
					if (this.meetingRequestsCache[index]) {
						if (this.meetingRequestsCache[index].getProperty("X-UID") == aItem.id) {
							cachedItem = this.meetingRequestsCache[index];
							break;
						}
					}
				}

				if (cachedItem) {
					// We have meeting request in our cache.
					// Send meetingrespons base on status and remove message in inbox.
					this.logInfo("BOA: iTIP action item with STATUS:"+aItem.getProperty("STATUS"));
					//var me = this.getInvitedAttendee(aItem);
				/*	if (me) {
						this.logInfo("BOA2: participationStatus:"+me.participationStatus);
					}
					else {
						this.logInfo("BOA3: geen status");
					} */

					//var aNewItem = aItem.clone();
					var aNewItem = this.cloneItem(aItem);
					aNewItem.setProperty("X-UID", cachedItem.getProperty("X-UID"));
					aNewItem.setProperty("X-ChangeKey", cachedItem.getProperty("X-ChangeKey"));
					aNewItem.setProperty("X-MEETINGREQUEST", cachedItem.getProperty("X-MEETINGREQUEST"));
					aNewItem.id = cachedItem.id;


					if ((aNewItem) && (this.sendMeetingRespons(aNewItem, null, "new"))) {
						// remove message in inbox.
						//this.removeMeetingItem(this.meetingRequestsCache[aItem.id]);
						//this.observers.notify("onDeleteItem", [this.meetingRequestsCache[aItem.id]]);
						//this.removeFromMeetingRequestCache(aItem.id);
						//this.meetingRequestsCache[aItem.id] = null;
						//this.meetingrequestAnswered[aItem.id] = false;
						this.notifyOperationComplete(aListener,
			        	                             Cr.NS_OK,
			                        	             Ci.calIOperationListener.ADD,
			                        	             aItem.id,
			                        	             aNewItem);
					}
					else {
						// User cancel.. How to respond!!
						this.notifyOperationComplete(aListener,
			        	                             Ci.calIErrors.OPERATION_CANCELLED,
			                        	             Ci.calIOperationListener.ADD,
			                        	             aItem.id,
			                        	             aItem);
					}
					return;
				}
				else {
					this.getMeetingRequestFromServer(aItem, aItem.id, Ci.calIOperationListener.ADD, aListener);
					return;

/*					// We do not have a meetingrequest in cache but it is an iTIP.
					// Is inbox polling off?
					if (!this.doPollInbox) {
						// We are not polling the inbox so treat it as a meetingrespons.
						this.logInfo("We get a new calendar item. It looks like an iTIP respons because aItem.id is set and we are not polling the inbox.");
						// We mis the exchange id and changekey which we need for responding.
						// We can get these by polling or searching the inbox but this is async. We can do this
						// because we received a listener. So search inbox for this uid and then use the full exchange message.
						// We could have multiple meetingrequest with the same uid but different id and changekey. How do we handle those??
						
						var self = this;
						this.addToQueue( erGetMeetingRequestByUIDRequest,
							{user: this.user, 
							 mailbox: this.mailbox,
							 folderBase: "inbox",
							 serverUrl: this.serverUrl,
							 item: aItem,
							 uid: aItem.id,
							 actionStart: Date.now(),
							 GUID: calExchangeCalendarGUID}, 
							function(erGetMeetingRequestByUIDRequest, aMeetingRequests) { self.getMeetingRequestByUIDOk(erGetMeetingRequestByUIDRequest, aMeetingRequests);}, 
							function(erGetMeetingRequestByUIDRequest, aCode, aMsg) { self.getMeetingRequestByUIDError(erGetMeetingRequestByUIDRequest, aCode, aMsg);},
							aListener,
							1);

						return;
					}
					else {
						this.logInfo("!! THIS IS A SITUATION WHICH NEVER SHOULD HAPPEN !!");
						this.logInfo("We have an iTIP respons and are polling the inbox but the item is not in the meetingrequestcache.");
						// We could do the same as we are not polling inbox..
						this.notifyOperationComplete(aListener,
			        	                             Ci.calIErrors.MODIFICATION_FAILED,
			                        	             Ci.calIOperationListener.ADD,
			                        	             aItem.id,
			                        	             aItem);
						return;
					} */
				}
			}
			var ewsItem = this.convertCalAppointmentToExchangeAppointment(aItem, "create", true);
		}
		if (isToDo(aItem)) {
			var ewsItem = this.convertCalTaskToExchangeTask(aItem, "create");
		}

		if (ewsItem) {
			var self = this;
			this.addToQueue( erCreateItemRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 folderBase: this.folderBase,
				 serverUrl: this.serverUrl,
				 item: aItem,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 createReq: ewsItem,
				 newItem: aItem,
				 actionStart: Date.now()}, 
				function(erCreateItemRequest, aId, aChangeKey) { self.createItemOk(erCreateItemRequest, aId, aChangeKey);}, 
				function(erCreateItemRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erCreateItemRequest, aCode, aMsg);},
				aListener,
				2);
		}
		else {
			// notify the listener
			this.notifyOperationComplete(aListener,
        	                             Ci.calIErrors.MODIFICATION_FAILED,
                        	             Ci.calIOperationListener.ADD,
                        	             aItem.id,
                        	             aItem);
		}

	},

	getMeetingRequestByUIDOk: function _getMeetingRequestByUIDOk(erGetMeetingRequestByUIDRequest, aMeetingRequests)
	{
		this.saveCredentials(erGetMeetingRequestByUIDRequest.argument);
		this.logInfo("getMeetingRequestByUIDOk: itemcount="+aMeetingRequests.length);

		if (aMeetingRequests.length > 0) {
			// Convert list to CalAppointment and try to find the matching one
			var tmpList = [];
			for (var i=0; i<aMeetingRequests.length; i++) {
				var tmpItem = this.convertExchangeAppointmentToCalAppointment(aMeetingRequests[i], true);
				if (tmpItem) {
					tmpList.push(tmpItem);
				}
			}
			var ewsItem = this.findItemInListByDatesAndID(tmpList, erGetMeetingRequestByUIDRequest.argument.item);

			//var ewsItem = this.convertExchangeAppointmentToCalAppointment(aMeetingRequests[aMeetingRequests.length-1], true);
					
			this.logInfo("getMeetingRequestByUIDOk: iTIP action item with STATUS:"+erGetMeetingRequestByUIDRequest.argument.item.getProperty("STATUS"));

			var me = this.getInvitedAttendee(erGetMeetingRequestByUIDRequest.argument.item);
			if (me) {
				this.logInfo("getMeetingRequestByUIDOk: me.participationStatus:"+me.participationStatus);
				var tmpMe = this.getInvitedAttendee(ewsItem);
				if (tmpMe) {
					this.logInfo("getMeetingRequestByUIDOk: tmpMe.participationStatus:"+tmpMe.participationStatus);
					tmpMe.participationStatus = me.participationStatus;

					switch (tmpMe.participationStatus) {
					case "ACCEPTED": 
						ewsItem.setProperty("STATUS", "CONFIRMED");
						break;
					case null:
					case "TENTATIVE": 
						ewsItem.setProperty("STATUS", "TENTATIVE");
						break;
					case "DECLINED": 
						ewsItem.setProperty("STATUS", "CANCELLED");
						break;
					}
				}
			}
			if (!me) {
				// I'm not directly invited. Just through distributionlist or mailgroup.
				this.logInfo("getMeetingRequestByUIDOk: I'm not directly invited so we are going to create a dummy attendee for now.");
				me = cal.createAttendee();
				me.id = "mailto:"+this.mailbox;
				me.commonName = this.userDisplayName;
				me.rsvp = "FALSE";
				me.userType = "INDIVIDUAL";
				me.role = "NON-PARTICIPANT";
				switch (erGetMeetingRequestByUIDRequest.argument.item.getProperty("STATUS")) {
					case "CONFIRMED":
						me.participationStatus = "ACCEPTED";
						break;
					case "TENTATIVE":
						me.participationStatus = "TENTATIVE";
						break;
					case "CANCELLED":
						me.participationStatus = "DECLINED";
						break;
				}
				
			}

			this.logInfo("getMeetingRequestByUIDOk: iTIP action item with ewsItem.STATUS:"+ewsItem.getProperty("STATUS"));

			ewsItem.setProperty("X-MEETINGREQUEST", true);
			if ((ewsItem) && (this.sendMeetingRespons(ewsItem, null, "new", me.participationStatus))) {
				if (erGetMeetingRequestByUIDRequest.listener) {
					this.notifyOperationComplete( erGetMeetingRequestByUIDRequest.listener,
		        	                             Cr.NS_OK,
		                        	             erGetMeetingRequestByUIDRequest.argument.operation,
		                        	             erGetMeetingRequestByUIDRequest.argument.item.id,
		                        	             ewsItem);
				}
				return;
			}
			// User cancel or other problem.
		}
		else {
			// inbox item has been removed between pressing iTIP button and requesting the info from the server.
			this.logInfo("Meeting request was removed from inbox after iTIP button was pressed and before Exchange server could be checked.");
			this.logInfo("Someone else working in the same inbox?");
		}

		// User cancel.. How to respond!!
		if (erGetMeetingRequestByUIDRequest.listener) {
			this.notifyOperationComplete(erGetMeetingRequestByUIDRequest.listener,
	       	                             Ci.calIErrors.OPERATION_CANCELLED, 
	                       	             erGetMeetingRequestByUIDRequest.argument.operation,
	                       	             erGetMeetingRequestByUIDRequest.argument.item.id,
	                       	             erGetMeetingRequestByUIDRequest.argument.item);
		}
	},

	getMeetingRequestByUIDError: function _getMeetingRequestByUIDError(erGetMeetingRequestByUIDRequest, aCode, aMsg)
	{
		this.saveCredentials(erGetMeetingRequestByUIDRequest.argument);
		this.logInfo("getMeetingRequestByUIDError: aCode:"+aCode+", aMsg:"+aMsg);

		this.notifyOperationComplete(erGetMeetingRequestByUIDRequest.listener,
       	                             Ci.calIErrors.OPERATION_CANCELLED,
                       	             Ci.calIOperationListener.ADD,
                       	             erGetMeetingRequestByUIDRequest.argument.item.id,
                       	             erGetMeetingRequestByUIDRequest.argument.item);

		var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
					.getService(Components.interfaces.nsIPromptService);
		promptService.alert(null, "Error", aMsg+" ("+aCode+")");
	},

	singleModified: function _singleModified(aModifiedSingle, doNotify)
	{
		try {
			this.logInfo("singleModified:"+aModifiedSingle.title+", startDate="+aModifiedSingle.startDate.toString());
		} catch(err) {}

		if (this.itemCache[aModifiedSingle.id]) {
			if (doNotify) {
				this.observers.notify("onModifyItem", [aModifiedSingle, this.itemCache[aModifiedSingle.id]]);
			}
			this.itemCache[aModifiedSingle.id] = aModifiedSingle;
		}
	},

	masterModified: function _masterModified(aModifiedMaster)
	{
		//this.logInfo("masterModified:"+aModifiedMaster.title);
		if (!this.recurringMasterCache[aModifiedMaster.getProperty("X-UID")]) {
			return null;
		}

		// Master was modified so tell the chidlren they have a new parent.
		for each(var item in this.itemCache) {
//			if ((item) && (item.parentItem.id != item.id) && (item.getProperty("X-UID") == aModifiedMaster.getProperty("X-UID"))) {
			if ((item) && (item.getProperty("X-UID") == aModifiedMaster.getProperty("X-UID"))) {
				//var newItem = item.clone();
				var newItem = this.cloneItem(item);
				newItem.parentItem = aModifiedMaster;
				this.singleModified(newItem, true);
				//this.observers.notify("onModifyItem", [newItem, item]);
				//this.itemCache[newItem.id] = newItem;
			}
		}
		
		// We send a modify for the master. But we do not show the masters in the calendar 
//miv		this.observers.notify("onModifyItem", [aModifiedMaster, this.recurringMasterCache[aModifiedMaster.getProperty("X-UID")]]);
		// Because we do not want it to be visible and the previous modify made it visible.
//miv		this.observers.notify("onDeleteItem", [aModifiedMaster]);
		this.recurringMasterCache[aModifiedMaster.getProperty("X-UID")] = aModifiedMaster;
	},

	//  calIOperation modifyItem(in calIItemBase aNewItem,
        //                   in calIItemBase aOldItem,
        //                   in calIOperationListener aListener);

	modifyItem: function _modifyItem(aNewItem, aOldItem, aListener)
	{

		this.logInfo("modifyItem");
		var result = Ci.calIErrors.MODIFICATION_FAILED;

	        if (this.readOnly) {
	            this.notifyOperationComplete(aListener,
	                                         Ci.calIErrors.CAL_IS_READONLY,
	                                         Ci.calIOperationListener.MODIFY,
	                                         null,
	                                         "Calendar is readonly");
	            return null;
	        }

	        if (!aNewItem) {
	            throw Cr.NS_ERROR_INVALID_ARG;
	        }

	        var this_ = this;
	        function reportError(errStr, errId) {
	            this_.notifyOperationComplete(aListener,
	                                          errId ? errId : Cr.NS_ERROR_FAILURE,
	                                          Ci.calIOperationListener.MODIFY,
	                                          aNewItem.id,
	                                          errStr);
	            return null;
	        }

	        if (aNewItem.id == null) {
	            // this is definitely an error
	            return reportError("ID for modifyItem item is null");
	        }

		var isMaster = false;

		if (isEvent(aNewItem)) {

			var invite = this.isInvitation(aOldItem, true);

			var doSendMeetingRespons = false;
			var meOld = this.getInvitedAttendee(aOldItem);
			if (!meOld) {
				meOld = cal.createAttendee();
				meOld.participationStatus = "NEEDS-ACTION";
			}

			var meNew = this.getInvitedAttendee(aNewItem);
			if (!meNew) {
				meNew = cal.createAttendee();
				meNew.participationStatus = "NEEDS-ACTION";
			}

			if (invite === true) {

				this.logInfo("1 meOld.participationStatus="+meOld.participationStatus+", meNew.participationStatus="+meNew.participationStatus);
				this.logInfo("1 aOldItem.status="+aOldItem.getProperty("STATUS")+", aNewItem.status="+aNewItem.getProperty("STATUS"));

				if ((meOld) && (meNew) && (meOld.participationStatus != meNew.participationStatus)) {
					doSendMeetingRespons = true;
				}

				if ((meNew) && (meNew.participationStatus == "NEEDS-ACTION") && (meOld.participationStatus != meNew.participationStatus)) {
					// They choose to confirm at a later state. Do not change this item.
			        	this.notifyOperationComplete(aListener,
			        	                             Cr.NS_OK,
			        	                             Ci.calIOperationListener.MODIFY,
			        	                             aNewItem.id,
			        	                             aNewItem);

			        	return null;
				}

				if ((meOld) && (meNew) && (aOldItem.getProperty("STATUS") != aNewItem.getProperty("STATUS")) && (!doSendMeetingRespons)) {
					switch (aNewItem.getProperty("STATUS")) {
					case "CONFIRMED": 
						meNew.participationStatus = "ACCEPTED";
						break;
					case null:
					case "TENTATIVE": 
						meNew.participationStatus = "TENTATIVE";
						break;
					case "CANCELLED": 
						meNew.participationStatus = "DECLINED";
						break;
					}
					doSendMeetingRespons = true;
				}

				/*if (!doSendMeetingRespons) {
					// Nothing changed in the status
					// User is not allowed to change a meeting of which he is not the organiser.
			        	this.notifyOperationComplete(aListener,
			        	                             Cr.NS_OK,
			        	                             Ci.calIOperationListener.MODIFY,
			        	                             aNewItem.id,
			        	                             aNewItem);

			        	return null;
				}*/

			}
		/*	else {
				if ((meOld) && (meNew) && (aOldItem.getProperty("STATUS") != aNewItem.getProperty("STATUS"))) {
					switch (aNewItem.getProperty("STATUS")) {
					case "CONFIRMED": 
						meNew.participationStatus = "ACCEPTED";
						break;
					case null:
					case "TENTATIVE": 
						meNew.participationStatus = "TENTATIVE";
						break;
					case "CANCELLED": 
						meNew.participationStatus = "DECLINED";
						break;
					}
					
					doSendMeetingRespons = true;
				}
			} */

			if (doSendMeetingRespons) {
				// The item is an invitation.
				// My status has changed. Send to EWS.
				this.logInfo("2 aOldItem.participationStatus="+meOld.participationStatus+", aNewItem.participationStatus="+(meNew ? meNew.participationStatus : ".."));
				this.logInfo("3a aOldItem.id="+aOldItem.id);
				this.logInfo("3b aNewItem.id="+aNewItem.id);

				var requestResponseItem = aNewItem;
				requestResponseItem.setProperty("X-MEETINGREQUEST", true);
				var aResponse = null;

				// Loop through meetingRequestsCache to find it.
				var cachedItem = null;
				for (var index in this.meetingRequestsCache) {
					if (this.meetingRequestsCache[index]) {
						if (this.meetingRequestsCache[index].getProperty("X-UID") == aNewItem.id) {
							cachedItem = this.meetingRequestsCache[index];
							break;
						}
					}
				}

				if (cachedItem) {
					this.logInfo("___________ Found in meeting request cache.");
					var tmpItem = cachedItem;
					var tmpUID = aNewItem.id;
					//requestResponseItem = aNewItem.clone();
					requestResponseItem = this.cloneItem(aNewItem);
					requestResponseItem.id = tmpItem.id;
					requestResponseItem.setProperty("X-UID",  tmpItem.getProperty("X-UID"));
					requestResponseItem.setProperty("X-ChangeKey",  tmpItem.getProperty("X-ChangeKey"));
					//this.removeFromMeetingRequestCache(tmpUID);
				}
				else {
					this.logInfo("___________ NOT Found in meeting request cache.");

					if (!this.itemCache[aNewItem.id]) {
						this.getMeetingRequestFromServer(aNewItem, aOldItem.getProperty("X-UID"), Ci.calIOperationListener.MODIFY, aListener);
						return;
					}

				/*	if (!this.itemCache[aNewItem.id]) {
						var inCalendar = null;
					
						for each (var item in this.itemCache) {
							if ((item) && (item.getProperty("X-UID") == aNewItem.id)) {
								inCalendar = item;
								break;
							}
						}
						if (inCalendar) {
							this.logInfo("++++++++++++ Found in Calendar.");
							requestResponseItem = inCalendar;
							aResponse = (meNew ? meNew.participationStatus : "");
						}
					}
					else {
						this.logInfo("++++++++++++ Found in itemCache.");
						requestResponseItem = this.itemCache[aNewItem.id];
						aResponse = (meNew ? meNew.participationStatus : "");
					}*/
				}

				if (this.sendMeetingRespons(requestResponseItem, null, "exisiting", aResponse)) {
					return;
					//result = Cr.NS_OK;
				}
				else {
					this.logInfo("modifyItem: canceled by user.");
					result = Cr.NS_OK;
				}
			}
			else {

				var input= { item: aNewItem, 
					     response: "sendtonone"};

				if (aNewItem.organizer) {
					this.logInfo("The organizer is:"+aNewItem.organizer.id);
				}
				else {
					this.logInfo("We have no organizer!");
				}

				var iAmOrganizer = ((aNewItem.organizer) && (aNewItem.organizer.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase()));
				if (iAmOrganizer) {

					input.response = "sendtoall";
	

					// Get the eventsummarywindow to attach dialog to.
					let wm = Cc["@mozilla.org/appshell/window-mediator;1"]
				                          .getService(Ci.nsIWindowMediator);
					let calWindow = wm.getMostRecentWindow("Calendar:EventSummaryDialog") || cal.getCalendarWindow() || wm.getMostRecentWindow("mail:3pane") ;
	
					var attendees = aNewItem.getAttendees({}).length + aOldItem.getAttendees({}).length;
					this.logInfo("  -- aOldItem.getAttendees({}).length="+aOldItem.getAttendees({}).length);
					this.logInfo("  -- aNewItem.getAttendees({}).length="+aNewItem.getAttendees({}).length);
					if (this.getInvitedAttendee(aOldItem)) {
						attendees--;
					}
					if (this.getInvitedAttendee(aNewItem)) {
						attendees--;
					}

					if ((calWindow) && (attendees > 0)) {
						calWindow.openDialog("chrome://exchangecalendar/content/sendUpdateTo.xul",
							"sendUpdateTo",
							"chrome,titlebar,toolbar,centerscreen,dialog,modal=yes,resizable=yes",
							input); 
					}

					if (attendees == 0) {
						this.logInfo(" -- There are no attendees left");
						input.response = "sendtonone";
					}
				}

				this.logInfo("modifyItem: it is a event. aOldItem.CalendarItemType=:"+aOldItem.getProperty("X-CalendarItemType"));

				if (aOldItem.parentItem.id == aOldItem.id) {
					// We have a Single or master
					var master = this.recurringMasterCache[aOldItem.getProperty("X-UID")];
					if (master) {
						isMaster = true;
						// See if the aNewItem is also the master record.
						var masterChanged = (aNewItem.parentItem.id == aNewItem.id);

				this.logInfo(" == aNewItem.id:"+aNewItem.id);
				this.logInfo(" == aNewItem.parentItem.id:"+aNewItem.parentItem.id);
				this.logInfo(" == aOldItem.id:"+aOldItem.id);
				this.logInfo(" == aOldItem.parentItem.id:"+aOldItem.parentItem.id);
	
						// We need to find out wat has changed;
						var changes = this.makeUpdateOneItem(aNewItem, aOldItem);

						if (changes) {
							this.logInfo("modifyItem: changed:"+String(changes));

							// We remove the item from cache and calendar because the update request will add
							// it again.
//							this.observers.notify("onDeleteItem", [aOldItem]);
						//x	this.deleteAlarms(this.itemCache[aOldItem.id]);

							this.removeChildrenFromMaster(this.recurringMasterCache[aOldItem.getProperty("X-UID")]);
							delete this.itemCache[aOldItem.id];
							delete this.recurringMasterCache[aOldItem.getProperty("X-UID")];

							this.logInfo(" When CHANGED master arrives for '"+aNewItem.title+"' then it's children we all be downloaded.");
							this.newMasters[aOldItem.getProperty("X-UID")] = true;

							var self = this;
							this.addToQueue( erUpdateItemRequest,
								{user: this.user, 
								 mailbox: this.mailbox,
								 folderBase: this.folderBase,
								 serverUrl: this.serverUrl,
								 item: aOldItem,
								 folderID: this.folderID,
								 changeKey: this.changeKey,
								 updateReq: changes,
								 newItem: aNewItem,
						 		 actionStart: Date.now(),
								 sendto: input.response}, 
								function(erUpdateItemRequest, aId, aChangeKey) { self.updateItemOk(erUpdateItemRequest, aId, aChangeKey);}, 
								function(erUpdateItemRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erUpdateItemRequest, aCode, aMsg);},
								aListener,
								2);
							//this.masterModified(aNewItem);
							return;
						}
						else {
							this.logInfo("modifyItem: No changes for master.");
							// No changes to a master could means that one of the occurrences
							// was deleted. 
							var removedOccurrence = this.getRemovedOccurrence(aOldItem, aNewItem);
							if (removedOccurrence) {
								// Delete this occurrence;
								var self = this;
								this.addToQueue( erGetOccurrenceIndexRequest,
									{user: this.user, 
									 mailbox: this.mailbox,
									 folderBase: this.folderBase,
									 serverUrl: this.serverUrl,
									 masterItem: removedOccurrence,
									 item: removedOccurrence,
									 folderID: this.folderID,
									 changeKey: this.changeKey,
									 action: "deleteItem",
									 itemType: "occurrence",
									 whichOccurrence: "single_occurence" },//dialogArg.answer}, 
									function(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey) { self.getOccurrenceIndexOk(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey);}, 
									function(erGetOccurrenceIndexRequest, aCode, aMsg) { self.getOccurrenceIndexError(erGetOccurrenceIndexRequest, aCode, aMsg);},
									null,
									2);
							}
							else {
								// Could be an alarm dismiss or snooze
							}
							this.masterModified(aNewItem);
							result = Cr.NS_OK;
						}
					}
					else {
						this.logInfo("modifyItem: Single event modification");
						// We need to find out wat has changed;
						this.logInfo(" ==1 invite="+invite);
						var changes = this.makeUpdateOneItem(aNewItem, aOldItem);
						if (changes) {
							this.logInfo("modifyItem: changed:"+String(changes));

							// We remove the item from cache and calendar because the update request will add
							// it again.
							// This is not true when we changed a single to a recurring item.
							if ((!aOldItem.recurrenceInfo) && (aNewItem.recurrenceInfo)) {
								// We need to request the children when the new master arrives.
								this.logInfo(" When master arrives for '"+aNewItem.title+"' then it's children we all be downloaded.");
								this.newMasters[aOldItem.getProperty("X-UID")] = true;
							}

							this.observers.notify("onDeleteItem", [aOldItem]);
							delete this.itemCache[aOldItem.id];

							var self = this;
							this.addToQueue( erUpdateItemRequest,
								{user: this.user, 
								 mailbox: this.mailbox,
								 folderBase: this.folderBase,
								 serverUrl: this.serverUrl,
								 item: aOldItem,
								 folderID: this.folderID,
								 changeKey: this.changeKey,
								 updateReq: changes,
								 newItem: aNewItem,
						 		 actionStart: Date.now(),
								 sendto: input.response}, 
								function(erUpdateItemRequest, aId, aChangeKey) { self.updateItemOk(erUpdateItemRequest, aId, aChangeKey);}, 
								function(erUpdateItemRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erUpdateItemRequest, aCode, aMsg);},
								aListener,
								2);
							//this.singleModified(aNewItem);
							return;
						}
						else {
							this.logInfo("modifyItem: No changes 1.");
							if (!invite) {
								aNewItem.parentItem = aNewItem;
								this.singleModified(aNewItem, true);
							}
							result = Cr.NS_OK;
						}
					}
				}
				else {
					this.logInfo("modifyItem: Occurrence or exception modification");
					
					// We need to find the Index of this item.
					var self = this;
					this.addToQueue( erGetOccurrenceIndexRequest,
						{user: this.user, 
						 mailbox: this.mailbox,
						 folderBase: this.folderBase,
						 serverUrl: this.serverUrl,
						 masterItem: aOldItem,
						 item: aOldItem,
						 folderID: this.folderID,
						 changeKey: this.changeKey,
						 newItem: aNewItem,
						 actionStart: Date.now(),
						 sendto: input.response}, 
						function(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey) { self.modifyItemgetOccurrenceIndexOk(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey);}, 
						function(erGetOccurrenceIndexRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erGetOccurrenceIndexRequest, aCode, aMsg);},
						aListener,
						2);
					this.singleModified(aNewItem);
					return;
				}
			}
		}
		else {
			if (isToDo(aNewItem)) {
				this.logInfo("modifyItem: it is a todo");
				var changes = this.makeUpdateOneItem(aNewItem, aOldItem);
				if (changes) {
					this.logInfo("modifyItem: changed:"+String(changes));
					var self = this;
					this.addToQueue( erUpdateItemRequest,
						{user: this.user, 
						 mailbox: this.mailbox,
						 folderBase: this.folderBase,
						 serverUrl: this.serverUrl,
						 item: aOldItem,
						 folderID: this.folderID,
						 changeKey: this.changeKey,
						 updateReq: changes,
						 newItem: aNewItem,
					 	 actionStart: Date.now()}, 
						function(erUpdateItemRequest, aId, aChangeKey) { self.updateItemOk(erUpdateItemRequest, aId, aChangeKey);}, 
						function(erUpdateItemRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erUpdateItemRequest, aCode, aMsg);},
						aListener,
						2);
					this.singleModified(aNewItem, true);
					return;
				}
				else {
					this.logInfo("modifyItem: No changes 2.");
					aNewItem.parentItem = aNewItem;
					this.singleModified(aNewItem, true);
					result = Cr.NS_OK;
				}
			}
			else {
				return reportError("Unknown itemtype for modifyItem");
			}
		}

		//this.observers.notify("onModifyItem", [aNewItem, aOldItem]);

        	this.notifyOperationComplete(aListener,
        	                             result,
        	                             Ci.calIOperationListener.MODIFY,
        	                             aNewItem.id,
        	                             aNewItem);

        	return null;
	},

	//  calIOperation deleteItem(in calIItemBase aItem,
        //                   in calIOperationListener aListener);
	deleteItem: function _deleteItem(aItem, aListener) {
		this.logInfo("deleteItem");

		if (aItem.id == null) {
			this.notifyOperationComplete(aListener,
                                         Ci.calIErrors.MODIFICATION_FAILED,
                                         Ci.calIOperationListener.DELETE,
                                         null,
                                         "ID is null for deleteItem");
			return;
		}

		// Check if this item is still in cache
		if ((aItem.id == aItem.parentItem.id) && (!this.itemCache[aItem.id]) && (!this.recurringMasterCache[aItem.getProperty("X-UID")]))	 {
			this.logInfo("Item is not in itemCache anymore. Probably not removed from view by Lightning..");
			this.notifyOperationComplete(aListener,
                                         Cr.NS_OK,
                                         Ci.calIOperationListener.DELETE,
                                         aItem.id,
                                         aItem);
			return;
		}

		var self = this;
		if (isEvent(aItem)) {
			this.logInfo("deleteItem is calIEvent");

			var iAmOrganizer = ((aItem.organizer) && (aItem.organizer.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase()));
			var isCancelled = aItem.getProperty("X-IsCancelled");
			var isInvitation = this.isInvitation(aItem, true);

			if ((isInvitation) && (!isCancelled)) {
//			if ((this.folderBase == "calendar") && (!this.folderID) && 
//			    (!iAmOrganizer) && (aItem.getProperty("STATUS") != "DECLINED") &&
//			    (aItem.getProperty("STATUS") != "NONE") && (!isCancelled)) {
				//var aOldItem = aItem.clone();
				var aOldItem = this.cloneItem(aItem);
				aOldItem.setProperty("STATUS", "DECLINED");

				if (!this.sendMeetingRespons(aOldItem, null, "exisiting", "DECLINED")) {
					this.logInfo("deleteItem: canceled by user.");
					this.notifyOperationComplete(erDeleteItemRequest.listener,
					      Ci.calIErrors.OPERATION_CANCELLED,
					      Ci.calIOperationListener.DELETE,
					      aItem.id,
					      aItem);
					return;
				}
				else {
					this.notifyOperationComplete(erDeleteItemRequest.listener,
					      Cr.NS_OK,
					      Ci.calIOperationListener.DELETE,
					      aItem.id,
					      aItem);
					return;
				}

			}

			var input= { item: aItem, 
				     response: "sendtonone"};

			if (iAmOrganizer) {

				input.response = "sendtoall";

				// Get the eventsummarywindow to attach dialog to.
				let wm = Cc["@mozilla.org/appshell/window-mediator;1"]
			                          .getService(Ci.nsIWindowMediator);
				let calWindow = wm.getMostRecentWindow("Calendar:EventSummaryDialog") || cal.getCalendarWindow() || wm.getMostRecentWindow("mail:3pane") ;

				var attendees = aItem.getAttendees({}).length;
				var meOld = this.getInvitedAttendee(aItem);

				if (meOld) {
					attendees--;
				}

				if ((calWindow) && (attendees > 0)) {
					calWindow.openDialog("chrome://exchangecalendar/content/sendUpdateTo.xul",
						"sendUpdateTo",
						"chrome,titlebar,toolbar,centerscreen,dialog,modal=yes,resizable=yes",
						input); 
				}

				if (attendees == 0) {
					input.response = "sendtonone";
				}

			}


			switch (aItem.getProperty("X-CalendarItemType")) {

				case "Single" :
					this.logInfo("-- Single CalendarItemType");

					this.addToQueue( erDeleteItemRequest, 
						{user: this.user, 
						 mailbox: this.mailbox,
						 folderBase: this.folderBase,
						 serverUrl: this.serverUrl,
						 item: aItem,
						 folderID: this.folderID,
						 changeKey: this.changeKey,
						 itemType: "single"}, 
						function(erGetOccurrenceIndexRequest) { self.deleteItemOk(erGetOccurrenceIndexRequest);}, 
						function(erGetOccurrenceIndexRequest, aCode, aMsg) { self.deleteItemError(erGetOccurrenceIndexRequest, aCode, aMsg);},
						aListener,
						2);

					break;
				case "Occurrence" :
				case "Exception" :
					this.logInfo("-- "+aItem.getProperty("X-CalendarItemType")+" CalendarItemType");

					this.addToQueue( erGetOccurrenceIndexRequest,
						{user: this.user, 
						 mailbox: this.mailbox,
						 folderBase: this.folderBase,
						 serverUrl: this.serverUrl,
						 masterItem: aItem,
						 item: aItem,
						 folderID: this.folderID,
						 changeKey: this.changeKey,
						 action: "deleteItem",
						 itemType: "occurrence",
						 whichOccurrence: "occurrence" },//dialogArg.answer}, 
						function(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey) { self.getOccurrenceIndexOk(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey);}, 
						function(erGetOccurrenceIndexRequest, aCode, aMsg) { self.getOccurrenceIndexError(erGetOccurrenceIndexRequest, aCode, aMsg);},
						aListener,
						2);

					break;
				case "RecurringMaster" :
					this.logInfo("-- RecurringMaster CalendarItemType");

					this.addToQueue( erDeleteItemRequest,
						{user: this.user, 
						 mailbox: this.mailbox,
						 folderBase: this.folderBase,
						 serverUrl: this.serverUrl,
						 item: aItem,
						 folderID: this.folderID,
						 changeKey: this.changeKey,
						 itemType: "master",
						 whichOccurrence: "all_occurrences"}, 
						function(erDeleteItemRequest) { self.deleteItemOk(erDeleteItemRequest);}, 
						function(erDeleteItemRequest, aCode, aMsg) { self.deleteItemError(erDeleteItemRequest, aCode, aMsg);},
						aListener,
						2);
					break;
				default :
					// TODO: This will happen when the sync to/from EWS has not yet happened.
					this.logInfo("WARNING: unknown CalendarItemType="+aItem.getProperty("X-CalendarItemType"));
			}
		}

		if (isToDo(aItem)) {
			this.logInfo("deleteItem is calITask");
			this.addToQueue( erDeleteItemRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 folderBase: this.folderBase,
				 serverUrl: this.serverUrl,
				 item: aItem,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 itemType: "single"}, 
				function(erDeleteItemRequest) { self.deleteItemOk(erDeleteItemRequest);}, 
				function(erDeleteItemRequest, aCode, aMsg) { self.deleteItemError(erDeleteItemRequest, aCode, aMsg);},
				aListener,
				2);
		}


	},

	//  calIOperation getItem(in string aId, in calIOperationListener aListener);
	getItem: function _getItem(aId, aListener, aRetry) {
		this.logInfo("getItem: aId:"+aId);

		if (!aListener)
			return;

		var item = null;

	/*	for (var index in this.meetingRequestsCache) {
			if (this.meetingRequestsCache[index]) {
				if (this.meetingRequestsCache[index].getProperty("X-UID") == aId) {
					this.logInfo("getItem is in meetingRequestsCache");
					this.logInfo("  id="+index);
					var item = this.meetingRequestsCache[index];
					break;
				}
			}
		}  */

		if (!item) {
			for (var index in this.itemCache) {
				if (this.itemCache[index]) {
					if (this.itemCache[index].getProperty("X-UID") == aId) {
						var item = this.itemCache[index];
						break;
					}
				}
			}
		}

		if (!item) {
			var cachedRequest = null;
      			for (var index in this.meetingCancelationsCache) {
				if (this.meetingCancelationsCache[index]) {
					if (this.meetingCancelationsCache[index].getProperty("X-UID") == aId) {
						cachedRequest = this.meetingCancelationsCache[index];
						break;
					}
				}
			}

			if (cachedRequest) {
				//We have a meeting cancelation but not any calendaritem.
				this.logInfo("This is odd we have a meeting cancelation but not any calendaritem. THIS SHOULD NOT HAPPEN");
				delete this.meetingCancelationsCache[cachedRequest.id];
				return;
			}

		}

		if (!item) {
			// Wait with an answer until next sync. This could be an iTIP request
			// we could have received am email with a meetingrequest which was not
			// yet received through sync. Therefore we do not find it in cache. And
			// iTIP will show accept buttons and stuff. We do not want those buttons because
			// when pressed they will try to add the item to the calendar which gets synced to
			// EWS and we get an conflict...(item allready exists in EWS).
			// After next sync we can do this check again.
			if (!aRetry) {
				this.logInfo("Not found putting it in ItemSyncQue");
				this.getItemSyncQueue.push( { id: aId,
							      listener: aListener } );
				this.refresh();
				return;
			}
			else {
				this.logInfo("getItem not FOUND");
				// querying by id is a valid use case, even if no item is returned:
				this.notifyOperationComplete(aListener,
	                                         Cr.NS_OK,
	                                         Ci.calIOperationListener.GET,
	                                         aId,
	                                         null);
				return;
			}
		}

		this.logInfo("Status:"+item.getProperty("STATUS"));

		var item_iid = null;
		if (isEvent(item))
			item_iid = Ci.calIEvent;
		else if (isToDo(item))
			item_iid = Ci.calITodo;
		else {
			this.notifyOperationComplete(aListener,
                                         Cr.NS_ERROR_FAILURE,
                                         Ci.calIOperationListener.GET,
                                         aId,
                                         "Can't deduce item type based on QI");
			return;
		}

		aListener.onGetResult (this, 
                               Cr.NS_OK,
                               item_iid, null,
                               1, [item]);

		this.notifyOperationComplete(aListener,
                                     Cr.NS_OK,
                                     Ci.calIOperationListener.GET,
                                     aId,
                                     null);
	},

	//  calIOperation getItems(in unsigned long aItemFilter,
        //                 in unsigned long aCount,
        //                 in calIDateTime aRangeStart,
        //                 in calIDateTime aRangeEndEx,
        //                 in calIOperationListener aListener);
	getItems: function _getItems(aItemFilter, aCount,
                                    aRangeStart, aRangeEnd, aListener) 
	{
		//this.logInfo("getItems 1");
		if (!this.isInitialized) {
			return;
		}

		var wantEvents = ((aItemFilter & Ci.calICalendar
			.ITEM_FILTER_TYPE_EVENT) != 0);

		var wantTodos = ((aItemFilter & Ci.calICalendar
			.ITEM_FILTER_TYPE_TODO) != 0);

		var asOccurrences = ((aItemFilter & Ci.calICalendar
			.ITEM_FILTER_CLASS_OCCURRENCES) != 0);

		var wantInvitations =  ((aItemFilter & Ci.calICalendar
	  		.ITEM_FILTER_REQUEST_NEEDS_ACTION) != 0);

		if (!wantEvents && !wantInvitations && !wantTodos) {
			this.notifyOperationComplete(aListener,
				Cr.NS_OK,
				Ci.calIOperationListener.GET,
				null, null);
			return;
		}

		// Try to make both start and end date be valid
		// No more than two years apart because exchange does not want this.
		if ((!aRangeStart) && (aRangeEnd)) {
			aRangeStart = aRangeEnd.clone();
			var tmpDur = cal.createDuration();
			tmpDur.hours = -24;
			aRangeStart.addDuration(tmpDur);
			//aRangeStart.year = aRangeStart.year - 1;
		}

		if ((aRangeStart) && (!aRangeEnd)) {
			aRangeEnd = aRangeStart.clone();
			var tmpDur = cal.createDuration();
			tmpDur.hours = 24;
			aRangeEnd.addDuration(tmpDur);
			//aRangeEnd.year = aRangeEnd.year + 1;
		}

		if (!aRangeStart) {
			// Set rangestart to today.
			aRangeStart = now();
		}

		if (!aRangeEnd) {
			// Set rangeend to today.
			aRangeEnd = now();
		}

		if (aRangeStart.year == 1970) {
			aRangeStart = aRangeEnd.clone();
			var tmpDur = cal.createDuration();
			tmpDur.hours = -24;
			aRangeStart.addDuration(tmpDur);
		}

		var dateChanged = false;

		if (!this.startDate) {
			this.startDate = aRangeStart.clone();
			dateChanged = true;
		}
		else {
			if (this.startDate.compare(aRangeStart) > 0) {
				this.startDate = aRangeStart.clone();
				dateChanged = true;
			}
		}

		if (!this.endDate) {
			this.endDate = aRangeEnd.clone();
			dateChanged = true;
		}
		else {
			if (this.endDate.compare(aRangeEnd) < 0) {
				this.endDate = aRangeEnd.clone();
				dateChanged = true;
			}
		}

		if (!dateChanged) {

			var events = [];
			var tasks = [];
			for (var index in this.itemCache) {
				if (isEvent(this.itemCache[index])) {
					if ( (this.itemCache[index].startDate.compare(aRangeEnd) < 1) &&
					     (this.itemCache[index].endDate.compare(aRangeStart) > -1) ) {
						events.push(this.itemCache[index]);
					} 
				}
				else {
					if (isToDo(this.itemCache[index])) {
						tasks.push(this.itemCache[index]);
					}
				}
			}

			if ((events.length > 0) && (wantEvents)) {
				aListener.onGetResult(this,
					     Cr.NS_OK,
					     Ci.calIEvent,
					     null,
					     events.length,
					     events);
			}
			if ((tasks.length > 0) && (wantTodos)) {
				aListener.onGetResult(this,
					     Cr.NS_OK,
					     Ci.calITodo,
					     null,
					     tasks.length,
					     tasks);
			}

			this.notifyOperationComplete(aListener,
				Cr.NS_OK,
				Ci.calIOperationListener.GET,
				null, null);

			return;
		}


       		var self = this;

		if ((!this.syncInboxState) && (!this.weAreInboxSyncing)) {
			if ((this.folderBase == "calendar") && (!this.folderID)) {

				// Start the inbox poller to check for meetinginvitations or cancelations.
				this.checkInbox();
			}
		}

		if (wantEvents) {
			this.addToQueue( erFindCalendarItemsRequest, 
				{user: this.user, 
				 mailbox: this.mailbox,
				 serverUrl: this.serverUrl,
				 count: aCount,
				 rangeStart: aRangeStart,
				 rangeEnd: aRangeEnd,
				 folderBase: this.folderBase,
				 itemFilter: aItemFilter,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 actionStart: Date.now() }, 
				function(erFindCalendarItemsRequest, aIds, aOccurrences) { self.findCalendarItemsOK(erFindCalendarItemsRequest, aIds, aOccurrences);}, 
				function(erFindCalendarItemsRequest, aCode, aMsg) { self.findCalendarItemsError(erFindCalendarItemsRequest, aCode, aMsg);},
				null);
		}
	
		if (wantTodos) {
			this.addToQueue( erFindTaskItemsRequest, 
				{user: this.user, 
				 mailbox: this.mailbox,
				 serverUrl: this.serverUrl,
				 folderBase: this.folderBase,
				 itemFilter: aItemFilter,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 actionStart: Date.now() }, 
				function(erFindTaskItemsRequest, aIds) { self.findTaskItemsOK(erFindTaskItemsRequest, aIds);}, 
				function(erFindTaskItemsRequest, aCode, aMsg) { self.findTaskItemsError(erFindTaskItemsRequest, aCode, aMsg);},
				null);
		}


		if (aListener) {
			this.notifyOperationComplete(aListener,
			      Cr.NS_OK,
			      Ci.calIOperationListener.GET,
			      null,
			      null);
		}
	    },


	//  calIOperation refresh();
	refresh: function _refresh() {

		if (this.shutdown) {
			return;
		}

		if ((!this.syncState) || (this.weAreSyncing)) {
			return;
		}

		if (!this.isInitialized) {	
			return;
		}

		//this.logInfo("Refresh. We start a sync.");
		var self = this;

		this.getSyncState();

		return;

	},

	//  void startBatch();

	//  void endBatch();
// End calICalendar

// Begin calISchedulingSupport
	//  boolean isInvitation(in calIItemBase aItem);
	isInvitation: function _isInvitation(aItem, ignoreStatus)
	{

		if (ignoreStatus == undefined) {
			ignoreStatus = true;
		}

		if (!aItem) {
			return false;
		}

		if ((aItem.getProperty("X-IsInvitation")) && (aItem.getProperty("X-IsInvitation") == "true")) {
			if ((!ignoreStatus) && (aItem.getProperty("STATUS") != "NONE")) {
				return false;
			}
			return true;
		}

		// If this is not an personal calendar
		if ((this.folderBase != "calendar") || (this.folderPath != "/")) {
			return false;
		}

		// Check if we have attendees
		var attendees = aItem.getAttendees({});
		if (!attendees) {
			return false;
		}

		if (attendees.length == 0) {
			return false;
		}

		if (aItem.getProperty("X-IsCancelled")) {
			return false;
		}

		if ((!aItem.getProperty("X-IsMeeting")) || (aItem.getProperty("X-IsMeeting") === false)) {
			return false;
		}

		// When I'm not the organizer I'm invited if i am an attendee.
		let org = aItem.organizer;
		if ((!org) || (org.id.replace(/^mailto:/, '').toLowerCase() != this.mailbox.toLowerCase())) {
			
			// Check if I am an attendee
			var attendees = aItem.getAttendees({});
			for each (var attendee in attendees) {
				if (attendee.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase()) {
				this.logInfo("isInvitation FOUND myself");

					if (aItem.getProperty("STATUS") != "NONE") {
						return false;
					}
					return true;
				}
			}
			// I'm not directly invited. Could be through mailgroup.
			// TODO: How do we react. 
		}
/*		else {
			if ((org) && (org.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase())) {
				this.logInfo("I'm the organiser");
			}
		}*/

		return false;
	},

	// boolean canNotify(in AUTF8String aMethod, in calIItemBase aItem);
	canNotify: function _canNotify(aMethod, aItem)
	{
		this.logInfo("canNotify: aMethod="+aMethod+":"+aItem.title);

		return true;
	},

	// calIAttendee getInvitedAttendee(in calIItemBase aItem);
	getInvitedAttendee: function _getInvitedAttendee(aItem)
	{
		//this.logInfo("getInvitedAttendee 1:"+aItem.title);
		if (!aItem) {
			return;
		}

		// Parse through the attendees
		var attendees = aItem.getAttendees({});
		for each (var attendee in attendees) {
			//this.logInfo("getInvitedAttendee 2:"+attendee.id);
			if (attendee.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase()) {
				this.logInfo("getInvitedAttendee FOUND myself:"+aItem.title);
				return attendee; //.clone();
			}
		}

		if (aItem.getProperty("X-IsInvitation") == "true") {
			var tmpAttendee = cal.createAttendee();
			tmpAttendee.id = "mailto:"+this.mailbox;
			tmpAttendee.commonName = this.userDisplayName;
			tmpAttendee.rsvp = "FALSE";
			tmpAttendee.userType = "INDIVIDUAL";
			tmpAttendee.role = "REQ-PARTICIPANT";
			tmpAttendee.participationStatus = "NEEDS-ACTION";
			return tmpAttendee;
		}

	},
// End calISchedulingSupport

// Begin calIFreeBusyProvider
	//    calIOperation getFreeBusyIntervals(in AUTF8String aCalId,
        //                               in calIDateTime aRangeStart,
        //                               in calIDateTime aRangeEnd,
        //                               in unsigned long aBusyTypes,
        //                               in calIGenericOperationListener aListener);

	getFreeBusyIntervals: function(aCalId, aRangeStart, aRangeEnd,
				       aBusyTypes, aListener)
	{
//		this.logInfo("getFreeBusyIntervals: " + aCalId + ", aBusyTypes:"+aBusyTypes);

		if (aCalId.indexOf("@") < 0 || aCalId.indexOf(".") < 0) {
            		// No valid email, screw it
			aListener.onResult(null, null);
			return;
		}

		var self = this;
		this.addToQueue( erGetUserAvailabilityRequest, 
			{user: this.user, 
			 mailbox: this.mailbox,
			 folderBase: this.folderBase,
			 serverUrl: this.serverUrl,
			 email: aCalId.replace(/^MAILTO:/, ""),
			 attendeeType: 'Required',
			 start: cal.toRFC3339(aRangeStart.getInTimezone(ews.ecUTC())),
			 end: cal.toRFC3339(aRangeEnd.getInTimezone(ews.ecUTC())),
			 calId: aCalId,
			 folderID: this.folderID,
			 changeKey: this.changeKey }, 
			function(erGetUserAvailabilityRequest, aEvents) { self.getUserAvailabilityRequestOK(erGetUserAvailabilityRequest, aEvents);}, 
			function(erGetUserAvailabilityRequest, aCode, aMsg) { self.getUserAvailabilityRequestError(erGetUserAvailabilityRequest, aCode, aMsg);},
			aListener);

	},
// End calIFreeBusyProvider

// Begin calIItipTransport
	//    readonly attribute AUTF8String scheme;
	get scheme()
	{
		this.logInfo("get scheme");
		return "";
	},

	//    attribute AUTF8String senderAddress;
	get senderAddress()
	{
		this.logInfo("get senderAddress");
		return "hihi";
	},

	set senderAddress(aValue)
	{
		this.logInfo("set senderAddress("+aValue+")");
	},

	//    readonly attribute AUTF8String type;
	//* IS DEFINED EARLIER AS PART OF calICalendar

    /**
     * Sends a calIItipItem to the recipients using the specified title and
     * alternative representation. If a calIItipItem is attached, then an ICS
     * representation of those objects are generated and attached to the email.
     * If the calIItipItem is null, then the item(s) is sent without any
     * text/calendar mime part.
     * @param count             size of recipient array
     * @param recipientArray    array of recipients
     * @param calIItipItem      set of calIItems encapsulated as calIItipItems
     */
	//    void sendItems(in PRUint32 count,
	//                   [array, size_is(count)] in calIAttendee recipientArray,
	//                   in calIItipItem item);
	sendItems: function _sendItems(count, recipientArray, item)
	{
		this.logInfo("sendItems");
	},
// End calIItipTransport

	isRemoved: function _isRemoved(aItem)
	{
		if (!aItem) {
			return null;
		}

		this.logInfo("isRemoved title:"+aItem.title+", status="+aItem.status);

		if (aItem.status == "") {
			return true;
		}

		return null;false;
	},

	getExceptions: function _getExceptions(aRecurrenceItems)
	{
		var tmpCount = 0;
		var exceptions = {};
		for each(var recurrenceItem in aRecurrenceItems) {
			tmpCount++;
			this.logInfo("getExceptions: nr:"+tmpCount+", isNegative:"+recurrenceItem.isNegative);
			if (recurrenceItem.isNegative) {
				// A deletion is an exception and therefore isNegative === true
				var occurrences = recurrenceItem.getOccurrences(this.startDate, this.startDate, this.endDate, 0,  {});
				this.logInfo("getExceptions: we have occurrences.length="+occurrences.length);
				for each(var occurrence in occurrences) {
					exceptions[occurrence.toString()] = occurrence;
				}				
			}
		}
		return exceptions;
	},

	getRemovedOccurrence: function _getRemovedOccurrence(aOldItem, aNewItem)
	{
		this.logInfo("getRemovedOccurrence");
		// When an occurences gets removed from we get an extra occurenceitem in the recurrenceinfo list.

		var newRecurrenceItems;
		newRecurrenceItems = aNewItem.recurrenceInfo.getRecurrenceItems({});

		var oldRecurrenceItems;
		oldRecurrenceItems = aOldItem.recurrenceInfo.getRecurrenceItems({});

		if ((!newRecurrenceItems) || (newRecurrenceItems.length == 0)) {
			// No recurrenceItems in the new item
			// Nothing can be checked if it is removed.
			if (newRecurrenceItems) {
				this.logInfo("newRecurrenceItems.length="+newRecurrenceItems.length);
			}
			this.logInfo("getRemovedOccurrence: newItem has occurrenceInfo but no recurrenceItems. newCount="+newRecurrenceItems.length+",oldCount="+oldRecurrenceItems.length);
			return null;
		}

		if (newRecurrenceItems.length > oldRecurrenceItems.length) {
			this.logInfo("getRemovedOccurrence: a New occurrence. newCount="+newRecurrenceItems.length+",  oldCount="+oldRecurrenceItems.length);
		}

		var newException = aNewItem.recurrenceInfo.getExceptionIds({});
		var oldException = aOldItem.recurrenceInfo.getExceptionIds({});
		if (newException.length != oldException.length) {
			this.logInfo("getRemovedOccurrence: Exceptions count changed. newCount="+newException.length+",  oldCount="+oldException.length);
		}
		
		var oldExceptions = this.getExceptions(oldRecurrenceItems);

		var newExceptions = this.getExceptions(newRecurrenceItems);

		// Check if the newExceptions allready exists in the oldExceptions. If so remove it.
		for (var exceptionStr in newExceptions) {
			if (oldExceptions[exceptionStr]) {
				this.logInfo("getRemovedOccurrence: '"+exceptionStr+"' also exists in the old occurrence list.");
				delete newExceptions[exceptionStr];
			}
		}

		// What we are left with is the removedOccurrence(s). It should be only one.
		for each(var exception in newExceptions) {
			if (exception) {
				// Go find the item in the cache
				for each(var item in this.itemCache) {
					if ((item) &&
					    (item.getProperty("X-UID") == aNewItem.getProperty("X-UID")) && 
					    (item.parentItem != item) &&
					    (item.recurrenceId.compare(exception) == 0)) {
						this.logInfo("getRemovedOccurrence: we found our removed occurrence");
						return item;
					}
				}
			}
		}

		this.logInfo("getRemovedOccurrence: we DID NOT FIND our removed occurrence");
		return null;
	},

	get isInitialized()
	{
		if (!this.id) {
			return false;
		}

		

		if (this.firstrun) {

			this.prefs = Cc["@mozilla.org/preferences-service;1"]
		                    .getService(Ci.nsIPrefService)
				    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl."+this.id+".");
			if (!this.prefs) {
				LOGInfo("ERROR: No access to our preferences");
				return false;
			}

		}

		var returnVal = false;
		try {
			this.prefs.getCharPref("ecServer");
			returnVal = true;
		}
		catch(err) {
			returnVal = false;
		}
		if (this.firstrun) {
			this.firstrun = false;

			// Convert oldstyle ExchangeCalendar prefs to newstyle. (since version 0.7.26)
			var oldPrefs = Cc["@mozilla.org/preferences-service;1"]
					.getService(Ci.nsIPrefService)
					.getBranch("calendar.registry."+this.id+".");
			try {
				oldPrefs.getCharPref("ecServer");
				// If we get here old prefs exists. We need to move them.
				this.logInfo("Performing upgrade of userpreferences.");
				this.prefs.setCharPref("ecServer", oldPrefs.getCharPref("ecServer"));
				oldPrefs.clearUserPref("ecServer");
				this.prefs.setCharPref("ecMailbox", oldPrefs.getCharPref("ecMailbox"));
				oldPrefs.clearUserPref("ecMailbox");
				this.prefs.setCharPref("ecDisplayname", oldPrefs.getCharPref("ecDisplayname"));
				oldPrefs.clearUserPref("ecDisplayname");
				this.prefs.setCharPref("ecUser", oldPrefs.getCharPref("ecUser"));
				oldPrefs.clearUserPref("ecUser");
				this.prefs.setCharPref("ecDomain", oldPrefs.getCharPref("ecDomain"));
				oldPrefs.clearUserPref("ecDomain");
				this.prefs.setCharPref("ecFolderbase", oldPrefs.getCharPref("ecFolderbase"));
				oldPrefs.clearUserPref("ecFolderbase");
				this.prefs.setCharPref("ecFolderpath", oldPrefs.getCharPref("ecFolderpath"));
				oldPrefs.clearUserPref("ecFolderpath");
				try {
					this.prefs.setCharPref("ecFolderID", oldPrefs.getCharPref("ecFolderID"));
					oldPrefs.clearUserPref("ecFolderID");
					this.prefs.setCharPref("ecChangeKey", oldPrefs.getCharPref("ecChangeKey"));
					oldPrefs.clearUserPref("ecChangeKey");
				}
				catch(err) {
				}
				this.addActivity(calGetString("calExchangeCalendar", "updateUserPrefs1", null, "exchangecalendar"), "xx", Date.now(), Date.now());
			}
			catch(err) {
				// If we get here old prefs do not exists. Al is fine.
			}

			getFreeBusyService().addProvider(this);

			// The first thing we want to do is check the folderbase and folderpath for their id & changekey.
			// It might have changed between restarts.
			this.newCalendarCheck();
			this.checkFolderPath();
			this.getSyncState();
			this.getTimeZones();

		}
		return returnVal;

	},

	getCharPref: function(aName) {
		return ews.safeGetCharPref(this.prefs, aName, null);
	},

	setCharPref: function(aName, aValue) {
		if (this.prefs) {
			return this.prefs.setCharPref(aName, aValue);
		}
	},

	get user() {
		var username = ews.safeGetCharPref(this.prefs, "ecUser", "");
		if (username.indexOf("@") > -1) {
			return username;
		}
		else {
			return this.domain+"\\"+ews.safeGetCharPref(this.prefs, "ecUser", "");
		}
	},

	set user(value) {
		if ((value.indexOf("\\") > -1) && (value.indexOf("@") == -1)) {
			this.domain = value.substr(0,value.indexOf("\\"));
			this.setCharPref("ecUser", value.substr(value.indexOf("\\")+1));
		}
		else {
			this.setCharPref("ecUser", value);
		}
	},

	get domain() {
		return ews.safeGetCharPref(this.prefs, "ecDomain", "");
	},

	set domain(value) {
		return this.setCharPref("ecDomain", value);
	},

	get mailbox() {
		return ews.safeGetCharPref(this.prefs, "ecMailbox", "");
	},

	get serverUrl() {
		return ews.safeGetCharPref(this.prefs, "ecServer", "");
	},

	get userDisplayName() {
		return ews.safeGetCharPref(this.prefs, "ecDisplayname", "");
	},

	get folderBase() {
		return ews.safeGetCharPref(this.prefs, "ecFolderbase", "calendar");
	},

	get folderPath() {
		return ews.safeGetCharPref(this.prefs, "ecFolderpath", "/");
	},

	get folderID() {
		return ews.safeGetCharPref(this.prefs, "ecFolderID", null);
	},

	set folderID(aValue) {
		this.prefs.setCharPref("ecFolderID", aValue);
	},

	get changeKey() {
		return ews.safeGetCharPref(this.prefs, "ecChangeKey", null);
	},

	set changeKey(aValue) {
		this.prefs.setCharPref("ecChangeKey", aValue);
	},

	get isPublicFolder()
	{
		if (publicFoldersMap[this.folderBase]) {
			return true;
		}
		else {
			return false;
		}
	},

	get doPollInbox() {
		if (this.doReset) {
			return false;
		}

		return ews.safeGetBoolPref(this.prefs, "ecPollInbox", true);
	},

	get pollInboxInterval() {
		return ews.safeGetIntPref(this.prefs, "ecPollInboxInterval", 3);
	},

	get doAutoRespondMeeting() {
		return ews.safeGetBoolPref(this.prefs, "ecAutoRespondMeetingRequest", false);
	},

	get autoResponseAnswer() {
		return ews.safeGetCharPref(this.prefs, "ecAutoRespondAnswer", "TENTATIVE");
	},

	get doAutoRemoveInvitationCancelation1() {
		return ews.safeGetBoolPref(this.prefs, "ecAutoRemoveInvitationCancellation1", false);
	},

	get doAutoRemoveInvitationCancelation2() {
		return ews.safeGetBoolPref(this.prefs, "ecAutoRemoveInvitationCancellation2", false);
	},

	get doAutoRemoveInvitationResponse1() {
		return ews.safeGetBoolPref(this.prefs, "ecAutoRemoveInvitationResponse1", true);
	},

	get sendAutoRespondMeetingRequestMessage() {
		return ews.safeGetBoolPref(this.prefs, "ecSendAutoRespondMeetingRequestMessage", false);
	},

	get autoRespondMeetingRequestMessage() {
		return ews.safeGetCharPref(this.prefs, "ecAutoRespondMeetingRequestMessage", "");
	},

	newCalendarCheck: function ecnewCalendarCheck() {

		// Check if we have ecServer preference.
		try {
			this.prefs.getCharPref("ecServer");
		}
		catch(err) {
			// we did not have the ecServer preference.
			// This is a new calendar. Fill it with the create values.
			this.logInfo("We have to create a new calendar in TB prefs");
			var createPrefs = Cc["@mozilla.org/preferences-service;1"]
		                    .getService(Ci.nsIPrefService)
				    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl.createcalendar.");

			this.prefs.setCharPref("ecServer", createPrefs.getCharPref("server"));
			this.prefs.setCharPref("ecMailbox", createPrefs.getCharPref("mailbox"));
			this.prefs.setCharPref("ecDisplayname", createPrefs.getCharPref("displayname"));
			this.prefs.setCharPref("ecUser", createPrefs.getCharPref("user"));
			this.prefs.setCharPref("ecDomain", createPrefs.getCharPref("domain"));
			this.prefs.setCharPref("ecFolderbase", createPrefs.getCharPref("folderbase"));
			this.prefs.setCharPref("ecFolderpath", createPrefs.getCharPref("folderpath"));
			try {
				this.prefs.setCharPref("ecFolderID", createPrefs.getCharPref("folderID"));
				this.prefs.setCharPref("ecChangeKey", createPrefs.getCharPref("changeKey"));
			}
			catch(err) {
			}

			this.setProperty("refreshInterval", "1");
		}

		//this.initialized = true;
		//this.myAvailable = true;

	},

	checkInbox: function _checkInbox()
	{
//		this.logInfo("checkInbox.");
		if ((this.weAreInboxSyncing) || (!this.doPollInbox)) {
			return;
		}

		this.weAreInboxSyncing = true;
		var self = this;

		this.addToQueue( erSyncInboxRequest,
			{user: this.user, 
			 mailbox: this.mailbox,
			 serverUrl: this.serverUrl,
			 folderBase: 'inbox',
			 folderID: null,
			 changeKey: null,
			 syncState: this.syncInboxState,
			 actionStart: Date.now() }, 
			function(erSyncInboxRequest, creations, updates, deletions, syncState) { self.syncInboxOK(erSyncInboxRequest, creations, updates, deletions, syncState);}, 
			function(erSyncInboxRequest, aCode, aMsg) { self.syncInboxError(erSyncInboxRequest, aCode, aMsg);},
			null);
	},

	syncInbox: function _syncInbox()
	{
//		this.logInfo("syncInbox.");
		if ((this.weAreInboxSyncing) || (!this.doPollInbox)) {
			return;
		}

		this.inboxPoller.cancel();

		var self = this;
		this.weAreInboxSyncing = true;
		this.addToQueue( erSyncInboxRequest,
			{user: this.user, 
			 mailbox: this.mailbox,
			 serverUrl: this.serverUrl,
			 folderBase: 'inbox',
			 folderID: null,
			 changeKey: null,
			 syncState: this.syncInboxState,
			 actionStart: Date.now() }, 
			function(erSyncInboxRequest, creations, updates, deletions, syncState) { self.syncInboxOK(erSyncInboxRequest, creations, updates, deletions, syncState);}, 
			function(erSyncInboxRequest, aCode, aMsg) { self.syncInboxError(erSyncInboxRequest, aCode, aMsg);},
			null);
	},

	removeMeetingItem: function _removeMeetingItem(aRequestItem)
	{
		var self = this;

		this.logInfo("Going to remove meetingItem:"+aRequestItem.title);
		this.addToQueue( erDeleteItemRequest, 
			{user: this.user, 
			 mailbox: this.mailbox,
			 folderBase: 'inbox',
			 serverUrl: this.serverUrl,
			 item: null,
			 folderID: null,
			 id: aRequestItem.id,
			 changeKey: aRequestItem.getProperty("X-ChangeKey"),
			 itemType: "meeting"}, 
			function(erDeleteItemRequest) { self.removeMeetingItemOk(erDeleteItemRequest);}, 
			function(erDeleteItemRequest, aCode, aMsg) { self.removeMeetingItemError(erDeleteItemRequest, aCode, aMsg);},
			null,
			2);
	},

	removeResponseItem: function _removeResponseItem(aResponse)
	{
		var self = this;

		this.logInfo("Going to remove responseItem:"+aResponse.nsTypes::Subject.toString()+" from:"+aResponse.nsTypes::Sender.nsTypes::Mailbox.nsTypes::Name.toString()+" ("+aResponse.nsTypes::Sender.nsTypes::Mailbox.nsTypes::EmailAddress.toString()+")");
		this.addToQueue( erDeleteItemRequest, 
			{user: this.user, 
			 mailbox: this.mailbox,
			 folderBase: 'inbox',
			 serverUrl: this.serverUrl,
			 item: null,
			 folderID: null,
			 id: aResponse.nsTypes::ItemId.@Id,
			 changeKey: aResponse.nsTypes::ItemId.@ChangeKey,
			 itemType: "response"}, 
			function(erDeleteItemRequest) { self.removeMeetingItemOk(erDeleteItemRequest);}, 
			function(erDeleteItemRequest, aCode, aMsg) { self.removeMeetingItemError(erDeleteItemRequest, aCode, aMsg);},
			null,
			2);
	},

	removeMeetingItemOk: function _removeMeetingItemOk(erDeleteItemRequest)
	{
		this.saveCredentials(erDeleteItemRequest.argument);
		this.logInfo("removeItemOk: "+erDeleteItemRequest.argument.itemType);

	},

	removeMeetingItemError: function _removeMeetingItemError(erDeleteItemRequest, aCode, aMsg)
	{
		this.saveCredentials(erDeleteItemRequest.argument);
		this.logInfo("removeItemError: "+erDeleteItemRequest.argument.itemType+" msg:"+String(aMsg));
	},

	syncInboxOK: function _syncInboxOK(erSyncInboxRequest, creations, updates, deletions, syncState)
	{
//		this.logInfo("syncInboxOk.");
		this.saveCredentials(erSyncFolderItemsRequest.argument);

		if ((creations.meetingrequests.length > 0) || (updates.meetingrequests.length > 0) || (deletions.meetingrequests.length > 0)) {
			this.addActivity(calGetString("calExchangeCalendar", "syncInboxRequests", [creations.meetingrequests.length, updates.meetingrequests.length, deletions.meetingrequests.length, this.name], "exchangecalendar"), "", erSyncInboxRequest.argument.actionStart, Date.now());
			this.refresh();
		}

		if ((creations.meetingCancellations.length > 0) || (updates.meetingCancellations.length > 0) || (deletions.meetingCancellations.length > 0)) {
			this.addActivity(calGetString("calExchangeCalendar", "syncInboxCancelations", [creations.meetingCancellations.length, updates.meetingCancellations.length, deletions.meetingCancellations.length, this.name], "exchangecalendar"), "", erSyncInboxRequest.argument.actionStart, Date.now());
			this.refresh();
		}

		if ((creations.meetingResponses.length > 0) || (updates.meetingResponses.length > 0) || (deletions.meetingResponses.length > 0)) {
			this.addActivity(calGetString("calExchangeCalendar", "syncInboxResponses", [creations.meetingResponses.length, updates.meetingResponses.length, deletions.meetingResponses.length, this.name], "exchangecalendar"), "", erSyncInboxRequest.argument.actionStart, Date.now());
			this.refresh();
		}

		this.weAreInboxSyncing = false;

		// Do something with the output.
		if ((this.syncInboxState) && (syncState == this.syncInboxState)) {
			this.logError("Same syncState received.");
		}

		this.syncInboxState = syncState;

		// Do something with the results.
//		this.logInfo("syncInboxOK meetingrequests: Creation:"+creations.meetingrequests.length+", Updates:"+updates.meetingrequests.length+", Deletions:"+deletions.meetingrequests.length);
//		this.logInfo("syncInboxOK meetingCancellations: Creation:"+creations.meetingCancellations.length+", Updates:"+updates.meetingCancellations.length+", Deletions:"+deletions.meetingCancellations.length);

		// Save requests into cache.
		for each (var request in creations.meetingrequests) {
			var meetingItem = this.convertExchangeAppointmentToCalAppointment(request, true);
			if (meetingItem) {
				this.logInfo(" -- MeetingRequest creation:"+ meetingItem.title+", UID:"+request.nsTypes::UID.toString()+",id:"+meetingItem.id+",changeKey:"+meetingItem.getProperty("X-ChangeKey"));
				meetingItem.setProperty("X-MEETINGREQUEST", true);
				meetingItem.setProperty("STATUS", "NONE")
				//this.meetingRequestsCache[request.nsTypes::UID.toString()] = meetingItem;
				this.meetingRequestsCache[meetingItem.id] = meetingItem;
			}
		}

		for each (var update in updates.meetingrequests) {
			var meetingItem = this.convertExchangeAppointmentToCalAppointment(update, true);
			if (meetingItem) {
				this.logInfo(" -- MeetingRequest update:"+ meetingItem.title+", UID:"+update.nsTypes::UID.toString()+",id:"+meetingItem.id+",changeKey:"+meetingItem.getProperty("X-ChangeKey"));
				meetingItem.setProperty("X-MEETINGREQUEST", true);
				
				if ((this.meetingRequestsCache[update.id]) && (this.meetingRequestsCache[update.id].getProperty("X-UID") == meetingItem.getProperty("X-UID"))) {
					this.logInfo("2 modifing  meeting request:"+update.id);
//					this.meetingRequestsCache[update.nsTypes::UID.toString()] = meetingItem;
					this.meetingRequestsCache[meetingItem.id] = meetingItem;
				}
				else {
					this.logInfo("WE DO NOT HAVE AN MEETING IN CACHE FOR THIS UPDATE!!!!. PLEASE REPORT");
				}
			}
		}

		for each (var deletion in deletions.meetingrequests) {
			var meetingItem = this.convertExchangeAppointmentToCalAppointment(deletion, true);
			if (meetingItem) {
				this.logInfo(" -- MeetingRequest deletion:"+ meetingItem.title+", UID:"+deletion.nsTypes::UID.toString()+",id:"+meetingItem.id+",changeKey:"+meetingItem.getProperty("X-ChangeKey"));
				meetingItem.setProperty("X-MEETINGREQUEST", true);
				this.removeFromMeetingRequestCache(deletion.id);			
				this.meetingrequestAnswered[deletion.id] = false;
			}
		}

		// Save cancelations into cache and remove request for which we received a cancelation.
		for each (var request in creations.meetingCancellations) {
			var cancelItem = this.convertExchangeAppointmentToCalAppointment(request, true);
			if (cancelItem) {
				cancelItem.setProperty("X-MEETINGCANCELATION", true);
				this.meetingCancelationsCache[request.id] = cancelItem;
			}
		}

		for each (var update in updates.meetingCancellations) {
			var cancelItem = this.convertExchangeAppointmentToCalAppointment(update, true);
			if (cancelItem) {
				cancelItem.setProperty("X-MEETINGCANCELATION", true);
				if (this.meetingCancelationsCache[update.id].getProperty("X-UID") == cancelItem.getProperty("X-UID")) {
					this.meetingCancelationsCache[update.id] = meetingItem;
				}
				else {
					this.logInfo("WE DO NOT HAVE AN MEETING IN CACHE FOR THIS UPDATE!!!!. PLEASE REPORT");
				}
			}
		}

		for each (var deletion in deletions.meetingCancellations) {
			delete this.meetingCancelationsCache[deletion.id];
		}

		var requestCount = 0;
		var cancelationCount = 0;
		var tmpInCalendarCache = {};
		for each (var index in this.meetingRequestsCache) {
			if (index) {
				// Remove request for which we have an calendaritem which is confirmed
				var tmpID = index.id;
				var tmpUID = index.getProperty("X-UID");
				var inCalendar = null;

				// First check recurring Masters
				if (this.recurringMasterCache[tmpUID]) {
					inCalendar = this.recurringMasterCache[tmpUID];
				}

				// Check single items
				if (!inCalendar) {
					for each (var item in this.itemCache) {
						if ((item) && (item.getProperty("X-UID") == tmpUID)) {
							inCalendar = item;
							break;
						}
					}
				}				
				
				var confirmed = false;

				// We turn this off for now (10-12-2011) This so meeting update items in mailbox are not removed.
				/*if (inCalendar) {
					//this.logInfo("request '"+inCalendar.title+"' allready in Calendar with status="+inCalendar.getProperty("STATUS"));
					//var newItem = 
					confirmed = (inCalendar.getProperty("STATUS") != "NONE");
					tmpInCalendarCache[tmpUID] = inCalendar;
				}*/

				if (confirmed) {	
					// Remove request. Meeting is in calendar and confirmed.
					this.removeMeetingItem(index);
					this.removeFromMeetingRequestCache(tmpID);
					//this.meetingRequestsCache[tmpUID] = null;
					this.meetingrequestAnswered[tmpID] = false;
				}
				else {
					// Keep this request
					requestCount++;
					//this.logInfo("meetingRequest:"+index.nsTypes::Subject.toString());

					// This is a new request. Check if we should autorespond.
					if (!this.meetingrequestAnswered[tmpID]) {
						this.meetingrequestAnswered[tmpID] = false;
					}

					if ((this.doAutoRespondMeeting) && (this.meetingrequestAnswered[tmpID] === false)) {

						var bodyText = null;
						if (this.sendAutoRespondMeetingRequestMessage) {
							bodyText = this.autoRespondMeetingRequestMessage;
							this.addActivity(calGetString("calExchangeCalendar", "sendAutoRespondMeetingRequestMessage", [index.title, this.name], "exchangecalendar"), "", Date.now(), Date.now());

						}

						if (inCalendar) {
							//var aItem = inCalendar.clone();
							var aItem = this.cloneItem(inCalendar);
							this.sendMeetingRespons(aItem, null, "existing", this.autoResponseAnswer, bodyText);
						}
						else {
							//var aItem = index.clone();
							var aItem = this.cloneItem(index);
							this.sendMeetingRespons(aItem, null, "existing", this.autoResponseAnswer, bodyText); // TODO: might have to set new
						}
						this.meetingrequestAnswered[tmpID] = true;
					}
				}

				// We turn this off for now (10-12-2011) This so meeting update items in mailbox are not removed.
/*				if (inCalendar) {
					// remove our cached request from calendar otherwise it will appear twice.
					this.observers.notify("onDeleteItem", [index]);
					this.removeFromMeetingRequestCache(tmpUID);
					//this.meetingRequestsCache[tmpUID] == null;
					this.meetingrequestAnswered[tmpUID] = false;
				}*/
			}
		}
		for each (var index in this.meetingCancelationsCache) {
			if (index) {
				// Remove cancelation for which we do not have an calendaritem.
				var tmpID = index.id;
				var tmpUID = index.getProperty("X-UID");
				var inCalendar = false;

				// First check recurring Masters
				if (this.recurringMasterCache[tmpUID]) {
					inCalendar = this.recurringMasterCache[tmpUID];
				}

				// Check single items
				if (!inCalendar) {
					for each (var item in this.itemCache) {
						if ((item) && (item.getProperty("X-UID") == tmpUID)) {
							inCalendar = item;
							break;
						}
					}
				}				

				// Check meetingrequest cache
				var inCache = false;					
				if (!inCalendar) {
					inCalendar = this.meetingRequestsCache[tmpID];
					if (inCalendar) {
						// We have a cancellation and a request but no item in the calendar
						inCache = true;
					}
				}
	
				if (inCalendar) {	
					// Keep this cancelation if it is allready confirmed. Or remove it when we we have it only in cache.
					if (((inCalendar.getProperty("STATUS") == "NONE") && (this.doAutoRemoveInvitationCancelation1)) ||
						(inCache)) {
						// We have an calendaritem for this cancelation but it is not yet confirmed.
						// We remove the cancelation and the meetingitem if it is not in cache
						// and user specified this in the EWS settings.
						if (inCache) {
							// We remove it from the cache
							if (this.meetingRequestsCache[tmpID]) {
								this.removeMeetingItem(this.meetingRequestsCache[tmpID]);
								this.removeFromMeetingRequestCache(tmpID);
								//this.meetingRequestsCache[tmpUID] = null;
							}
							this.removeMeetingItem(index);
							this.observers.notify("onDeleteItem", [inCalendar]);
							this.removeFromMeetingRequestCache(tmpID);
							//this.meetingRequestsCache[tmpUID] = null;
							this.meetingrequestAnswered[tmpID] = false;
							delete this.meetingCancelationsCache[tmpID];
						}
						else {
							// We remove it from the real calendar.
							this.removeMeetingItem(index);
							this.deleteItem(inCalendar);
						}
					}
					else {
						if (! inCalendar.getProperty("X-IsCancelled")) {
							if (!this.doAutoRemoveInvitationCancelation2) {
								// It is in calendar and allready confirmed. We update the current calendaritem.
								// Change title of current calendaritem.
								//var newItem = inCalendar.clone();
								var newItem = this.cloneItem(inCalendar);
								newItem.setProperty("X-IsCancelled", true);
								newItem.title = index.title;
								// Set status in calendar as free.
								newItem.setProperty("TRANSP", "TRANSPARENT");
								if (inCache) {
									this.observers.notify("onModifyItem", [newItem, inCalendar]);
									this.meetingRequestsCache[tmpID] = newItem;
								}
								else {
									this.modifyItem(inCalendar, newItem, null);
								}
								cancelationCount++;
							}
							else {
								// Remove calendar item and cancellation message
								// because user specified so in the EWS settings.
								// We remove it from the real calendar and in the inbox.
								this.addActivity(calGetString("calExchangeCalendar", "autoRemoveConfirmedInvitationOnCancellation", [inCalendar.title, this.name], "exchangecalendar"), "", Date.now(), Date.now());
								inCalendar.setProperty("X-IsCancelled", true);
								this.deleteItem(inCalendar);
								this.removeMeetingItem(index);
								this.removeFromMeetingRequestCache(tmpID);
								//this.meetingRequestsCache[tmpUID] = null;
							}
						}
						delete this.meetingCancelationsCache[tmpID];
						this.logInfo("meetingCancelation:"+index.title);
					}
				}
				else {
					// Meeting is not in calendar yet. We remove it if user specified it
					// as a setting in EWS settings.
					if (this.doAutoRemoveInvitationCancelation1) {
						if (this.meetingRequestsCache[tmpID]) {
							this.removeMeetingItem(this.meetingRequestsCache[tmpID]);
							this.removeFromMeetingRequestCache(tmpID);
							//this.meetingRequestsCache[tmpUID] = null;
						}
						this.removeMeetingItem(index);
						delete this.meetingCancelationsCache[tmpID];
					}
				}
			}
		}

		// Process Meetingresponses
		// Save responses into cache and remove request for which we received a cancelation.
		for each (var response in creations.meetingResponses) {
			this.meetingResponsesCache[response.nsTypes::ItemId.@Id.toString()] = response;
		}

		for each (var response in updates.meetingResponses) {
			if (this.meetingResponsesCache[response.nsTypes::ItemId.@Id.toString()]) {
				this.meetingResponsesCache[response.nsTypes::ItemId.@Id.toString()] = response;
			}
			else {
				this.logInfo("WE DO NOT HAVE AN RESPONSE IN CACHE FOR THIS UPDATE!!!!. PLEASE REPORT");
			}
		}

		for each (var response in deletions.meetingResponses) {
			if (this.meetingResponsesCache[response.nsTypes::ItemId.@Id.toString()]) {
				delete this.meetingResponsesCache[response.nsTypes::ItemId.@Id.toString()];
			}
		}


		if (this.doAutoRemoveInvitationResponse1) {
			for each(var response in this.meetingResponsesCache) {
				// Check if we have this meeting 
				var tmpUID = response.nsTypes::UID.toString();
				var inCalendar = false;

				// First check recurring Masters
				if (this.recurringMasterCache[tmpUID]) {
					inCalendar = this.recurringMasterCache[tmpUID];
				}

				// Check single items
				if (!inCalendar) {
					for each (var item in this.itemCache) {
						if ((item) && (item.getProperty("X-UID") == tmpUID)) {
							inCalendar = item;
							break;
						}
					}
				}

				if (inCalendar) {
					// Check if we are the organiser of this item.
					var iAmOrganizer = ((inCalendar.organizer) && (inCalendar.organizer.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase()));
					if (!iAmOrganizer) {
						// Remove the response in the inbox. Do not update calendar.
						this.removeResponseItem(response);
						delete this.meetingResponsesCache[response.nsTypes::ItemId.@Id.toString()];
					}
				}				
			}
		}

		if ((requestCount > 0) || (cancelationCount > 0)) {
			this.refresh();
		}

//		this.logInfo("syncInboxOK: left with meetingRequests:"+requestCount+", meetingCancelations:"+cancelationCount);

		this.startSyncInboxPoller();
	},

	startSyncInboxPoller: function _startSyncInboxPoller()
	{
		if (!this.doPollInbox) {
			return;
		}

		this.inboxPoller.cancel();
	        let self = this;
		this.inboxPoller.initWithCallback({ notify: function setTimeout_notify() {self.syncInbox();	}}, this.pollInboxInterval * 1000, this.inboxPoller.TYPE_REPEATING_SLACK);
	},

	syncInboxError: function _syncFolderItemsError(erSyncFolderItemsRequest, aCode, aMsg)
	{
		this.saveCredentials(erSyncFolderItemsRequest.argument);
		this.weAreInboxSyncing = false;
		//this.processItemSyncQueue();
		
		this.startSyncInboxPoller();
	},

	doAvailability: function _doAvailability(aCalId, aCi)
	{
		const x = Ci.calIFreeBusyInterval;

		const types = {
			"Free" 		: x.FREE,
			"Tentative"	: x.BUSY_TENTATIVE,
			"Busy"		: x.BUSY,
			"OOF"		: x.BUSY_UNAVAILABLE,
			"NoData"	: x.UNKNOWN
		};

		var start = this.tryToSetDateValue(aCi.nsTypes::StartTime);
		var end   = this.tryToSetDateValue(aCi.nsTypes::EndTime);
		var type  = types[aCi.nsTypes::BusyType];

		return new cal.FreeBusyInterval(aCalId, type,
					       	  start, end);
	},

	getUserAvailabilityRequestOK: function _getUserAvailabilityRequestOK(erGetUserAvailabilityRequest, aEvents)
	{
		this.saveCredentials(erGetUserAvailabilityRequest.argument);
		var items = new Array();
		for (var index in aEvents) {
			var item = this.doAvailability(erGetUserAvailabilityRequest.argument.calId, aEvents[index]);
			items.push(item);
		}
		
		erGetUserAvailabilityRequest.listener.onResult(null, items);
	},
	
	getUserAvailabilityRequestError: function _getUserAvailabilityRequestError(erGetUserAvailabilityRequest, aCode, aMsg)
	{
		this.saveCredentials(erGetUserAvailabilityRequest.argument);
		erGetUserAvailabilityRequest.listener.onResult(null, null);
	},

	resetCalendar: function _resetCalendar()
	{
		//this.logInfo(" resetCalendar 1");

		for each(var tmpJob in this.tmpJobs) {
			if (tmpJob) {
				if ((tmpJob.isRunning) && (tmpJob.parent)) {
					tmpJob.parent.stopRequest();
				}
			}
		}

		// Clean the job queue.
		for each(var tmpQueue in this.queue) {
			this.observerService.notifyObservers(this, "onExchangeProgressChange", -1*tmpQueue.length); 
		}

		this.queue = new Array;

		this.doReset = true;

		this.resetStart = Date.now();

		this.inboxPoller.cancel();
		for (var index in this.timers) {
			if (this.timers[index]) {
				this.timers[index].cancel();
				delete this.timers[index];
			}
		}
		if (this.calendarPoller) {
			this.calendarPoller.cancel();
		}

		this.performReset();
	},

	performReset: function _performReset()
	{
		//this.logInfo(" performReset 1");

		if (!this.doReset) {
			return;
		}

		this.doReset = false;

		for each(var tmpJob in this.tmpJobs) {
			if (tmpJob) {
				if ((tmpJob.isRunning) && (tmpJob.parent)) {
					tmpJob.parent.stopRequest();
				}
			}
		}

		// Clean the job queue again.
		for each(var tmpQueue in this.queue) {
			this.observerService.notifyObservers(this, "onExchangeProgressChange", -1*tmpQueue.length); 
		}

		this.queue = new Array;

		// Now we can initialize.
		this.syncState = null;
		this.weAreSyncing = false;
		this.firstSyncDone = false;
		
		// Remove all items in cache from calendar.
		for (var index in this.itemCache) {
			if (this.itemCache[index]) {
				//this.itemCache[index].clearAlarms();
				this.deleteAlarms(this.itemCache[index]);
				this.observers.notify("onDeleteItem", [this.itemCache[index]]);
			}
		}

		// Reset caches.
		this.itemCache = [];
		this.recurringMasterCache = [];

		if (this.startDate) {
			var oldBeginDate = this.startDate.clone();
		}
		else {
			this.logInfo(" THIS IS STRANGE beginDate");
			var oldBeginDate = now();
		}
		if (this.endDate) {
			var oldEndDate = this.endDate.clone();
		}
		else {
			this.logInfo(" THIS IS STRANGE endDate");
			var oldEndDate = now();
		}

		this.startDate = null;
		this.endDate = null;
	
		// Reload calendar on currently known dateRanges.
		for (var index in this.meetingRequestsCache) {
			if (this.meetingRequestsCache[index]) {
				this.observers.notify("onDeleteItem", [this.meetingRequestsCache[index]]);
				this.removeFromMeetingRequestCache(index);
				//this.meetingRequestsCache[index] = null;
				this.meetingrequestAnswered[index] = false;
			}
		}
		this.meetingRequestsCache = [];
		this.meetingCancelationsCache = [];
		this.meetingrequestAnswered = [];
		this.meetingResponsesCache = [];
		this.syncInboxState = null;
		this.weAreInboxSyncing = false;

		this.supportsTasks = true;
		this.supportsEvents = true;

		this.checkFolderPath();
		this.getSyncState();
		this.getTimeZones();

		this.logInfo("oldBeginDate:"+oldBeginDate.toString()+", oldEndDate:"+oldEndDate.toString());
		this.getItems(Ci.calICalendar.ITEM_FILTER_TYPE_TODO +
				Ci.calICalendar.ITEM_FILTER_TYPE_EVENT
				, 0, oldBeginDate, oldEndDate, null);

		// Make an event for thistory.
		this.addActivity(calGetString("calExchangeCalendar", "resetEventMessage", [this.name], "exchangecalendar"), "", this.resetStart, Date.now());
	},

	addActivity: function _addActivity(aTitle, aText, aStartDate, aEndDate)
	{
		if (!gActivityManager) {
			return;
		}

		let event = Cc["@mozilla.org/activity-event;1"].createInstance(nsIAE);  
  
		event.init(aTitle,  
		           null,   
		           aText,   
		           aStartDate,     
		           aEndDate);          
  
		gActivityManager.addActivity(event);
	},

	makeRecurrenceRule: function _makeRecurrenceRule(aItem, e)
	{
		if (!aItem.parentItem) {
			return;
		}

		if (!aItem.recurrenceInfo || aItem.parentItem.id != aItem.id) {
			if (!aItem.recurrenceInfo) {
				this.logInfo("makeRecurrenceRule: We have no recurrenceInfo");
			}
			if (aItem.parentItem.id != aItem.id) {
				this.logInfo("makeRecurrenceRule: We have aItem.parentItem.id != aItem.id");
			}
			return;
		}

		var rrule = null;
		for each (var ritem in aItem.recurrenceInfo.getRecurrenceItems({})) {
			if (calInstanceOf(ritem, Ci.calIRecurrenceRule)) {
				rrule = ritem;
this.logInfo(" ;;;; rrule:"+rrule.icalProperty.icalString);
				//break;
			}
		}

		if (!rrule) {
			// XXX exception?
			this.logInfo("makeRecurrenceRule: We have no rrule");
			return;
		}

		var r = <nsTypes:Recurrence xmlns:nsTypes={nsTypes}/>;

		/* can't get parameters of RRULEs... have to do it manually :/ */
		var prop = {};
		for each (let ps in rrule.icalProperty.value.split(';')) {
			let m = ps.split('=');
			prop[m[0]] = m[1];
		}

		if (aItem.startDate) {
			var startDate = aItem.startDate.clone();
		}
		else {
			var startDate = cal.now();
		}
		startDate.isDate = true;
	
		prop["BYMONTHDAY"] = prop["BYMONTHDAY"] || startDate.day;
		prop["BYMONTH"] = prop["BYMONTH"] || (startDate.month + 1);

		switch (rrule.type) {
		case 'YEARLY':
			if (prop["BYDAY"]) {
				var m = prop["BYDAY"].match(/^(-?\d)(..)$/);
				r.nsTypes::RelativeYearlyRecurrence.nsTypes::DaysOfWeek = dayRevMap[m[2]];
				r.nsTypes::RelativeYearlyRecurrence.nsTypes::DayOfWeekIndex = weekRevMap[m[1]];
				r.nsTypes::RelativeYearlyRecurrence.nsTypes::Month = monthIdxMap[prop["BYMONTH"] - 1];
			} else {
				r.nsTypes::AbsoluteYearlyRecurrence.nsTypes::DayOfMonth = prop["BYMONTHDAY"];
				r.nsTypes::AbsoluteYearlyRecurrence.nsTypes::Month = monthIdxMap[prop["BYMONTH"] - 1];
			}
			break;
		case 'MONTHLY':
			if (prop["BYDAY"]) {
				r.nsTypes::RelativeMonthlyRecurrence.nsTypes::Interval = rrule.interval;
				var m = prop["BYDAY"].match(/^(-?\d)(..)$/);
				r.nsTypes::RelativeMonthlyRecurrence.nsTypes::DaysOfWeek = dayRevMap[m[2]];
				r.nsTypes::RelativeMonthlyRecurrence.nsTypes::DayOfWeekIndex = weekRevMap[m[1]];
			} else {
				r.nsTypes::AbsoluteMonthlyRecurrence.nsTypes::Interval = rrule.interval;
				r.nsTypes::AbsoluteMonthlyRecurrence.nsTypes::DayOfMonth = prop["BYMONTHDAY"];
			}
			break;
		case 'WEEKLY':
			r.nsTypes::WeeklyRecurrence.nsTypes::Interval = rrule.interval;
			var days = [];
			var daystr = prop["BYDAY"] || dayIdxMap[startDate.weekday];
			for each (let day in daystr.split(",")) {
				days.push(dayRevMap[day]);
			}
			r.nsTypes::WeeklyRecurrence.nsTypes::DaysOfWeek = days.join(' ');
			break;
		case 'DAILY':
			r.nsTypes::DailyRecurrence.nsTypes::Interval = rrule.interval;
			break;
		}

		if (rrule.isByCount && rrule.count != -1) {
			r.nsTypes::NumberedRecurrence.nsTypes::StartDate = cal.toRFC3339(startDate.getInTimezone(ews.ecUTC()));
			r.nsTypes::NumberedRecurrence.nsTypes::NumberOfOccurrences = rrule.count;
		} else if (!rrule.isByCount && rrule.untilDate) {
			var endDate = rrule.untilDate.clone();
			endDate.isDate = true;
			r.nsTypes::EndDateRecurrence.nsTypes::StartDate = cal.toRFC3339(startDate.getInTimezone(ews.ecUTC()));
			r.nsTypes::EndDateRecurrence.nsTypes::EndDate = cal.toRFC3339(endDate.getInTimezone(ews.ecUTC()));
		} else {
			r.nsTypes::NoEndRecurrence.nsTypes::StartDate = cal.toRFC3339(startDate.getInTimezone(ews.ecUTC()));
		}

		/* We won't write WKST/FirstDayOfWeek for now because it is Exchange 2010 and up */

		e.appendChild(r);
	},

	getAlarmTime: function _getAlarmTime(aItem)
	{
		if (!aItem) {
			return null;
		}

		var alarms = aItem.getAlarms({});		
		var alarm = alarms[0];		
		if (!alarm) {
			return null;
		}

		switch (alarm.related) {
		case Ci.calIAlarm.ALARM_RELATED_ABSOLUTE:
			var alarmTime = alarm.alarmDate;
			break;
		case Ci.calIAlarm.ALARM_RELATED_START:
			if (isEvent(aItem)) {
				var alarmTime = aItem.startDate.clone();
			}
			else {
				var alarmTime = aItem.entryDate.clone();
			}
			alarmTime.addDuration(alarm.offset);
			break;
		case Ci.calIAlarm.ALARM_RELATED_END:
			if (isEvent(aItem)) {
				var alarmTime = aItem.endDate.clone();
			}
			else {
				var alarmTime = aItem.dueDate.clone();
			}
			alarmTime.addDuration(alarm.offset);
			break;
		}

		return alarmTime;
	},

	getSingleSnoozeState: function _getSingleSnoozeState(e, aSingle)
	{
		this.logInfo("getSingleSnoozeState");
		var tmpStr = "";
		var mozSnooze = aSingle.getProperty("X-MOZ-SNOOZE-TIME");
		if (mozSnooze) {
			this.logInfo("getSingleSnoozeState: X-MOZ-SNOOZE-TIME");
			tmpStr = tmpStr + "|X-MOZ-SNOOZE-TIME="+mozSnooze;

			let eprop = <nsTypes:ExtendedProperty xmlns:nsTypes={nsTypes}/>;
			eprop.nsTypes::ExtendedFieldURI.@PropertySetId = calExchangeCalendarGUID;
			eprop.nsTypes::ExtendedFieldURI.@PropertyName = "X-MOZ-SNOOZE-TIME";
			eprop.nsTypes::ExtendedFieldURI.@PropertyType = "String";
			eprop.nsTypes::Value = mozSnooze;

			e.appendChild(eprop);
		}
		this.logInfo("getSingleSnoozeState END");
		return tmpStr;
	},

	getMasterSnoozeStates: function _getMasterSnoozeStates(e, aMaster)
	{
		this.logInfo("getMasterSnoozeStates");
		var tmpStr = "";
		if ((aMaster.id == aMaster.parentItem.id) && (aMaster.recurrenceInfo) && (aMaster.propertyEnumerator)) {
			var props = aMaster.propertyEnumerator;
			while (props.hasMoreElements()) {
				var prop = props.getNext().QueryInterface(Components.interfaces.nsIProperty);
				if (prop.name.indexOf("X-MOZ-SNOOZE-TIME-") == 0) {
					this.logInfo("getMasterSnoozeStates: "+prop.name+"="+prop.value);

					// Find occurrence/exception this snooze belongs to
					var nativeTime = prop.name.substr(18);
					var occ;
					for each(var child in this.itemCache) {
						if ((child.recurrenceId) && (child.recurrenceId.nativeTime == nativeTime)) {
							occ = child;
						}
					}
					if (occ) {
						// We found the occurrence which was snoozed/dismisses
						this.logInfo("getMasterSnoozeStates: Found occurrence which was snoozed. startDate:"+occ.startDate.toString());
						//this.logInfo("getMasterSnoozeStates: occurrence alarmLastAck:"+this.getAlarmLastAck(e, occ));
						//this.logInfo("getMasterSnoozeStates: occurrence snooze:"+this.getSingleSnoozeState(e, occ));
					}

					tmpStr = tmpStr + "|"+prop.name+"="+prop.value;
				}
			}
		}

		if (tmpStr != "") {
			let eprop = <nsTypes:ExtendedProperty xmlns:nsTypes={nsTypes}/>;
			eprop.nsTypes::ExtendedFieldURI.@PropertySetId = calExchangeCalendarGUID;
			eprop.nsTypes::ExtendedFieldURI.@PropertyName = "X-MOZ-SNOOZE-TIME-";
			eprop.nsTypes::ExtendedFieldURI.@PropertyType = "String";
			eprop.nsTypes::Value = tmpStr;

			e.appendChild(eprop);
		}
		this.logInfo("getMasterSnoozeStates END");
		return tmpStr;
	},

	getAlarmLastAck: function _getAlarmLastAck(e, aItem)
	{
		this.logInfo("getAlarmLastAck");
		var tmpStr = "";
		if (aItem.alarmLastAck) {
			this.logInfo("getAlarmLastAck: alarmLastAck:"+aItem.alarmLastAck.toString());
			var tmpDateTime = aItem.alarmLastAck.getInTimezone(ews.ecUTC());
			tmpStr = tmpStr + "alarmLastAck="+cal.toRFC3339(tmpDateTime.getInTimezone(ews.ecDefaultTimeZone()));

			
			let eprop = <nsTypes:ExtendedProperty xmlns:nsTypes={nsTypes}/>;
			eprop.nsTypes::ExtendedFieldURI.@PropertySetId = calExchangeCalendarGUID;
			eprop.nsTypes::ExtendedFieldURI.@PropertyName = "alarmLastAck";
			eprop.nsTypes::ExtendedFieldURI.@PropertyType = "SystemTime";
			eprop.nsTypes::Value = cal.toRFC3339(tmpDateTime.getInTimezone(ews.ecDefaultTimeZone()));

			e.appendChild(eprop);

		}
		this.logInfo("getAlarmLastAck END");
		return tmpStr;
	},

	addSnoozeDismissState: function _addSnoozeDismissState(e, aItem, aAlarmTime)
	{
		// Check if we have a single item or not
		var tmpStr = this.getAlarmLastAck(e, aItem);

		this.logInfo("addSnoozeDismissState: Start1");

		if (aAlarmTime) {
			this.logInfo("addSnoozeDismissState: item has alarms");

			var tmpDateTime;

			if ((aItem.id != aItem.parentItem.id) && (aItem.parentItem.recurrenceInfo)) {
				this.logInfo("addSnoozeDismissState: Occurrence or Exception");
				// Find out which one got snoozed/dismisses
			}
			else {
				if (aItem.recurrenceInfo) {
					this.logInfo("addSnoozeDismissState: Master");
					tmpStr = tmpStr + this.getMasterSnoozeStates(e, aItem);
				}
				else {
					this.logInfo("addSnoozeDismissState: Single");
					tmpStr = tmpStr + this.getSingleSnoozeState(e, aItem);
				}

			}
			// If an item has no X-MOZ-SNOOZE then is it dismissed
		}
		else {
			this.logInfo("addSnoozeDismissState: item has no alarms");
			if ((aItem.id != aItem.parentItem.id) && (aItem.parentItem.recurrenceInfo)) {
				this.logInfo("addSnoozeDismissState: Occurrence or Exception");
			}
			else {
				if (aItem.recurrenceInfo) {
					this.logInfo("addSnoozeDismissState: Master");
					tmpStr = tmpStr + this.getMasterSnoozeStates(e, aItem);
				}
				else {
					this.logInfo("addSnoozeDismissState: Single");
					tmpStr = tmpStr + this.getSingleSnoozeState(e, aItem);
				}

			}
		}

		this.logInfo("addSnoozeDismissState: End:"+tmpStr);
	},

	writeEP_DismissSnoozeState: function _writeEP_DismissSnoozeState(aItem, e)
	{
		let par = aItem.parentItem;
		let nextRem = cal.createDateTime("4501-01-01T00:00:00Z");
	
		let lastAck = par.alarmLastAck;
		let nextOcc;
	
		if (par.recurrenceInfo) {
			if (!lastAck) {
				/* no ack yet, get first occurrence */
				nextOcc = par.recurrenceInfo.getOccurrences(par.startDate, null, 1, {})[0];
			} else {
				nextOcc = par.recurrenceInfo.getNextOccurrence(lastAck);
			}
		} else {
			nextOcc = aItem;
		}
	
		let alarmDate = null;
		let alarm = aItem.getAlarms({})[0];
		if (nextOcc && alarm) {
			alarmDate = cal.alarms.calculateAlarmDate(nextOcc, alarm);
		}
	
		let lastSnooze = cal.createDateTime();
		if (par.getProperty('X-MOZ-SNOOZE-TIME')) {
			lastSnooze = cal.createDateTime(par.getProperty('X-MOZ-SNOOZE-TIME'));
		}
	
		if (aItem.propertyEnumerator) {
			let props = aItem.propertyEnumerator;
			while (props.hasMoreElements()) {
				let prop = props.getNext().QueryInterface(Components.interfaces.nsIProperty);
				if (prop.name.substr(0, 18) == 'X-MOZ-SNOOZE-TIME-') {
					let st = cal.createDateTime(prop.value);
					if (st.compare(lastSnooze) > 0) {
						lastSnooze = st;
					}
				}
			}
		}
	
		if (lastSnooze.compare(cal.createDateTime()) > 0 &&	/* valid snooze */
		    alarmDate &&					/* have upcoming alarm */
		    lastSnooze.compare(alarmDate) < 0 &&		/* snooze before alarm */
		    (!lastAck || lastSnooze.compare(lastAck) > 0)) {	/* snozze after ack */
			nextRem = lastSnooze;
		} else if (alarmDate &&
			   (!lastAck || alarmDate.compare(lastAck) > 0)) {
			nextRem = alarmDate;
		}
	
		nextRem = nextRem.getInTimezone(cal.UTC());
	
		let eprop = <nsTypes:ExtendedProperty xmlns:nsTypes={nsTypes}/>;
		eprop.nsTypes::ExtendedFieldURI.@DistinguishedPropertySetId = "Common";
		eprop.nsTypes::ExtendedFieldURI.@PropertyId = "34144";
		eprop.nsTypes::ExtendedFieldURI.@PropertyType = "SystemTime";
		eprop.nsTypes::Value = cal.toRFC3339(nextRem);
	
		e.appendChild(eprop);
	},

	convertCalAppointmentToExchangeAppointment: function _convertCalAppointmentToExchangeAppointment(aItem, aAction, isNew)
	{
		if (!aAction) {
			aAction = "modify";
		}

		// The order in which create items are specified is important.
		// EWS expects the right order.

		var e = <nsTypes:CalendarItem xmlns:nsTypes={nsTypes} xmlns:nsMessages={nsMessages}/>;

		e.nsTypes::Subject = aItem.title;

		const privacies = { "PUBLIC": "Normal",
				"CONFIDENTIAL": "Confidential", 
				"PRIVATE" : "Private",
				null: "Normal" };
		e.nsTypes::Sensitivity = privacies[aItem.privacy];

		e.nsTypes::Body.@BodyType = "Text";
		e.nsTypes::Body = aItem.getProperty('DESCRIPTION') || "";

		var categories = aItem.getCategories({});
		for each (var category in categories) {
			e.nsTypes::Categories.list += <nsTypes:String xmlns:nsTypes={nsTypes}>{category}</nsTypes:String>;
		}
	
		e.nsTypes::Importance = "Normal";
		if (aItem.priority > 5) {
			e.nsTypes::Importance = "Low";
		}
		if (aItem.priority == 5) {
			e.nsTypes::Importance = "Normal";
		}
		if (aItem.priority < 5) {
			e.nsTypes::Importance = "High";
		}
		if (aItem.priority == 0) {
			e.nsTypes::Importance = "Normal";
		}

		var alarmTime = this.getAlarmTime(aItem);
		if (alarmTime) {
			this.logInfo("alarmTime = "+alarmTime.toString());

			this.logInfo("length="+aItem.getAlarms({}).length);

			for each(var alarm in aItem.getAlarms({})) {
				break;
			}

			switch (alarm.related) {
			case Ci.calIAlarm.ALARM_RELATED_ABSOLUTE:
				this.logInfo("ALARM_RELATED_ABSOLUTE");
				var newAlarmTime = alarm.alarmDate.clone();
				var offset = cal.createDuration();
				offset.inSeconds = 0;
				break;
			case Ci.calIAlarm.ALARM_RELATED_START:
				this.logInfo("ALARM_RELATED_START");
				var newAlarmTime = aItem.startDate.clone();
				var offset = alarm.offset.clone();
				break;
			case Ci.calIAlarm.ALARM_RELATED_END:
				this.logInfo("ALARM_RELATED_END");
				var newAlarmTime = aItem.endDate.clone();
				var offset = alarm.offset.clone();
				break;
			}
	
			if (newAlarmTime.isDate) {
				newAlarmTime.isDate = false;
				newAlarmTime = newAlarmTime.getInTimezone(ews.ecDefaultTimeZone());
			}

			if (!this.isInvitation(aItem, true)) {
				e.nsTypes::ReminderDueBy = cal.toRFC3339(newAlarmTime);
			}
			e.nsTypes::ReminderIsSet = 'true';
			if (offset.inSeconds != 0) {
				e.nsTypes::ReminderMinutesBeforeStart = (offset.inSeconds / 60) * -1;
			}
			else {
				e.nsTypes::ReminderMinutesBeforeStart = 0;
			}

		}
		else {
			e.nsTypes::ReminderIsSet = 'false';
		}

		// Save snooze/dismiss state
		this.addSnoozeDismissState(e, aItem, alarmTime);

		// Save Microsoft remindernexttime 
		this.writeEP_DismissSnoozeState(aItem, e);		

		// Save lastLightningModifiedTime
/*		let eprop = <nsTypes:ExtendedProperty xmlns:nsTypes={nsTypes}/>;
		eprop.nsTypes::ExtendedFieldURI.@PropertySetId = calExchangeCalendarGUID;
		eprop.nsTypes::ExtendedFieldURI.@PropertyName = "lastLightningModified";
		eprop.nsTypes::ExtendedFieldURI.@PropertyType = "SystemTime";
		eprop.nsTypes::Value = cal.toRFC3339(cal.now());

		e.appendChild(eprop); */ // Removed because of BUG 26

		if (aItem.getProperty("X-UID")) {
			e.nsTypes::UID = aItem.getProperty("X-UID");
		}
		else {
			if (aItem.id) {
				// This is when we accept and an iTIP
				e.nsTypes::UID = aItem.id;
				aItem.setProperty("X-UID", aItem.id);
				if (aItem.currenceInfo) {
					this.logInfo("we have recurrence info");
					aItem.setProperty("X-CalendarItemType", "RecurringMaster");
					this.recurringMasterCache[aItem.id] = aItem;
				}
				else {
					aItem.setProperty("X-CalendarItemType", "Single");
				}
			}
		}

		if (!this.isInvitation(aItem, true)) {
			// If not timezone specified set them to the lightning preference.
			if ((aItem.startDate.timezone.isFloating) && (!aItem.startDate.isDate)) {
				aItem.startDate = aItem.startDate.getInTimezone(ews.ecDefaultTimeZone());
			}
	
			if ((aItem.endDate.timezone.isFloating) && (!aItem.endDate.isDate)) {
				aItem.endDate = aItem.endDate.getInTimezone(ews.ecDefaultTimeZone());
			}

			var tmpStart = aItem.startDate.clone();
			var tmpEnd = aItem.endDate.clone();

			if (aItem.startDate.isDate) {
				tmpStart.isDate = false;
				tmpEnd.isDate = false;
				var tmpDuration = cal.createDuration();
				tmpDuration.minutes = -1;
				tmpEnd.addDuration(tmpDuration);
	
				// We make a non-UTC datetime value for EWS.
				// EWS will use the MeetingTimeZone or StartTimeZone and EndTimeZone to convert.
				//LOG("  ==== tmpStart:"+cal.toRFC3339(tmpStart));
				e.nsTypes::Start = cal.toRFC3339(tmpStart).substr(0, 19); //cal.toRFC3339(tmpStart).length-6);
				e.nsTypes::End = cal.toRFC3339(tmpEnd).substr(0, 19); //cal.toRFC3339(tmpEnd).length-6);
			}
			else {
				// We set in bias advanced to UCT datetime values for EWS.
				e.nsTypes::Start = cal.toRFC3339(tmpStart);
				e.nsTypes::End = cal.toRFC3339(tmpEnd);
			}

			e.nsTypes::IsAllDayEvent = aItem.startDate.isDate;
	
			const freeBusy = { "TRANSPARENT": "Free",
					  "OPAQUE": "Busy",
					  null: "Busy" };
			e.nsTypes::LegacyFreeBusyStatus = freeBusy[aItem.getProperty("TRANSP")];
	
			e.nsTypes::Location = aItem.getProperty("LOCATION") || "";

			var attendees = aItem.getAttendees({});
			for each (var attendee in attendees) {
				const attendeeStatus = {
					"NEEDS-ACTION"	: "Unknown",
					"TENTATIVE"	: "Tentative",
					"ACCEPTED"	: "Accept",
					"DECLINED"	: "Decline",
					null		: "Unknown"
				};
	
				var ae = <nsTypes:Attendee xmlns:nsTypes={nsTypes}/>;
	
				ae.nsTypes::Mailbox.nsTypes::Name = attendee.commonName;
				ae.nsTypes::Mailbox.nsTypes::EmailAddress = attendee.id.replace(/^mailto:/, '');
				ae.nsTypes::ResponseType = attendeeStatus[attendee.participationStatus];
	
				switch (attendee.role) {
				case "REQ-PARTICIPANT":
					e.nsTypes::RequiredAttendees.nsTypes::Attendee += ae;
					break;
				case "OPT-PARTICIPANT":
					e.nsTypes::OptionalAttendees.nsTypes::Attendee += ae;
					break;
				}
			}

			this.makeRecurrenceRule(aItem, e);
	
			if (this.isVersion2007) {
				e.nsTypes::MeetingTimeZone.@TimeZoneName = this.getEWSTimeZoneId(tmpStart.timezone);
			}
			else {
				e.nsTypes::StartTimeZone.@Id = this.getEWSTimeZoneId(tmpStart.timezone);
				e.nsTypes::EndTimeZone.@Id = this.getEWSTimeZoneId(tmpEnd.timezone);
			}

		}

this.logInfo("convertCalAppointmentToExchangeAppointment: "+String(e));

		return e;
	},

	convertCalTaskToExchangeTask: function _convertCalTaskToExchangeTask(aItem, aAction)
	{
		if (!aAction) {
			aAction = "modify";
		}

		var e = <nsTypes:Task xmlns:nsTypes={nsTypes} xmlns:nsMessages={nsMessages}/>;

		e.nsTypes::Subject = aItem.title;

		const privacies = { "PUBLIC": "Normal",
				"CONFIDENTIAL": "Confidential", 
				"PRIVATE" : "Private",
				null: "Normal" };
		e.nsTypes::Sensitivity = privacies[aItem.privacy];

		e.nsTypes::Body.@BodyType = "Text";
		e.nsTypes::Body = aItem.getProperty('DESCRIPTION') || "";

		var categories = aItem.getCategories({});
		for each (var category in categories) {
			e.nsTypes::Categories.list += <nsTypes:String xmlns:nsTypes={nsTypes}>{category}</nsTypes:String>;
		}
	
		e.nsTypes::Importance = "Normal";
		if (aItem.priority > 5) {
			e.nsTypes::Importance = "Low";
		}
		if (aItem.priority == 5) {
			e.nsTypes::Importance = "Normal";
		}
		if (aItem.priority < 5) {
			e.nsTypes::Importance = "High";
		}
		if (aItem.priority == 0) {
			e.nsTypes::Importance = "Normal";
		}
		 

	/*	Not allowed to set by EWS
		if (aItem.entryDate) {
			e.nsTypes::DateTimeCreated = cal.toRFC3339(aItem.entryDate);
		}
			*/

		var alarmTime = this.getAlarmTime(aItem);
		if (alarmTime) {
			this.logInfo("alarmTime = "+alarmTime.toString());

			var alarm = aItem.getAlarms({})[0];
			switch (alarm.related) {
			case Ci.calIAlarm.ALARM_RELATED_ABSOLUTE:
				var newAlarmTime = alarm.alarmDate;
				var offset = cal.createDuration();
				offset.inSeconds = 0;
				break;
			case Ci.calIAlarm.ALARM_RELATED_START:
				var newAlarmTime = aItem.startDate;
				var offset = alarm.offset.clone();
				break;
			case Ci.calIAlarm.ALARM_RELATED_END:
				var newAlarmTime = aItem.endDate;
				var offset = alarm.offset.clone();
				break;
			}
	
			e.nsTypes::ReminderDueBy = cal.toRFC3339(newAlarmTime);
			e.nsTypes::ReminderIsSet = 'true';
			if (offset.inSeconds != 0) {
				e.nsTypes::ReminderMinutesBeforeStart = (offset.inSeconds / 60) * -1;
			}
			else {
				e.nsTypes::ReminderMinutesBeforeStart = 0;
			}
		}
		else {
			e.nsTypes::ReminderIsSet = 'false';
		}

		if (aItem.completedDate) {

			tmpStart = aItem.completedDate.clone();

			if (tmpStart.timezone.isFloating) {
				tmpStart = tmpStart.getInTimezone(ews.ecDefaultTimeZone());
			}

			tmpStart = tmpStart.getInTimezone(ews.ecUTC());

			tmpStart.isDate = true;
			tmpStart.isDate = false;
		/*	var tmpDuration = cal.createDuration();
			tmpDuration.minutes = -1;
			tmpStart.addDuration(tmpDuration);*/

			e.nsTypes::CompleteDate = cal.toRFC3339(tmpStart);
		}
		if (aItem.dueDate) {
			if (aItem.dueDate.timezone.isFloating) {
				aItem.dueDate = aItem.dueDate.getInTimezone(ews.ecDefaultTimeZone());
			}

			e.nsTypes::DueDate = cal.toRFC3339(aItem.dueDate);
		}
		if (aItem.percentComplete) {
			e.nsTypes::PercentComplete = aItem.percentComplete;
		}


		this.makeRecurrenceRule(aItem, e);


		if (aItem.entryDate) {
			if (aItem.entryDate.timezone.isFloating) {
				aItem.entryDate = aItem.entryDate.getInTimezone(ews.ecDefaultTimeZone());
			}

			e.nsTypes::StartDate = cal.toRFC3339(aItem.entryDate);
		}

		const statuses = { "NONE": "NotStarted",
				"IN-PROCESS": "InProgress", 
				"COMPLETED" : "Completed",
				"NEEDS-ACTION" : "WaitingOnOthers",
				"CANCELLED" : "Deferred",
				null: "NotStarted" };
		e.nsTypes::Status = statuses[aItem.status];

//this.logInfo("CHANGED:"+String(e));

		return e;
	},

	createItemOk: function _createItemOk(erCreateItemRequest, aId, aChangeKey)
	{
		this.saveCredentials(erCreateItemRequest.argument);
		this.logInfo("createItemOk");

		//var newItem = erCreateItemRequest.argument.item.clone();
		var newItem = this.cloneItem(erCreateItemRequest.argument.item);
		newItem.id = aId;
		newItem.setProperty("X-ChangeKey", aChangeKey);
		if ((!newItem.recurrenceInfo) || ((newItem.parentItem) && (newItem.parentItem.id != newItem.id))) {
			this.itemCache[newItem.id] = newItem;
		}

		if (erCreateItemRequest.listener) {
			this.notifyOperationComplete(erCreateItemRequest.listener,
			      Cr.NS_OK,
			      Ci.calIOperationListener.ADD,
			      erCreateItemRequest.argument.item.id,
			      erCreateItemRequest.argument.item);
		}

		// Make an event for thistory.
		if (isEvent(newItem)) {
			if ((!newItem.recurrenceInfo) || (newItem.parentItem != newItem)) {
				//this.observers.notify("onAddItem", [newItem]);
			}
			this.addActivity(calGetString("calExchangeCalendar", "addCalendarEventMessage", [newItem.title, this.name], "exchangecalendar"), "", erCreateItemRequest.argument.actionStart, Date.now());
		}
		else {
			this.observers.notify("onAddItem", [newItem]);
			this.addActivity(calGetString("calExchangeCalendar", "addTaskEventMessage", [newItem.title, this.name], "exchangecalendar"), "", erCreateItemRequest.argument.actionStart, Date.now());
		}

		// We do a refresh to get all details of the new item which EWS added.
		this.refresh();
	},

	makeUpdateOneItem: function _makeUpdateOneItem(aNewItem, aOldItem, aIndex, aMasterId, aMasterChangeKey)
	{
		var upd = <nsTypes:ItemChange xmlns:nsTypes={nsTypes}/>;

		if (!aIndex) {
			upd.ItemId = <nsTypes:ItemId Id={aOldItem.id} ChangeKey={aOldItem.getProperty("X-ChangeKey")} xmlns:nsTypes={nsTypes}/>;
		}
		else {
			upd.OccurrenceItemId = <nsTypes:OccurrenceItemId RecurringMasterId={aMasterId} ChangeKey={aMasterChangeKey} InstanceIndex={aIndex} xmlns:nsTypes={nsTypes}/>;
		}
	
		const noDelete = {
			'MeetingTimeZone'		: true,
			'ReminderIsSet'			: true,
			'ReminderMinutesBeforeStart'	: true,
			'ReminderDueBy'			: true,
			'PercentComplete'		: true
		};
	
		var ce = new XMLList();

		if (isEvent(aOldItem)) {
			var oe = this.convertCalAppointmentToExchangeAppointment(aOldItem, null, false);
			var ne = this.convertCalAppointmentToExchangeAppointment(aNewItem, null, false);
		}
		if (isToDo(aOldItem)) {
			var oe = this.convertCalTaskToExchangeTask(aOldItem);
			var ne = this.convertCalTaskToExchangeTask(aNewItem);
		}
	
		for each (var prop in oe) {
			if (ne[prop.name()].length() > 0 || noDelete[prop.localName()]) {
				continue;
			}
	
			var de = <nsTypes:DeleteItemField xmlns:nsTypes={nsTypes}/>;
			if (prop.localName() == "ExtendedProperty") {
				de.nsTypes::ExtendedFieldURI = prop.nsTypes::ExtendedFieldURI;
			} else {
				de.nsTypes::FieldURI.@FieldURI = fieldPathMap[prop.localName()] + ':' + prop.localName();
			}
	
			ce += de;
		}

		for each (var prop in ne) {

			// Always save lastLightningModified field
			var doSave = false;
			if ((prop.localName() == "ExtendedProperty") && (prop.nsTypes::ExtendedFieldURI.@PropertyName == "lastLightningModified")) {
				doSave = true;
			}

			if (! doSave) {
				if (oe.children().contains(prop)) {
					continue;
				}
			}
	
			var se = <nsTypes:SetItemField xmlns:nsTypes={nsTypes}/>;
			if (prop.localName() == "ExtendedProperty") {
				se.nsTypes::ExtendedFieldURI = prop.nsTypes::ExtendedFieldURI;
			} else {
				se.nsTypes::FieldURI.@FieldURI = fieldPathMap[prop.localName()] + ':' + prop.localName();
			}
			if (isEvent(aOldItem)) {
				se.nsTypes::CalendarItem.content = prop;
			}
			if (isToDo(aOldItem)) {
				se.nsTypes::Task.content = prop;
			}
	
			ce += se;
		}
	
		upd.nsTypes::Updates.content = ce;
	
		if (!ce[0]) {
			return null;
		}
		return upd;
	},

	sendMeetingRespons: function _sendMeetingRespons(aItem, aListener, aItemType, aResponse, aBodyText)
	{
		// Check if I'm the organiser. Do not send to myself.
		if (aItem.organizer) {
			if (aItem.organizer.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase()) {
				return true;
			}
		}

		var me = this.getInvitedAttendee(aItem);
		if ((!me) && (!aResponse)) {
			return false;
		}

		if (aResponse) {
			var tmpResponse = aResponse;
		}
		else {
			var tmpResponse = me.participationStatus;
		}

		var input= { item: aItem, 
			     response: tmpResponse,
			     answer: "",
			     bodyText: ""};

		if (aBodyText) {
			input.bodyText = aBodyText;
			input.answer = "send";
		}

		// Get the eventsummarywindow to attach dialog to.
		let wm = Cc["@mozilla.org/appshell/window-mediator;1"]
	                          .getService(Ci.nsIWindowMediator);
		let calWindow = wm.getMostRecentWindow("Calendar:EventSummaryDialog") || cal.getCalendarWindow();

		if ((calWindow) && (!aBodyText)) {
			calWindow.openDialog("chrome://exchangecalendar/content/invitationResponse.xul",
				"invitationResponseId",
				"chrome,titlebar,toolbar,centerscreen,dialog,modal=yes,resizable=yes",
				input); 
		}

		if (input.answer == "send") {
				
			var self = this;
			this.addToQueue( erSendMeetingResponsRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 folderBase: this.folderBase,
				 serverUrl: this.serverUrl,
				 item: aItem,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 response: input.response,
				 bodyText: input.bodyText,
				 senderMailbox: this.mailbox,
		 		 actionStart: Date.now(),
				 itemType: aItemType}, 
				function(erSendMeetingResponsRequest) { self.sendMeetingResponsOk(erSendMeetingResponsRequest);}, 
				function(erSendMeetingResponsRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erSendMeetingResponsRequest, aCode, aMsg);},
				aListener,
				2);
			return true;
		}
		else {
			this.logInfo("sendMeetingRespons: canceled by user.");
			return false;
		}
	},

	sendMeetingResponsOk: function _sendMeetingResponsOk(erSendMeetingResponsRequest)
	{
		this.logInfo("sendMeetingResponsOk");
		this.saveCredentials(erSendMeetingResponsRequest.argument);

		if (erSendMeetingResponsRequest.listener) {
		        this.notifyOperationComplete(erSendMeetingResponsRequest.listener,
		                                     Cr.NS_OK,
		                                     Ci.calIOperationListener.MODIFY,
		                                     erSendMeetingResponsRequest.argument.item.id,
		                                     erSendMeetingResponsRequest.argument.item);
		}
		
		if ( !erSendMeetingResponsRequest.argument.item.getProperty("X-MEETINGREQUEST")) {
			//this.observers.notify("onModifyItem", [erSendMeetingResponsRequest.argument.item, this.itemCache[erSendMeetingResponsRequest.argument.item.id]]);
			this.itemCache[erSendMeetingResponsRequest.argument.item.id] = erSendMeetingResponsRequest.argument.item;
		}
		else {
			// From meetingrequest cache. Do not remove meetingrequest from cache this will be done when invitation is removed
			//this.removeFromMeetingRequestCache(erSendMeetingResponsRequest.argument.item.getProperty("X-UID"));
			//this.meetingrequestAnswered[erSendMeetingResponsRequest.argument.item.getProperty("X-UID")] = false;
		}

		this.addActivity(calGetString("calExchangeCalendar", "ewsMeetingResponsEventMessage", [erSendMeetingResponsRequest.argument.item.title, erSendMeetingResponsRequest.argument.response, this.name], "exchangecalendar"), erSendMeetingResponsRequest.argument.bodyText, erSendMeetingResponsRequest.argument.actionStart, Date.now());
		this.refresh();
	},

	modifyItemgetOccurrenceIndexOk: function _modifyItemgetOccurrenceIndexOk(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey)
	{
		this.logInfo("modifyItemgetOccurrenceIndexOk");
		this.saveCredentials(erGetOccurrenceIndexRequest.argument);

		var changes = this.makeUpdateOneItem(erGetOccurrenceIndexRequest.argument.newItem, erGetOccurrenceIndexRequest.argument.masterItem, aIndex, aMasterId, aMasterChangeKey);
		if (changes) {

			// We remove the item from cache and calendar because the update request will add
			// it again.

			// bug 13.o when we change an occurrence it will send an updated master
		//	this.observers.notify("onDeleteItem", [this.itemCache[erGetOccurrenceIndexRequest.argument.newItem.id]]);
		//	delete this.itemCache[erGetOccurrenceIndexRequest.argument.newItem.id];
			this.observers.notify("onModifyItem", [erGetOccurrenceIndexRequest.argument.newItem, this.itemCache[erGetOccurrenceIndexRequest.argument.newItem.id]]);
			this.itemCache[erGetOccurrenceIndexRequest.argument.newItem.id] = erGetOccurrenceIndexRequest.argument.newItem;


			var self = this;
			this.logInfo("modifyItemgetOccurrenceIndexOk: changed:"+String(changes));
			this.addToQueue( erUpdateItemRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 folderBase: this.folderBase,
				 serverUrl: this.serverUrl,
				 item: erGetOccurrenceIndexRequest.argument.masterItem,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 updateReq: changes,
				 newItem: erGetOccurrenceIndexRequest.argument.newItem,
				 actionStart: Date.now(),
			         sendto: erGetOccurrenceIndexRequest.argument.sendto}, 
				function(erUpdateItemRequest, aId, aChangeKey) { self.updateItemOk(erUpdateItemRequest, aId, aChangeKey);}, 
				function(erUpdateItemRequest, aCode, aMsg) { self.whichOccurrencegetOccurrenceIndexError(erUpdateItemRequest, aCode, aMsg);},
				erGetOccurrenceIndexRequest.listener,
				2);
			
		}
		else {
			this.logInfo("modifyItem: No changes 3.");
			if (!this.isInvitation(erGetOccurrenceIndexRequest.argument.newItem, true)) {
				this.singleModified(erGetOccurrenceIndexRequest.argument.newItem, true);
			}
		}

	},

	whichOccurrencegetOccurrenceIndexError: function _whichOccurrencegetOccurrenceIndexError(erGetOccurrenceIndexRequest, aCode, aMsg)
	{
		this.logInfo("whichOccurrencegetOccurrenceIndexError:("+aCode+")"+aMsg);
		this.saveCredentials(erGetOccurrenceIndexRequest.argument);

		// TODO: We need to retry when we get here from adoptItem.
//		if ((aCode == -8) && (this.itemCache[erGetOccurrenceIndexRequest.argument.item.id])) {
		if ((aCode == erGetOccurrenceIndexRequest.ER_ERROR_ER_ERROR_SOAP_ERROR) && (this.itemCache[erGetOccurrenceIndexRequest.argument.item.id])) {
			// Probably the item on the EWS server was changed and that
			// update was not received by us yet.
			// Do a refresh en retry modification.
			this.logInfo("We have a conflict with the server for this update. We are going to refresh and then retry.");
			this.refresh();
			//var newItem = erGetOccurrenceIndexRequest.argument.newItem.clone();
			var newItem = this.cloneItem(erGetOccurrenceIndexRequest.argument.newItem);
			//var oldItem = this.itemCache[erGetOccurrenceIndexRequest.argument.item.id].clone();
			var oldItem = this.cloneItem(this.itemCache[erGetOccurrenceIndexRequest.argument.item.id]);
			if (erGetOccurrenceIndexRequest.argument.item.getProperty("X-RetryCount")) {
				oldItem.setProperty("X-RetryCount", erGetOccurrenceIndexRequest.argument.item.getProperty("X-RetryCount")+1);
			}
			else {
				oldItem.setProperty("X-RetryCount", 1);
			}

			this.logInfo("X-RetryCount="+oldItem.getProperty("X-RetryCount"));

			if (oldItem.getProperty("X-RetryCount") < 25) {
				this.modifyItem(newItem, oldItem, erGetOccurrenceIndexRequest.listener);
			}
			else {
				this.logInfo("To many update retries. Giving up.");
				if (erGetOccurrenceIndexRequest.listener) {
				        this.notifyOperationComplete(erGetOccurrenceIndexRequest.listener,
				                                     Ci.calIErrors.READ_FAILED,
				                                     Ci.calIOperationListener.MODIFY,
				                                     erGetOccurrenceIndexRequest.argument.newItem.id,
				                                     erGetOccurrenceIndexRequest.argument.newItem);
				}
			}
		}
		else {
			this.addActivity(calGetString("calExchangeCalendar", "ewsErrorEventMessage", [this.name, aMsg, aCode], "exchangecalendar"), aMsg, erGetOccurrenceIndexRequest.argument.actionStart, Date.now());

			var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);
			if (aMsg.indexOf(":") > -1) {
				var msgStr = aMsg.substr(aMsg.indexOf(":")+1);
				switch (msgStr) {
					case "ErrorCalendarMeetingRequestIsOutOfDate" :
						msgStr = "Meeting request is Out Of Date. You cannot use it anymore. ("+msgStr+")";
						break;
					case "ErrorItemNotFound" :
						msgStr = "Calendar item is not available anymore in the exchange database. ("+msgStr+")";
						break;
				}
				promptService.alert(null, "Info", msgStr);
			}
			else {
				promptService.alert(null, "Info", aMsg);
			}

			if (erGetOccurrenceIndexRequest.listener) {
			        this.notifyOperationComplete(erGetOccurrenceIndexRequest.listener,
			                                     Ci.calIErrors.MODIFICATION_FAILED,
			                                     Ci.calIOperationListener.MODIFY,
			                                     erGetOccurrenceIndexRequest.argument.newItem.id,
			                                     erGetOccurrenceIndexRequest.argument.newItem);
			}
		}
	},

	updateItemOk: function _updateItemOk(erUpdateItemRequest, aId, aChangeKey)
	{
		this.logInfo("updateItemOk");

		this.saveCredentials(erUpdateItemRequest.argument);

		//var newItem = erUpdateItemRequest.argument.newItem.clone();
 		var newItem = this.cloneItem(erUpdateItemRequest.argument.newItem);

		// Make an event for thistory.
		if (isEvent(newItem)) {
			this.addActivity(calGetString("calExchangeCalendar", "updateCalendarEventMessage", [newItem.title, this.name], "exchangecalendar"), "", erUpdateItemRequest.argument.actionStart, Date.now());
		}
		else {
			this.addActivity(calGetString("calExchangeCalendar", "updateTaskEventMessage", [newItem.title, this.name], "exchangecalendar"), "", erUpdateItemRequest.argument.actionStart, Date.now());
		}

		if (isEvent(newItem)) {
			newItem.setProperty("X-ChangeKey", aChangeKey);
			newItem.setProperty("X-UID", erUpdateItemRequest.argument.item.getProperty("X-UID"));
			newItem.setProperty("X-CalendarItemType", erUpdateItemRequest.argument.item.getProperty("X-CalendarItemType"));
		
			if (aId != newItem.id) {
				this.logInfo("updateItemOk: WHY DID THE ID CHANGE !!!");
			}

			if (erUpdateItemRequest.argument.item.id != erUpdateItemRequest.argument.item.parentItem.id) {
				//var master = erUpdateItemRequest.argument.item.parentItem.clone();
				var master = this.cloneItem(erUpdateItemRequest.argument.item.parentItem);
				newItem.parentItem = master;
				master.recurrenceInfo.modifyException(newItem, false);
	
				// Update our cache
				this.itemCache[erUpdateItemRequest.argument.item.id] = newItem;
			}
			else {
				if (erUpdateItemRequest.argument.item.getProperty("X-CalendarItemType") == "Single") {
					// Update our cache
					this.itemCache[erUpdateItemRequest.argument.item.id] = newItem;
				}
				else {
					// Update our cache
					// TODO: Do we need to update our children. If yes how..!!
					var oldMaster = this.recurringMasterCache[erUpdateItemRequest.argument.item.getProperty("X-UID")];
					this.recurringMasterCache[erUpdateItemRequest.argument.item.getProperty("X-UID")] = newItem;
					//oldMaster.clearAlarms();
					//x this.deleteAlarms(oldMaster);
					//this.observers.notify("onDeleteItem", [oldMaster]);
					// For now we do a refresh.
					this.refresh();
					if (erUpdateItemRequest.listener) {
					        this.notifyOperationComplete(erUpdateItemRequest.listener,
					                                     Cr.NS_OK,
					                                     Ci.calIOperationListener.MODIFY,
					                                     newItem.id,
					                                     newItem);
					}
					return;
				}
			}
		}
		if (isToDo(newItem)) {
			this.itemCache[erUpdateItemRequest.argument.item.id] = newItem;
		}

		if (erUpdateItemRequest.listener) {
		        this.notifyOperationComplete(erUpdateItemRequest.listener,
		                                     Cr.NS_OK,
		                                     Ci.calIOperationListener.MODIFY,
		                                     newItem.id,
		                                     newItem);
		}
		
		//erUpdateItemRequest.argument.item.clearAlarms();
		//this.observers.notify("onDeleteItem", [erUpdateItemRequest.argument.item]);
        	//this.observers.notify("onAddItem", [newItem]);
		this.refresh();
	},

	getOccurrenceIndexOk: function _getOccurrenceIndexOk(erGetOccurrenceIndexRequest, aIndex, aMasterId, aMasterChangeKey)
	{
//		this.logInfo("getOccurrenceIndexOk index="+aIndex);
		this.saveCredentials(erGetOccurrenceIndexRequest.argument);
		var self = this;
		switch (erGetOccurrenceIndexRequest.argument.action) {
			case "deleteItem":
				this.addToQueue( erDeleteItemRequest,
					{user: erGetOccurrenceIndexRequest.argument.user, 
					 mailbox: this.mailbox,
					 folderBase: this.folderBase,
					 serverUrl: erGetOccurrenceIndexRequest.argument.serverUrl,
					 item: erGetOccurrenceIndexRequest.argument.item,
					 folderID: erGetOccurrenceIndexRequest.argument.folderID,
					 masterID: aMasterId,
					 masterChangeKey: aMasterChangeKey,
					 itemIndex: aIndex,
					 itemType: erGetOccurrenceIndexRequest.argument.itemType,
					 whichOccurrence: erGetOccurrenceIndexRequest.argument.whichOccurrence}, 
					function(erDeleteItemRequest) { self.deleteItemOk(erDeleteItemRequest);},
					function(erDeleteItemRequest, aCode, aMsg) { self.deleteItemError(erDeleteItemRequest, aCode, aMsg);},
					erGetOccurrenceIndexRequest.listener,
					2);
				break;
		}
	},

	getOccurrenceIndexError: function _getOccurrenceIndexError(erGetOccurrenceIndexRequest, aCode, aMsg)
	{
//		this.logInfo("getOccurrenceIndexError");
		this.saveCredentials(erGetOccurrenceIndexRequest.argument);
		switch (erGetOccurrenceIndexRequest.argument.action) {
			case "deleteItem":
				if (erGetOccurrenceIndexRequest.listener) {
					this.notifyOperationComplete(erGetOccurrenceIndexRequest.listener,
					      Ci.calIErrors.MODIFICATION_FAILED,
					      Ci.calIOperationListener.DELETE,
					      null,
					      aMsg);
				}
				break;
		}
	},

	deleteItemOk: function _deleteItemOk(erDeleteItemRequest)
	{
		this.saveCredentials(erDeleteItemRequest.argument);
		this.logInfo("deleteItemOK");

		if (erDeleteItemRequest.listener) {
			this.notifyOperationComplete(erDeleteItemRequest.listener,
			      Cr.NS_OK,
			      Ci.calIOperationListener.DELETE,
			      erDeleteItemRequest.argument.item.id,
			      erDeleteItemRequest.argument.item);
		}

		this.logInfo("itemType:"+erDeleteItemRequest.itemType+", Subject:"+erDeleteItemRequest.argument.item.title);
		switch (erDeleteItemRequest.itemType) {
			case "master" :
				// Also remove the children.
				// 12-12-2011 Turned this of because this is done when we receive the update from the exchange server.
				//this.removeChildrenFromMaster(erDeleteItemRequest.argument.item);
				//delete this.recurringMasterCache[erDeleteItemRequest.argument.item.getProperty("X-UID")];
				break;
			case "single":
			case "occurrence":
				//erDeleteItemRequest.argument.item.clearAlarms();

			// This will be done on receiving the update from EWS.
				//this.deleteAlarms(erDeleteItemRequest.argument.item);
				//this.observers.notify("onDeleteItem", [erDeleteItemRequest.argument.item]);
				//delete this.itemCache[erDeleteItemRequest.argument.item.id];
				break;
		}


		if (isEvent(erDeleteItemRequest.argument.item)) {
			this.addActivity(calGetString("calExchangeCalendar", "deleteCalendarEventMessage", [erDeleteItemRequest.argument.item.title, this.name], "exchangecalendar"), "", erDeleteItemRequest.argument.actionStart, Date.now());
		}
		else {
			this.addActivity(calGetString("calExchangeCalendar", "deleteTaskEventMessage", [erDeleteItemRequest.argument.item.title, this.name], "exchangecalendar"), "", erDeleteItemRequest.argument.actionStart, Date.now());
		}

		//delete erDeleteItemRequest.argument.item;
		this.refresh();
	},

	deleteItemError: function _deleteItemError(erDeleteItemRequest, aCode, aMsg)
	{
		this.saveCredentials(erDeleteItemRequest.argument);
		this.logInfo("deleteItemError msg:"+String(aMsg));
		if (erDeleteItemRequest.listener) {
			this.notifyOperationComplete(erDeleteItemRequest.listener,
			      Ci.calIErrors.MODIFICATION_FAILED,
			      Ci.calIOperationListener.DELETE,
			      null,
			      aMsg);
		}
	},

	saveCredentials: function _saveCredentials(aCredentials)
	{
		if (aCredentials) {
			if ((aCredentials.user != "") && (aCredentials.user != "\\") && (aCredentials.user != "/")) {
				this.user = aCredentials.user;
			}
		}
	},

	addToQueue: function _addToQueue(aRequest, aArgument, aCbOk, aCbError, aListener, aQueueNumber)
	{
		if (!aQueueNumber) {
			aQueueNumber = 1;
		}

		if (!this.queue[aQueueNumber]) {
			this.queue[aQueueNumber] = [];
		}

		this.queue[aQueueNumber].push({ecRequest:aRequest,
				 arguments: aArgument,
				 cbOk: aCbOk,
				 cbError: aCbError,
				 listener: aListener});

		this.observerService.notifyObservers(this, "onExchangeProgressChange", "1");  


		if (!this.timers[aQueueNumber]) {
			this.logInfo("Arming timer for queue:"+aQueueNumber);
			this.timers[aQueueNumber] = Cc["@mozilla.org/timer;1"]
					.createInstance(Ci.nsITimer);

		        let self = this;
			let timerCallback = {
				notify: function setTimeout_notify() {
					self.processQueue(aQueueNumber);
				}
			};
			if (!this.shutdown) {
				this.timers[aQueueNumber].initWithCallback(timerCallback, 50, this.timers[aQueueNumber].TYPE_REPEATING_SLACK);
			}
			this.logInfo("Timer for for queue:"+aQueueNumber);

		}
	},

	processQueue: function _processQueue(aQueueNumber)
	{
		if (this.tmpJobs[aQueueNumber]) {
			if (this.tmpJobs[aQueueNumber].isRunning) {
				return;
			}
		}

		if (this.doReset) {
			if (this.weAreSyncing) {
				return;
			}

			this.performReset();
			this.logInfo("Reset done.");
		}

		if (this.queue[aQueueNumber].length > 0) {
			var queueItem = this.queue[aQueueNumber][0];
			this.queue[aQueueNumber].shift();

			this.observerService.notifyObservers(this, "onExchangeProgressChange", "-1");  

			queueItem.arguments["ServerVersion"] = getEWSServerVersion(this.serverUrl);
			this.tmpJobs[aQueueNumber] = new queueItem.ecRequest(queueItem.arguments, queueItem.cbOk, queueItem.cbError, queueItem.listener);
		}

	},

	findCalendarItemsOK: function _findCalendarItemsOK(erFindCalendarItemsRequest, aIds, aOccurrences)
	{
		//this.logInfo("findCalendarItemsOK: aIds.length="+aIds.length+", aOccurrences.length="+aOccurrences.length);

		this.saveCredentials(erFindCalendarItemsRequest.argument);

		if ((aIds.length == 0) && (aOccurrences.length)) {
			return;
		}

       		var self = this;

		// If we have occurrences and/or exceptions. Find the masters. followed by the occurrences.
		if (aOccurrences.length > 0) {
			this.logInfo("findCalendarItemsOK: aOccurrences.length="+aOccurrences.length);
			this.addToQueue( erFindMasterOccurrencesRequest, 
			{user: this.user, 
			 mailbox: this.mailbox,
			 folderBase: this.folderBase,
			 serverUrl: this.serverUrl,
			 occurrences: aOccurrences,
			 folderID: this.folderID,
			 changeKey: this.changeKey,
			 GUID: calExchangeCalendarGUID}, 
			function(erGetItemsRequest, aIds) { self.findMasterOccurrencesOk(erGetItemsRequest, aIds);}, 
			function(erGetItemsRequest, aCode, aMsg) { self.findCalendarItemsError(erGetItemsRequest, aCode, aMsg);},
			null);

		}

		// We have single and/or master items. Get full details and cache them.
		if (aIds.length > 0) {
			this.addToQueue( erGetItemsRequest, 
			{user: this.user, 
			 mailbox: this.mailbox,
			 folderBase: this.folderBase,
			 serverUrl: this.serverUrl,
			 ids: aIds,
			 folderID: this.folderID,
			 changeKey: this.changeKey,
			 occurrences: aOccurrences,
			 GUID: calExchangeCalendarGUID}, 
			function(erGetItemsRequest, aIds) { self.getCalendarItemsOK(erGetItemsRequest, aIds);}, 
			function(erGetItemsRequest, aCode, aMsg) { self.findCalendarItemsError(erGetItemsRequest, aCode, aMsg);},
			null);
		}

		if (!this.calendarPoller) {

			// start the calendar poller
			this.calendarPoller = Cc["@mozilla.org/timer;1"]
					.createInstance(Ci.nsITimer);
		        var self = this;
			var timerCallback = {
				notify: function setTimeout_notify() {
					self.refresh();
				}
			};

			this.calendarPoller.initWithCallback(timerCallback, ews.safeGetIntPref(this.prefs, "ecCalendarPollInterval", 10) * 1000, this.calendarPoller.TYPE_REPEATING_SLACK);
		}
	},

	findCalendarItemsError: function _findCalendarItemsError(erFindCalendarItemsRequest, aCode, aMsg) 
	{
		this.logInfo("findCalendarItemsError aCode:"+aCode+", aMsg:"+aMsg);
		if ((aCode == -8) && (aMsg == "ErrorCalendarFolderIsInvalidForCalendarView")) {
			this.supportsEvents = false;
		}
		this.saveCredentials(erFindCalendarItemsRequest.argument);
	},

	findMasterOccurrencesOk: function _findMasterOccurrencesOk(erGetItemsRequest, aIds)
	{
		this.logInfo("findMasterOccurrencesOk:Start: aIds.length="+aIds.length);

		this.saveCredentials(erFindCalendarItemsRequest.argument);

		if (aIds.length == 0) {
			return;
		}

		// Cache full master details.
		this.updateCalendar(erGetItemsRequest, aIds);
		this.logInfo("findMasterOccurrencesOk:End: aIds.length="+aIds.length);

	},

	findOccurrencesOK: function _findOccurrencesOK(erFindOccurrencesRequest, aIds)
	{
//		this.logInfo("findOccurrencesOK: aIds.length="+aIds.length);
		// Get full details of occurrences and exceptions and cache them.
		this.findCalendarItemsOK(erFindOccurrencesRequest, aIds, []);
	},

	findTaskItemsOK: function _findTaskItemsOK(erFindTaskItemsRequest, aIds)
	{
//		this.logInfo("findTaskItemsOK: aIds.length:"+aIds.length);
		this.saveCredentials(erFindTaskItemsRequest.argument);

		if (aIds.length == 0) {
			return;
		}

       		var self = this;

		this.addToQueue( erGetItemsRequest, 
			{user: this.user, 
			 mailbox: this.mailbox,
			 folderBase: this.folderBase,
			 serverUrl: this.serverUrl,
			 ids: aIds,
			 folderID: this.folderID,
			 changeKey: this.changeKey,
			 GUID: calExchangeCalendarGUID }, 
			function(erGetItemsRequest, aIds) { self.getTaskItemsOK(erGetItemsRequest, aIds);}, 
			function(erGetItemsRequest, aCode, aMsg) { self.getTaskItemsError(erGetItemsRequest, aCode, aMsg);},
			null);
	},

	findTaskItemsError: function _findTaskItemsError(erFindTaskItemsRequest, aCode, aMsg)
	{
		this.saveCredentials(erFindTaskItemsRequest.argument);

	},

	tryToSetDateValue: function _TryToSetDateValue(ewsvalue, aDefault)
	{
		if ((ewsvalue) && (ewsvalue.toString().length)) {
			return cal.fromRFC3339(ewsvalue, ews.ecTZService().UTC).getInTimezone(ews.ecDefaultTimeZone());
		}

		return aDefault;
	},

	tryToSetValue: function _tryToSetValue(ewsvalue, aDefault)
	{
		if (ewsvalue) {
			return ewsvalue;
		}

		return aDefault;
	},

	processItemSyncQueue: function _processItemSyncQueue()
	{
		if (this.processItemSyncQueueBusy) {
			return;
		}

		this.processItemSyncQueueBusy = true;

		// Process this.getItemSyncQueue
		while (this.getItemSyncQueue.length > 0) {
			this.logInfo("processItemSyncQueue: id:"+this.getItemSyncQueue[0].id);
			this.getItem(this.getItemSyncQueue[0].id, this.getItemSyncQueue[0].listener, true);
			this.getItemSyncQueue.shift();
		}

		this.processItemSyncQueueBusy = false;

	},

	getTaskItemsOK: function _getTaskItemsOK(erGetItemsRequest, aItems)
	{
		this.logInfo("getTaskItemsOK: aItems.length:"+aItems.length);
		this.saveCredentials(erGetItemsRequest.argument);


		if (aItems.length == 0) {
			this.syncBusy = false;
			this.processItemSyncQueue();

			return;
		}

		this.updateCalendar(erGetItemsRequest, aItems);

		this.syncBusy = false;

		this.processItemSyncQueue();

	},

	getTaskItemsError: function _getTaskItemsError(erGetItemsRequest, aCode, aMsg) 
	{
//		this.logInfo("getTaskItemsError: aMsg:"+ aMsg);
		this.saveCredentials(erGetItemsRequest.argument);

		this.syncBusy = false;

		this.processItemSyncQueue();

	},

	getMeetingRequestFromServer: function _getMeetingRequestFromServer(aItem, aUID, aOperation, aListener)
	{
		this.logInfo("aUID="+aUID);

		// We do not have a meetingrequest in cache but it is an iTIP.
		// Is inbox polling off?
		if (!this.doPollInbox) {
			// We are not polling the inbox so treat it as a meetingrespons.
			this.logInfo("We get a new calendar item. It looks like an iTIP respons because aItem.id is set and we are not polling the inbox.");
			// We mis the exchange id and changekey which we need for responding.
			// We can get these by polling or searching the inbox but this is async. We can do this
			// because we received a listener. So search inbox for this uid and then use the full exchange message.
			// We could have multiple meetingrequest with the same uid but different id and changekey. How do we handle those??
						
			var self = this;
			this.addToQueue( erGetMeetingRequestByUIDRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 folderBase: "inbox",
				 serverUrl: this.serverUrl,
				 item: aItem,
				 uid: aUID,
				 actionStart: Date.now(),
				 operation: aOperation,
				 GUID: calExchangeCalendarGUID}, 
				function(erGetMeetingRequestByUIDRequest, aMeetingRequests) { self.getMeetingRequestByUIDOk(erGetMeetingRequestByUIDRequest, aMeetingRequests);}, 
				function(erGetMeetingRequestByUIDRequest, aCode, aMsg) { self.getMeetingRequestByUIDError(erGetMeetingRequestByUIDRequest, aCode, aMsg);},
				aListener,
				1);

			return;
		}
		else {
			this.logInfo("!! THIS IS A SITUATION WHICH NEVER SHOULD HAPPEN !!");
			this.logInfo("We have an iTIP respons and are polling the inbox but the item is not in the meetingrequestcache.");
			this.logInfo("We see this when someone imports an ICS file.");
			aItem.id = undefined;
			this.adoptItem(aItem, aListener);
		/*	// We could do the same as we are not polling inbox..
			if (aListener) {
				this.notifyOperationComplete(aListener,
	        	                             Ci.calIErrors.MODIFICATION_FAILED,
	                        	             aOperation,
	                        	             aUID,
	                        	             aItem);
			} */
			return;
		}
	},

	createAttendee: function _createAttendee(aElement, aType, aMyResponseType) 
	{
		let mbox = aElement.nsTypes::Mailbox;
		let attendee = createAttendee();

		if (!aType) {
			aType = "REQ-PARTICIPANT";
		}

		attendee.id = 'mailto:' + mbox.nsTypes::EmailAddress.toString();
		attendee.commonName = mbox.nsTypes::Name.toString();
		attendee.rsvp = "FALSE";
		attendee.userType = "INDIVIDUAL";
		attendee.role = aType;

		if (aElement.nsTypes::ResponseType.length() > 0) {
			attendee.participationStatus = participationMap[aElement.nsTypes::ResponseType.toString()];

			// check if we specified a myResponseType for the complete item and the specified mailbox is equal to the mailbox for the calendar.
			//this.logInfo("aMyResponseType:"+aMyResponseType+", EmailAddress:"+mbox.nsTypes::EmailAddress.toString().toLowerCase()+", mailbox:"+this.mailbox.toLowerCase());
			if ((aMyResponseType) && (mbox.nsTypes::EmailAddress.toString().toLowerCase() == this.mailbox.toLowerCase())) {
				attendee.participationStatus = participationMap[aMyResponseType];
				//this.logInfo("Setting my response type from the global myresponsetype for the item.");
			}
		}

		return attendee;
	},

	createMeAsAttendee: function _createMeAsAttendee(aMyResponseType) 
	{
		var attendee = createAttendee();

		attendee.id = 'mailto:' + this.mailbox;
		attendee.commonName = this.userDisplayName;
		attendee.rsvp = "FALSE";
		attendee.userType = "INDIVIDUAL";
		attendee.role = "REQ-PARTICIPANT";

		attendee.participationStatus = participationMap[aMyResponseType];

		return attendee;
	},

	readRecurrenceRule: function _readRecurrenceRule(aElement)
	{
		/*
		 * The Mozilla recurrence API is bothersome and dictated by libical.
		 *
		 * We need to obey and build an iCalendar string which we feed in
		 * to get the proper recurrence info.
		 */

		var comps = {};
	
		for each (var rec in aElement) {
			switch (rec.localName()) {
			case "RelativeYearlyRecurrence":
			case "AbsoluteYearlyRecurrence":
				comps['FREQ'] = "YEARLY";
				break;
			case "RelativeMonthlyRecurrence":
			case "AbsoluteMonthlyRecurrence":
				comps['FREQ'] = "MONTHLY";
				break;
			case "WeeklyRecurrence":
				comps['FREQ'] = "WEEKLY";
				break;
			case "DailyRecurrence":
				comps['FREQ'] = "DAILY";
				break;
			case "NoEndRecurrence":
			case "EndDateRecurrence":
			case "NumberedRecurrence":
				break;
			default:
				this.logInfo("skipping " + rec.localName());
				continue;
			}
	
			var weekdays = [];
			var week = [];
			for each (var comp in rec) {
				switch (comp.localName()) {
				case 'DaysOfWeek':
					for each (let day in comp.toString().split(" ")) {
						weekdays = weekdays.concat(dayMap[day]);
					}
					break;
				case 'DayOfWeekIndex':
					week = weekMap[comp.toString()];
					break;
				case 'Month':
					comps['BYMONTH'] = monthMap[comp.toString()];
					break;
				case 'DayOfMonth':
					comps['BYMONTHDAY'] = comp.toString();
					break;
				case 'FirstDayOfWeek':
					comps['WKST'] = dayMap[comp.toString()];
					break;
				case 'Interval':
					comps['INTERVAL'] = comp.toString();
					break;
				case 'StartDate':
					/* Dunno what to do with this; no place to set */
					break;
				case 'EndDate':
					comps['UNTIL'] = comp.toString().replace(/Z$/, '');
					break;
				case 'NumberOfOccurrences':
					comps['COUNT'] = comp.toString();
					break;
				}
			}

			let wdtemp = weekdays;
			weekdays = [];
			for each (let day in wdtemp) {
				weekdays.push(week + day);
			}
			if (weekdays.length > 0) {
				comps['BYDAY'] = weekdays.join(',');
			}
		}

		var compstrs = [];
		for (var k in comps) {
			compstrs.push(k + "=" + comps[k]);
		}
	
		if (compstrs.length == 0) {
			return null;
		}
	
		var recrule = cal.createRecurrenceRule();
	
		/* need to do this re-assign game so that the string gets parsed */
		var prop = recrule.icalProperty;
		prop.value = compstrs.join(';');
		recrule.icalProperty = prop;

		return recrule;
	},

	readRecurrence: function _readRecurrence(aItem, aElement)
	{
		var recrule = this.readRecurrenceRule(aElement.nsTypes::Recurrence.children());
	
		if (recrule === null) {
			return null;
		}
	
		var recurrenceInfo = cal.createRecurrenceInfo(aItem);
		recurrenceInfo.setRecurrenceItems(1, [recrule]);
	
		return recurrenceInfo;
	},

	readDeletedOccurrences: function _readDeletedOccurrences(aItem, aElement)
	{
		for each (var mit in aElement.nsTypes::DeletedOccurrences.children()) {
			for (var index in this.itemCache) {
				if ( (this.itemCache[index]) &&
				     (this.itemCache[index].parentItem.id == aItem.id) &&
				     (this.itemCache[index].startDate.compare(this.tryToSetDateValue(mit.nsTypes::Start)) == 0) ) {
					this.logInfo("readDeletedOccurrences 2a");
					aItem.recurrenceInfo.removeOccurrenceAt(this.itemCache[index].startDate);
					//this.itemCache[index].clearAlarms();
					this.deleteAlarms(this.itemCache[index]);
					this.observers.notify("onDeleteItem", [this.itemCache[index]]);
					break;
				}
			}

			
		}
	},

	removeChildrenFromMaster: function _removeChildrenFromMaster(aMaster)
	{
		this.logInfo("removeChildrenFromMaster start. Title:"+aMaster.title);
		// Remove children of this master. They will be added later.
		for (var index in this.itemCache) {
			if ( (this.itemCache[index]) &&
			     (isEvent(this.itemCache[index])) &&
			     (this.itemCache[index].parentItem.id == aMaster.id) ) {
				this.logInfo("removeChildrenFromMaster: Removing child:"+this.itemCache[index].title+", startdate="+this.itemCache[index].startDate.toString());
				this.itemCache[index].parentItem = this.itemCache[index];
				this.observers.notify("onDeleteItem", [this.itemCache[index]]);
				delete this.itemCache[index];
			}
		}
		this.logInfo("removeChildrenFromMaster end.:"+aMaster.title);
	},

	setCommonValues: function _setCommonValues(aItem, aExchangeItem)
	{
		switch(aExchangeItem.nsTypes::Importance.toString()) {
			case "Low" : 
				aItem.priority = 9;
				break;
			case "Normal" : 
				aItem.priority = 5;
				break;
			case "High" : 
				aItem.priority = 1;
				break;
		}

		switch (aExchangeItem.nsTypes::Sensitivity.toString()) {
			case "Normal" : 
				aItem.privacy = "PUBLIC";
				break;
			case "Confidential" : 
				aItem.privacy = "CONFIDENTIAL";
				break;
			case "Personal" : 
				aItem.privacy = "PRIVATE";
				break;
			case "Private" : 
				aItem.privacy = "PRIVATE";
				break;
			default :
				aItem.privacy = "PUBLIC";
		}

		switch (aExchangeItem.nsTypes::LegacyFreeBusyStatus.toString()) {
			case "Free" : 
				aItem.setProperty("TRANSP", "TRANSPARENT");
				break;
			case "Busy" : 
				aItem.setProperty("TRANSP", "OPAQUE");
				break;
			case "Tentative" : 
				aItem.setProperty("TRANSP", "OPAQUE");
				break;
			case "OOF" : 
				aItem.setProperty("TRANSP", "OPAQUE");
				break;
		}

		if (aExchangeItem.nsTypes::IsCancelled.toString() == "true") {
			this.setStatus(aItem, "Decline");
		}
		else {
			this.setStatus(aItem, aExchangeItem.nsTypes::MyResponseType.toString());
		}
	},

	setStatus: function _setStatus(aItem, aStatus)
	{
		switch (aStatus) {
			case "Unknown" : 
				aItem.setProperty("STATUS", "NONE");
				break;
			case "Organizer" : 
				aItem.setProperty("STATUS", "CONFIRMED");
				break;
			case "Tentative" : 
				aItem.setProperty("STATUS", "TENTATIVE");
				break;
			case "Accept" : 
				aItem.setProperty("STATUS", "CONFIRMED");
				break;
			case "Decline" : 
				aItem.setProperty("STATUS", "CANCELLED");
				break;
			case "NoResponseReceived" : 
				aItem.setProperty("STATUS", "NONE");
				break;
			default:
				aItem.setProperty("STATUS", "NONE");
				break;
		}
	},

	cloneItem: function _cloneItem(aItem) {
		var newItem = aItem.clone();
		if (aItem.id == aItem.parentItem.id) {
			newItem.parentItem = newItem;
		}
		else {
			newItem.parentItem = aItem.parentItem;
		}

		return newItem;
	},

	// Taken from soapin.js from Exchange Provider add-on
	// This as part of the conversion from Exchange Provider add-on to Exchange Calendar add-on.
	readEP_DismissSnoozeState: function _readEP_DismissSnoozeState(aItem, it)
	{
		let nextRem = it.nsTypes::ExtendedProperty.(nsTypes::ExtendedFieldURI.@PropertyId == '34144').nsTypes::Value[0];
	
		if (!nextRem) {
			return;
		}
	
		nextRem = cal.fromRFC3339(nextRem.toString(), cal.UTC());
		let lastAck = nextRem.clone();
		lastAck.addDuration(cal.createDuration('-PT1S'));
	
		aItem.alarmLastAck = lastAck;
	
		/*
		 * According to MS, we should compare against:
		 * nextRem.compare(cal.fromRFC3339("4501-01-01T00:00:00Z", cal.UTC())) == 0
		 * However, servers out there seem to have a garbled database, so we now
		 * check more liberally.
		 *
		 * <http://msdn.microsoft.com/en-us/library/ee204414(v=EXCHG.80).aspx>
		 */
		if (nextRem.compare(cal.fromRFC3339("3000-01-01T00:00:00Z", cal.UTC())) > 0) {
			return;
		}
	
		if (!aItem.recurrenceInfo) {
			aItem.setProperty('X-MOZ-SNOOZE-TIME', nextRem.icalString);
		} else {
			let nextOcc = aItem.recurrenceInfo.getNextOccurrence(lastAck);
			let alarm = aItem.getAlarms({})[0];
	
			if (nextOcc && alarm) {
				let nextOccAlarm = cal.alarms.calculateAlarmDate(nextOcc, alarm);
				let alarmDiff = nextOccAlarm.compare(nextRem);
	
				if (alarmDiff == 0) {
					/* no item was snoozed, this is the regular alarm date */
				} else {
					if (alarmDiff > 0) {
						/*
						 * The alarm of the next item will happen after
						 * the remider time.  This means we did snooze
						 * a previous alarm.
						 */
						nextOcc = aItem.recurrenceInfo.getPreviousOccurrence(nextRem);
					}
					aItem.setProperty('X-MOZ-SNOOZE-TIME-' + nextOcc.recurrenceId.nativeTime,
							  nextRem.icalString);
				}
			}
		}
	},


	convertExchangeAppointmentToCalAppointment: function _convertExchangeAppointmentToCalAppointment(aCalendarItem, isMeetingRequest)
	{
		this.logInfo("convertExchangeAppointmentToCalAppointment:"+String(aCalendarItem));
		var item = createEvent();
		item.calendar = this.superCalendar;

		item.id = this.tryToSetValue(aCalendarItem.nsTypes::ItemId.@Id.toString(), item.id);
		if (! item.id) {
			this.logInfo("Item.id is missing. this is a required field.");
			return null;
		}

		item.setProperty("X-ChangeKey", aCalendarItem.nsTypes::ItemId.@ChangeKey.toString());
		item.setProperty("X-CalendarItemType", aCalendarItem.nsTypes::CalendarItemType.toString());
		item.setProperty("X-UID", aCalendarItem.nsTypes::UID.toString());

		item.title = this.tryToSetValue(aCalendarItem.nsTypes::Subject.toString(), "");
		if (! item.title) {
			item.title = "";
		}
		this.logInfo("convertExchangeAppointmentToCalAppointment: item.title:"+item.title);

		item.setProperty("X-LastModifiedTime", aCalendarItem.nsTypes::LastModifiedTime.toString());
		// Check if we allready have this item and if this one is newer.
		if (!isMeetingRequest) {
			if (this.itemCache[item.id]) {
				// We allready have this item.
				var oldItem = this.itemCache[item.id];
				if (item.getProperty("X-LastModifiedTime") <= oldItem.getProperty("X-LastModifiedTime")) {
					//this.logInfo("We received an older or not modified item. We are going to skip it. OldLastModified:"+oldItem.getProperty("X-LastModifiedTime")+", currentLastModified:"+item.getProperty("X-LastModifiedTime"));
					return null;
				}
			}
			else {
				// Check if we have a master.
				if ((aCalendarItem.nsTypes::CalendarItemType.toString() == "RecurringMaster") && (this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()])) {
					// We allready have this master item.
					var oldItem = this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()];
					if (item.getProperty("X-LastModifiedTime") <= oldItem.getProperty("X-LastModifiedTime")) {
						this.logInfo("We received an older or not modified master item. We are going to skip it. OldLastModified:"+oldItem.getProperty("X-LastModifiedTime")+", currentLastModified:"+item.getProperty("X-LastModifiedTime"));
						return null;
					}
				}
			}			
		}
		

		if (aCalendarItem.nsTypes::IsCancelled.toString() == "true") {
			item.setProperty("X-IsCancelled", true);
		}
		else {
			item.setProperty("X-IsCancelled", false);
		}

		if (aCalendarItem.nsTypes::IsMeeting.toString() == "true") {
			item.setProperty("X-IsMeeting", true);
		}
		else {
			item.setProperty("X-IsMeeting", false);
		}

		item.setProperty("DESCRIPTION", aCalendarItem.nsTypes::Body.toString());

		this.setCommonValues(item, aCalendarItem);

		item.setProperty("X-IsInvitation", "false");
		// Check what kind of item this is.
		for each (var prop in aCalendarItem.nsTypes::ResponseObjects.*) {
			switch (prop.localName()) {
				case "AcceptItem":
				case "TentativelyAcceptItem":
				case "DeclineItem":
					item.setProperty("X-IsInvitation", "true");
					item.setProperty("X-MOZ-SEND-INVITATIONS", true);
					break
			}
		}

/*
  <t:ResponseObjects>
    <t:AcceptItem/>
    <t:TentativelyAcceptItem/>
    <t:DeclineItem/>
    <t:ReplyToItem/>
    <t:ReplyAllToItem/>
    <t:ForwardItem/>
    <t:CancelCalendarItem/>  // part of my own items.
  </t:ResponseObjects>

*/

		var cats = [];
		for each (var cat in aCalendarItem.nsTypes::Categories.nsTypes::String) {
			cats.push(cat.toString());
		}
		item.setCategories(cats.length, cats);

		item.startDate = this.tryToSetDateValue(aCalendarItem.nsTypes::Start, item.startDate);
		if (! item.startDate) {
			this.logInfo("We have an empty startdate. Skipping this item.");
			return null;
		}

		item.endDate = this.tryToSetDateValue(aCalendarItem.nsTypes::End, item.endDate);
		if (! item.endDate) {
			this.logInfo("We have an empty enddate. Skipping this item.");
			return null;
		}

		if ((aCalendarItem.nsTypes::ReminderIsSet == "true")) {

			let alarm = cal.createAlarm();
			alarm.action = "DISPLAY";
			alarm.repeat = 0;

			//var alarmDueDate = item.startDate.clone();
			var alarmDueDate = this.tryToSetDateValue(aCalendarItem.nsTypes::ReminderDueBy, item.startDate.clone());

			let alarmOffset = cal.createDuration();
			alarmOffset.minutes = - aCalendarItem.nsTypes::ReminderMinutesBeforeStart;

			// This is a bug fix for when the offset is more than a year)
			if (alarmOffset.minutes < (-60*24*365)) {
				alarmOffset.minutes = -5;
			}

			alarmOffset.normalize();

			if (item.startDate.compare(alarmDueDate) != 0) {
				// We have a different due date.
				alarmDueDate.addDuration(alarmOffset);	
				alarm.related = Ci.calIAlarm.ALARM_RELATED_ABSOLUTE;
				alarm.alarmDate = alarmDueDate.clone();
			}
			else {
				// We have an offset to the startdate.
				alarm.related = Ci.calIAlarm.ALARM_RELATED_START;
				alarm.offset = alarmOffset;
			}

			item.addAlarm(alarm);
		}

		// Check for Attachments
/*		if (item.title.indexOf("att") > -1) {
			this.logInfo("convertExchangeAppointmentToCalAppointment:"+String(aCalendarItem));
		}

		if (aCalendarItem.nsTypes::HasAttachments.toString() == "true") {
			for each(var fileAttachment in aCalendarItem.nsTypes::Attachments.nsTypes::FileAttachment) {
				this.logInfo(" -- Attachment: name="+fileAttachment.nsTypes::Name.toString());

				var newAttachment = createAttachment();
				newAttachment.uri = makeURL("exchangecalendar://"+this.id+"/?id="+encodeURIComponent(item.id)+"&name="+encodeURIComponent(fileAttachment.nsTypes::Name.toString()));

				// We first store it as a Property.
				item.setProperty("X-FileAttachment-"+fileAttachment.nsTypes::AttachmentId.@Id.toString(), { name: fileAttachment.nsTypes::Name.toString(),
																size: Number(fileAttachment.nsTypes::Size.toString()),
																id: fileAttachment.nsTypes::AttachmentId.@Id.toString(),
																lastModified: fileAttachment.nsTypes::LastModifiedTime.toString()});
				item.addAttachment(newAttachment);
			}
		} */

		// Check if our custom fields are set
		var extendedProperties = aCalendarItem.nsTypes::ExtendedProperty;
		var doNotHandleOldAddon = false; 
		for each(var extendedProperty in extendedProperties) {

			var propertyName = extendedProperty.nsTypes::ExtendedFieldURI.@PropertyName.toString();
			switch (propertyName) {
				case "alarmLastAck" :
					//this.logInfo("  alarmLastAck:"+extendedProperty.nsTypes::Value.toString());
					item.alarmLastAck = this.tryToSetDateValue(extendedProperty.nsTypes::Value, null);
					doNotHandleOldAddon = true;
					break;
				case "X-MOZ-SNOOZE-TIME" :
					//this.logInfo("  X-MOZ-SNOOZE-TIME:"+extendedProperty.nsTypes::Value.toString());
					item.setProperty("X-MOZ-SNOOZE-TIME", extendedProperty.nsTypes::Value.toString());
					doNotHandleOldAddon = true;
					break;
				case "X-MOZ-SNOOZE-TIME-" :
					//this.logInfo("  X-MOZ-SNOOZE-TIME-:"+extendedProperty.nsTypes::Value.toString());
					var snoozeTimes = extendedProperty.nsTypes::Value.toString().split("|");
					snoozeTimes.shift();
					for each(var snoozeTime in snoozeTimes) {
						//this.logInfo("             1:"+snoozeTime);
						var propName = snoozeTime.substr(0, snoozeTime.indexOf("="));
						var propValue = snoozeTime.substr(snoozeTime.indexOf("=")+1);
						//this.logInfo("             2:"+propName+"="+propValue);
						if (propValue.indexOf("+") > -1) {
							var tmpTime = this.tryToSetDateValue(propValue, null).getInTimezone(ews.ecUTC());
							propValue = cal.toRFC3339(tmpTime);
							//this.logInfo("             3:"+propName+"="+propValue);
						}
						item.setProperty(propName, propValue);
					}
					doNotHandleOldAddon = true;
					break;
				case "lastLightningModified":
					var lastLightningModified = this.tryToSetDateValue(extendedProperty.nsTypes::Value, null);
					var lastModifiedTime = this.tryToSetDateValue(aCalendarItem.nsTypes::LastModifiedTime, null);

					if ((lastLightningModified) && (lastModifiedTime)) {
						if (lastModifiedTime.compare(lastLightningModified) == 1) {
							this.logInfo("  -- Item has been modified on the Exchange server with another client.");
							item.setProperty("X-ChangedByOtherClient", true);
						}
					}

					break;
			}
		}

		if (aCalendarItem.nsTypes::IsAllDayEvent == "true") {
			// Check if the time is 00:00:00
		//	if ((item.startDate.hour == 0) && (item.startDate.minute == 0) && (item.startDate.second == 0)) {
				item.startDate.isDate = true;
		//	}

		//	if ((item.endDate.hour == 0) && (item.endDate.minute == 0) && (item.endDate.second == 0)) {
				item.endDate.isDate = true;
		//	}
		}

		item.setProperty("DTSTAMP", this.tryToSetDateValue(aCalendarItem.nsTypes::DateTimeReceived));
		item.setProperty("LOCATION", aCalendarItem.nsTypes::Location.toString());

		var myResponseType = null;
		if (aCalendarItem.nsTypes::MyResponseType.length() > 0) {
			myResponseType = aCalendarItem.nsTypes::MyResponseType.toString();
		}

		if (aCalendarItem.nsTypes::Organizer.length() > 0) {
			this.logInfo(" ==A ORGANIZER== title:"+item.title+", org:"+String(aCalendarItem.nsTypes::Organizer));
			var org = this.createAttendee(aCalendarItem.nsTypes::Organizer, "CHAIR");
			org.isOrganizer = true;
			item.organizer = org;
		}

		if (aCalendarItem.localName() == "MeetingRequest") {
			this.logInfo(" X-IsInvitation : MeetingRequest title="+item.title);
			item.setProperty("X-IsInvitation", "true");
			item.setProperty("X-MOZ-SEND-INVITATIONS", true);
		}
		else {
			this.logInfo("                  MeetingItem title="+item.title);
			var iAmInTheList = false;
			var tmpAttendee;

			for each (var at in aCalendarItem.nsTypes::RequiredAttendees.nsTypes::Attendee) {
				tmpAttendee = this.createAttendee(at, "REQ-PARTICIPANT", myResponseType);
				item.addAttendee(tmpAttendee);
				if ((! iAmInTheList) && (tmpAttendee.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase())) {
					iAmInTheList = true;
				}
			}
			for each (var at in aCalendarItem.nsTypes::OptionalAttendees.nsTypes::Attendee) {
				tmpAttendee = this.createAttendee(at, "OPT-PARTICIPANT", myResponseType);
				item.addAttendee(tmpAttendee);
				if ((! iAmInTheList) && (tmpAttendee.id.replace(/^mailto:/, '').toLowerCase() == this.mailbox.toLowerCase())) {
					iAmInTheList = true;
				}
			}

			if ((myResponseType) && (! iAmInTheList)) {
				item.addAttendee(this.createMeAsAttendee(myResponseType));
			}
		}
		
		item.parentItem = item;
//		item.recurrenceId = this.tryToSetDateValue(aCalendarItem.nsTypes::RecurrenceId, item.recurrenceId);
		item.recurrenceId = null;
		if (!isMeetingRequest) {
			this.logInfo(" == item.title:"+item.title+", calendarItemType:"+aCalendarItem.nsTypes::CalendarItemType.toString());
			switch (aCalendarItem.nsTypes::CalendarItemType.toString()) {
				case "Exception" :
					// Try to find master. If found add Exception and link recurrenceinfo.
					item.recurrenceId = this.tryToSetDateValue(aCalendarItem.nsTypes::RecurrenceId, item.startDate);
					var master = this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()];
					if (master) {
						// We allready have a master in Cache.
						item.parentItem = master;
						master.recurrenceInfo.modifyException(item, true);
					}
					else {
						this.logError("HAS NO MASTER: STRANGE: Exception:"+item.title);
					}

					/*if (!item.recurrenceInfo) {
						item.recurrenceInfo = cal.createRecurrenceInfo(item);
					}*/
					break;
				case "Occurrence" :
					// This is a occurrence. Try to find the master and link recurrenceinfo.
					item.recurrenceId = this.tryToSetDateValue(aCalendarItem.nsTypes::RecurrenceId, item.startDate);
					var master = this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()];
					if (master) {
						// We allready have a master in Cache.
						item.parentItem = master;
					}
					else {
						this.logError("HAS NO MASTER: STRANGE: Occurrence:"+item.title);
					}
					
					/*if (!item.recurrenceInfo) {
						item.recurrenceInfo = cal.createRecurrenceInfo(item);
					}*/
					break;
				case "RecurringMaster" :
	
					// Remove children of this master. They will be added later.
					//this.logInfo("RecurringMaster: Going to remove children.");

					//this.removeChildrenFromMaster(item);  // BUG 13.o
	
					// This is a master so create recurrenceInfo.
					item.recurrenceInfo = this.readRecurrence(item, aCalendarItem);
	
					// Try to find occurrences of this master which have their parentItem not
					// yet set to this one. Also set right recurrenceinfo for exceptions.
					// !!! This should not happen. Masters are allways cached before occurrence or exception.
					// 24-10-2011 I never have seen the below mentioned line in my logs. That is OK
					var loadChildren = false;
					var childCount = 0;

					for (var index in this.itemCache) {

						if ( (this.itemCache[index]) &&
						     (isEvent(this.itemCache[index])) &&
						     ( (this.itemCache[index].getProperty("X-CalendarItemType") == "Occurrence") ||
						       (this.itemCache[index].getProperty("X-CalendarItemType") == "Exception") ) &&
						     (this.itemCache[index].getProperty("X-UID") == item.getProperty("X-UID")) &&
						     (this.itemCache[index].parentItem.id == item.id) ) {
							childCount++;
						}

						if ( (this.itemCache[index]) &&
						     (isEvent(this.itemCache[index])) &&
						     ( (this.itemCache[index].getProperty("X-CalendarItemType") == "Occurrence") ||
						       (this.itemCache[index].getProperty("X-CalendarItemType") == "Exception") ) &&
						     (this.itemCache[index].getProperty("X-UID") == item.getProperty("X-UID")) &&
						     (this.itemCache[index].parentItem.id != item.id) ) {
							this.logInfo("convertExchangeAppointmentToCalAppointment: WE SEE THIS LINE. SOLVE IT subject:"+item.title);
							this.itemCache[index].parentItem = item;
							if (this.itemCache[index].getProperty("X-CalendarItemType") == "Exception") {
								item.recurrenceInfo.modifyException(this.itemCache[index], true);
							}
						}

					}

					this.logInfo("title:"+item.title+", childCount:"+childCount+", firstSyncDone:"+this.firstSyncDone);

//					if ((childCount == 0) && (this.firstSyncDone)) {
					if ((this.firstSyncDone)) {
						this.logInfo("We have a master and the sync is done. We will download it's children.");
						if (childCount > 0) {
							this.logInfo("  It also has children so we first remove them.");
							this.removeChildrenFromMaster(item);
						}
						loadChildren = true;
					}

					// Reset snooze/lastAck state if we allready have this master in cache
					if (this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()]) {
						var oldMaster = this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()];
						if (oldMaster.alarmLastAck) {
							item.alarmLastAck = oldMaster.alarmLastAck.clone();
						}
						if (oldMaster.propertyEnumerator) {
							let enumerator = oldMaster.propertyEnumerator;
							while (enumerator.hasMoreElements()) {
								var prop = enumerator.getNext().QueryInterface(Ci.nsIProperty);
								var name = prop.name;
								if (name.indexOf("X-MOZ-SNOOZE-TIME") == 0) {
									item.setProperty(name, oldMaster.getProperty(name));
								}
							}
						}
					}
	
					this.recurringMasterCache[aCalendarItem.nsTypes::UID.toString()] = item;
	
					if ((loadChildren) || (this.newMasters[aCalendarItem.nsTypes::UID.toString()])) {

						this.logInfo("We have a master and it was set as new. So we download it's children.title:"+item.title);
						delete this.newMasters[aCalendarItem.nsTypes::UID.toString()];

						var self = this;
						// Request children from EWS server.
						var childRequestItem = {Id: aCalendarItem.nsTypes::ItemId.@Id.toString(),
							  ChangeKey: aCalendarItem.nsTypes::ItemId.@ChangeKey.toString(),
							  type: aCalendarItem.nsTypes::CalendarItemType.toString(),
							  uid: aCalendarItem.nsTypes::UID.toString(),
							  start: aCalendarItem.nsTypes::Start.toString(),
							  end: aCalendarItem.nsTypes::End.toString()};
					
						this.addToQueue( erFindOccurrencesRequest, 
						{user: this.user, 
						 mailbox: this.mailbox,
						 folderBase: this.folderBase,
						 serverUrl: this.serverUrl,
						 masterItem: childRequestItem,
						 folderID: this.folderID,
						 changeKey: this.changeKey,
						 startDate: this.startDate,
						 endDate: this.endDate,
				 		 GUID: calExchangeCalendarGUID}, 
						function(erGetItemsRequest, aIds) { self.findOccurrencesOK(erGetItemsRequest, aIds);}, 
						function(erGetItemsRequest, aCode, aMsg) { self.findCalendarItemsError(erGetItemsRequest, aCode, aMsg);},
						null);
					}

					item.recurrenceId = this.tryToSetDateValue(aCalendarItem.nsTypes::RecurrenceId, item.startDate);
	
					// Microsoft remindernexttime
					if (!doNotHandleOldAddon) {
						this.readEP_DismissSnoozeState(item, aCalendarItem);
					}

					return null;  // The master will not be visible

					break;
			}
		}

		// Microsoft remindernexttime
		if (!doNotHandleOldAddon) {
			this.readEP_DismissSnoozeState(item, aCalendarItem);
		}

	/*	if (!item.recurrenceInfo) {
			item.recurrenceInfo = cal.createRecurrenceInfo(item);
		}*/

		item.setProperty("X-fromExchange", true);
		return item;

	},

	convertExchangeTaskToCalTask: function _convertExchangeTaskToCalTask(aTask)
	{
		//this.logInfo("TasK:"+String(aTask));
		var item = createTodo();

		item.entryDate = this.tryToSetDateValue(aTask.nsTypes::StartDate, null);
		item.dueDate = this.tryToSetDateValue(aTask.nsTypes::DueDate, item.dueDate);
		item.completedDate = this.tryToSetDateValue(aTask.nsTypes::CompleteDate, item.completedDate);
		item.percentComplete = this.tryToSetValue(aTask.nsTypes::PercentComplete.toString(), item.percentComplete);
		item.calendar = this.superCalendar;

		item.id = this.tryToSetValue(aTask.nsTypes::ItemId.@Id.toString(), item.id);
		item.setProperty("X-ChangeKey", aTask.nsTypes::ItemId.@ChangeKey.toString());
		item.setProperty("X-UID", "dummy");

		item.title = this.tryToSetValue(aTask.nsTypes::Subject.toString(), item.title);

		this.setCommonValues(item, aTask);

		var cats = [];
		for each (var cat in aTask.nsTypes::Categories.nsTypes::String) {
			cats.push(cat.toString());
		}
		item.setCategories(cats.length, cats);

		switch(aTask.nsTypes::Status.toString()) {
			case "NotStarted" : 
				item.status = "NONE";
				break;
			case "InProgress" : 
				item.status = "IN-PROCESS";
				break;
			case "Completed" : 
				item.status = "COMPLETED";
				break;
			case "WaitingOnOthers" : 
				item.status = "NEEDS-ACTION";
				break;
			case "Deferred" : 
				item.status = "CANCELLED";
				break;
		}

		if (aTask.nsTypes::ReminderIsSet == "true") {

			let alarm = cal.createAlarm();
			alarm.action = "DISPLAY";
			alarm.repeat = 0;

			if ((item.entryDate) && (item.entryDate.compare(cal.now()) > 0)) {
				var alarmDueDate = item.entryDate.clone();
			}
			else {
				var alarmDueDate = this.tryToSetDateValue(aTask.nsTypes::ReminderDueBy, item.entryDate);
			}

			if (alarmDueDate) {
				let alarmOffset = cal.createDuration();
				alarmOffset.minutes = - aTask.nsTypes::ReminderMinutesBeforeStart;
				alarmOffset.normalize();

				if ((!item.entryDate) || (item.entryDate.compare(alarmDueDate) != 0)) {
					// We have a different due date.
					alarmDueDate.addDuration(alarmOffset);	
					alarm.related = Ci.calIAlarm.ALARM_RELATED_ABSOLUTE;
					alarm.alarmDate = alarmDueDate.clone();
				}
				else {
					// We have an offset to the startdate.
					alarm.related = Ci.calIAlarm.ALARM_RELATED_START;
					alarm.offset = alarmOffset;
				}

				// Only add alarms which are in the future.
				var alarmDate = cal.alarms.calculateAlarmDate(item, alarm);			
				if (alarmDate.compare(cal.now()) > 0) {
					item.addAlarm(alarm);
				}
			}
		}

		if (aTask.nsTypes::IsRecurring.toString() == "true") {
			item.parentItem = item;
			// This is a master so create recurrenceInfo.
			item.recurrenceInfo = this.readRecurrence(item, aTask);
			if (this.syncBusy) {
				this.readDeletedOccurrences(item, aTask);
			}
		}

		var tmpDate;

		tmpDate = this.tryToSetDateValue(aTask.nsTypes::DateTimeCreated);
		if (tmpDate) {
            		item.setProperty("CREATED", tmpDate);
		}

		tmpDate = this.tryToSetDateValue(aTask.nsTypes::LastModifiedTime);
		if (tmpDate) {
			item.setProperty("LAST-MODIFIED", tmpDate);
		}

		item.setProperty("DESCRIPTION", aTask.nsTypes::Body.toString());

		item.parentItem = item;

		item.setProperty("X-fromExchange", true);
		return item;
	},

	convertExchangeToCal: function _convertExchangeToCal(aExchangeItem)
	{
//		this.logInfo("convertExchangeToCal");
		if (!aExchangeItem) { return; }

		var switchValue = aExchangeItem.nsTypes::ItemClass.toString();
		if (switchValue.indexOf(".{") > -1) {
			switchValue = switchValue.substr(0,switchValue.indexOf(".{"));
		}
		switch (switchValue) {
			case "IPM.Appointment" :
			case "IPM.Appointment.Occurrence" :
			case "IPM.OLE.CLASS" :
			case "IPM.Schedule.Meeting.Request":
			case "IPM.Schedule.Meeting.Canceled":
				return this.convertExchangeAppointmentToCalAppointment(aExchangeItem);
				break;
			case "IPM.Task" :
				return this.convertExchangeTaskToCalTask(aExchangeItem);
				break;
			default :
				this.logInfo("WARNING: convertExchangeToCal: unknown ItemClass = '"+switchValue+"'");
		}
	},

	updateCalendar: function _updateCalendar(erGetItemsRequest, aItems)
	{
//		this.logInfo("updateCalendar");
		var items = [];
		for each (var exchItem in aItems) {

			var item = this.convertExchangeToCal(exchItem);
			if (item) {
				if (!this.itemCache[item.id]) {
					// This is a new unknown item
					this.itemCache[item.id] = item;

					this.logInfo("updateCalendar: onAddItem:"+ item.title);
					this.observers.notify("onAddItem", [item]);
				}
				else {
					// I Allready known this one.
					this.logInfo("updateCalendar: onModifyItem");

					//Copy snooze/lastack status
					if (this.itemCache[item.id].alarmLastAck) {
						item.alarmLastAck = this.itemCache[item.id].alarmLastAck.clone();
					}
					if (this.itemCache[item.id].getProperty("X-MOZ-SNOOZE-TIME")) {
						item.setProperty("X-MOZ-SNOOZE-TIME", this.itemCache[item.id].getProperty("X-MOZ-SNOOZE-TIME"));
					}

					this.singleModified(item, true);
					//this.observers.notify("onModifyItem", [item, this.itemCache[item.id]]);
					// Remove Alarms of old item.
					/*var tmpItem = this.itemCache[item.id].clone();
					tmpItem.clearAlarms();
					this.itemCache[item.id] = item;*/
				}
			}

		}

	},


	getCalendarItemsOK: function _getCalendarItemsOK(erGetItemsRequest, aItems)
	{
//		this.logInfo("getCalendarItemsOK: aItems.length="+aItems.length);
		this.saveCredentials(erGetItemsRequest.argument);

		if (aItems.length == 0) {
			return;
		}

		this.updateCalendar(erGetItemsRequest, aItems);

	},

	getCalendarItemsError: function _getCalendarItemsError(erGetItemsRequest, aCode, aMsg)
	{
		this.saveCredentials(erGetItemsRequest.argument);

	},

	get isVersion2010()
	{
		if (getEWSServerVersion(this.serverUrl).indexOf("2010") > -1 ) {
			return true;
		}

		return false;
	},

	get isVersion2007()
	{
		if (getEWSServerVersion(this.serverUrl).indexOf("2007") > -1 ) {
			return true;
		}

		return false;
	},

	getSyncState: function _getSyncState()
	{
		if (!this.weAreSyncing) {
			// We do not yet have a syncState. Get it first.
			var self = this;
			this.weAreSyncing = true;
			this.addToQueue(erSyncFolderItemsRequest,
					{user: this.user, 
				 mailbox: this.mailbox,
				 serverUrl: this.serverUrl,
				 folderBase: this.folderBase,
				 folderID: this.folderID,
				 changeKey: this.changeKey,
				 syncState: this.syncState,
				 actionStart: Date.now() },
				function(erSyncFolderItemsRequest, creations, updates, deletions, syncState) { self.syncFolderItemsOK(erSyncFolderItemsRequest, creations, updates, deletions, syncState);}, 
				function(erSyncFolderItemsRequest, aCode, aMsg) { self.syncFolderItemsError(erSyncFolderItemsRequest, aCode, aMsg);},
				null);

		}
	},

	syncFolderItemsOK: function _syncFolderItemsOK(erSyncFolderItemsRequest, creations, updates, deletions, syncState)
	{
		this.folderPathStatus = 0;
		this.saveCredentials(erSyncFolderItemsRequest.argument);

		//this.logInfo("syncFolderItemsOK: Creation:"+creations.length+", Updates:"+updates.length+", Deletions:"+deletions.length);

		if ((creations.length > 0) || (updates.length > 0) || (deletions.length > 0)) {
			this.addActivity(calGetString("calExchangeCalendar", "syncFolderEventMessage", [creations.length, updates.length, deletions.length, this.name], "exchangecalendar"), "", erSyncFolderItemsRequest.argument.actionStart, Date.now());
		}

		if (!this.syncState) {
			// This was the first time. we now save the syncState;
			this.syncState = syncState;
			this.weAreSyncing = false;
			this.processItemSyncQueue();

		}
		else {
			this.weAreSyncing = false;
			if ((creations.length == 0) && (updates.length == 0) && (deletions.length == 0)) {
				// Nothing was changed. 
				//this.logInfo("Sync finished. Nothing finished.");
				this.syncState = syncState;

				if (this.getItemSyncQueue.length > 0) {
					this.logInfo("We have "+this.getItemSyncQueue.length+" items in this.getItemSyncQueue");
				}

				this.processItemSyncQueue();

				if (!this.firstSyncDone) { 
					this.firstSyncDone = true;
					this.logInfo("First sync is done. Normal operation is starting.");
				}

				return;
			}

			this.syncBusy = true;

			// Do something with the output.
			if ((this.syncState) && (syncState == this.syncState)) {
				this.logError("Same syncState received.");
			}

			this.syncState = syncState;

			var self = this;
			if (creations.length > 0) {
				this.addToQueue( erGetItemsRequest, 
					{user: this.user, 
					 mailbox: this.mailbox,
					 folderBase: this.folderBase,
					 serverUrl: this.serverUrl,
					 ids: creations,
					 folderID: this.folderID,
					 changeKey: this.changeKey,
					 GUID: calExchangeCalendarGUID }, 
					function(erGetItemsRequest, aIds) { self.getTaskItemsOK(erGetItemsRequest, aIds);}, 
					function(erGetItemsRequest, aCode, aMsg) { self.getTaskItemsError(erGetItemsRequest, aCode, aMsg);},
					null);
			}

			if (updates.length > 0) {
				this.addToQueue( erGetItemsRequest, 
					{user: this.user, 
					 mailbox: this.mailbox,
					 folderBase: this.folderBase,
					 serverUrl: this.serverUrl,
					 ids: updates,
					 folderID: this.folderID,
					 changeKey: this.changeKey,
					 GUID: calExchangeCalendarGUID }, 
					function(erGetItemsRequest, aIds) { self.getTaskItemsOK(erGetItemsRequest, aIds);}, 
					function(erGetItemsRequest, aCode, aMsg) { self.getTaskItemsError(erGetItemsRequest, aCode, aMsg);},
					null);
			}

			if (deletions.length > 0) {
				for each (var deleted in deletions) {
					var item = this.itemCache[deleted.Id];
					if (item) {
						// We have this one. Remove it.
						this.logInfo("Going to remove an item");
						// Single item or occurrence.
						if (item.parentItem.id == item.id) {
							this.logInfo("This is a Single to delete");
						}
						else {
							this.logInfo("This is a Occurrence or Exception to delete. THIS SHOULD NEVER HAPPEN.");
						}
						//item.clearAlarms();
						this.deleteAlarms(item);
						this.observers.notify("onDeleteItem", [item]);
						delete this.itemCache[item.id];
					}
					else {
						// Find matching master record.
						var master;
						for (var index in this.recurringMasterCache) {
							if ((this.recurringMasterCache[index]) && (this.recurringMasterCache[index].id == deleted.Id)) {
								master = this.recurringMasterCache[index]
							}
						}
						if (master) {
							// This is a master recurrence. Also remove children.
							this.logInfo("This is Master to delete");
							this.removeChildrenFromMaster(master);
							this.observers.notify("onDeleteItem", [master]);
							delete this.recurringMasterCache[master.getProperty("X-UID")];
						}
						else {
							this.logInfo("Do not know what you are trying to delete !!!");
						}
					}
				}
				this.syncBusy = false;

				this.processItemSyncQueue();

			}

			if (!this.firstSyncDone) { 
				this.firstSyncDone = true;
				this.logInfo("First sync is done. Normal operation is starting.");
			}

		}

	},

	syncFolderItemsError: function _syncFolderItemsError(erSyncFolderItemsRequest, aCode, aMsg)
	{
		this.folderPathStatus = -1;
		this.saveCredentials(erSyncFolderItemsRequest.argument);
		this.weAreSyncing = false;
		this.processItemSyncQueue();
	},

	// Check if specified folder still exists. If so get new id and changekey.
	checkFolderPath: function _checkFolderPath()
	{
		if (this.folderPath != "/") {
			this.logInfo("checkFolderPath 1");
			this.folderPathStatus = 1;
			var self = this;

			this.addToQueue( erFindFolderRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 serverUrl: this.serverUrl,
				 folderBase: this.folderBase,
				 folderPath: this.folderPath,
				 actionStart: Date.now()}, 
				function(erFindFolderRequest, aId, aChangeKey, aFolderClass) { self.checkFolderPathOk(erFindFolderRequest, aId, aChangeKey, aFolderClass);}, 
				function(erFindFolderRequest, aCode, aMsg) { self.checkFolderPathError(erFindFolderRequest, aCode, aMsg);},
				null);
		}
		else {
			this.logInfo("checkFolderPath 2");
			this.folderPathStatus = 1;
			var self = this;

			this.addToQueue( erGetFolderRequest,
				{user: this.user, 
				 mailbox: this.mailbox,
				 serverUrl: this.serverUrl,
				 folderBase: this.folderBase,
				 folderPath: this.folderPath,
				 actionStart: Date.now()}, 
				function(erGetFolderRequest, aId, aChangeKey, aFolderClass) { self.getFolderOk(erGetFolderRequest, aId, aChangeKey, aFolderClass);}, 
				function(erGetFolderRequest, aCode, aMsg) { self.checkFolderPathError(erGetFolderRequest, aCode, aMsg);},
				null);
		}
	},

	setSupportedItems: function _setSupportedItems(aFolderClass)
	{
		switch (aFolderClass) {
			case "IPF.Appointment":
				this.supportsEvents = true;
				this.supportsTasks = false;
				break;
			case "IPF.Task":
				this.supportsEvents = false;
				this.supportsTasks = true;
				break;
			case "IPF.Note":
				this.supportsEvents = true;
				this.supportsTasks = true;
				break;
			default:
				this.supportsEvents = false;
				this.supportsTasks = false;
				break;
		}		
	},

	checkFolderPathOk: function _checkFolderPathOk(erFindFolderRequest, aId, aChangeKey, aFolderClass)
	{
		this.saveCredentials(erFindFolderRequest.argument);
		this.logInfo("checkFolderPathOk: aId"+aId+", aChangeKey:"+aChangeKey+", aFolderClass:"+aFolderClass);
		
		this.folderID = aId;
		this.changeKey = aChangeKey;
		
		this.setSupportedItems(aFolderClass);		
	},

	getFolderOk: function _checkFolderPathOk(erGetFolderRequest, aId, aChangeKey, aFolderClass)
	{
		this.saveCredentials(erGetFolderRequest.argument);
		this.logInfo("getFolderOk: aId"+aId+", aChangeKey:"+aChangeKey+", aFolderClass:"+aFolderClass);
		
		this.setSupportedItems(aFolderClass);		
	},

	checkFolderPathError: function _checkFolderPathError(erFindFolderRequest, aCode, aMsg)
	{
		this.folderPathStatus = -1;
		this.logInfo("checkFolderPathError: Msg"+aMsg);
		// We could not get the specified folder anymore. Stop working.
	},

	getTimeZones: function _getTimeZones()
	{
		// This only works for Exchange 2010 servers
		this.logInfo("getTimeZones 1");
		var self = this;
		if (this.isVersion2010) {
			this.addToQueue(erGetTimeZonesRequest,
					{user: this.user, 
					 serverUrl: this.serverUrl,
					 actionStart: Date.now() },
					function(erGetTimeZonesRequest, aTimeZoneDefinitions) { self.getTimeZonesOK(erGetTimeZonesRequest, aTimeZoneDefinitions);}, 
					function(erGetTimeZonesRequest, aCode, aMsg) { self.getTimeZonesError(erGetTimeZonesRequest, aCode, aMsg);},
					null);
		}

		if (this.isVersion2007) {
			this.logInfo("getTimeZones 2");
			this.logInfo("getTimeZones for 2007");
			this.EWSTimeZones = ecGetEWSTimeZones(ews_2010_timezonedefinitions);
			this.haveTimeZones = true;
		}
	},

	getTimeZonesOK: function _getTimeZonesOK(erGetTimeZonesRequest, aTimeZoneDefinitions)
	{
		this.saveCredentials(erGetTimeZonesRequest.argument);
		this.EWSTimeZones = aTimeZoneDefinitions;
		this.logInfo("getTimeZonesOK");
		this.haveTimeZones = true;
	},

	getTimeZonesError: function _getTimeZonesError(erGetTimeZonesRequest, aCode, aMsg)
	{
		this.logInfo("getTimeZonesError: Msg"+aMsg);
		this.haveTimeZones = false;
	},

	calculateBiasOffsets: function _calculateBiasOffsets(aCalTimeZone)
	{
		var tzcomp = aCalTimeZone.icalComponent;
	
		var dsttz = null;
		for (var comp = tzcomp.getFirstSubcomponent("DAYLIGHT");
		     comp;
		     comp = tzcomp.getNextSubcomponent("DAYLIGHT")) {
			if (!dsttz || dsttz.getFirstProperty("DTSTART").valueAsDatetime.compare(
					comp.getFirstProperty("DTSTART").valueAsDatetime) < 0) {
				dsttz = comp;
			}
		}
		var stdtz = null;
		for (var comp = tzcomp.getFirstSubcomponent("STANDARD");
		     comp;
		     comp = tzcomp.getNextSubcomponent("STANDARD")) {
			if (!stdtz || stdtz.getFirstProperty("DTSTART").valueAsDatetime.compare(
					comp.getFirstProperty("DTSTART").valueAsDatetime) < 0) {
				stdtz = comp;
			}
		}
	
		if (!stdtz) {
			return {};
		}
	
		// Get TZOFFSETTO from standard time.
		var m = stdtz.getFirstProperty("TZOFFSETTO").value.match(/^([+-]?)(\d\d)(\d\d)$/);
		var biasOffset = cal.createDuration();
		biasOffset.hours = m[2];
		biasOffset.minutes = m[3];
		if (m[1] == '+') {
			biasOffset.isNegative = true;
		}

		var tmpBaseOffset = biasOffset.icalString;

		var daylightOffset = null;
		if (dsttz) {
			var m = dsttz.getFirstProperty("TZOFFSETTO").value.match(/^([+-]?)(\d\d)(\d\d)$/);
			daylightOffset = cal.createDuration();
			daylightOffset.hours = m[2];
			daylightOffset.minutes = m[3];
			if (m[1] == '+') {
				daylightOffset.isNegative = true;
			}
		}
	
		if (daylightOffset) {
			return { standard: biasOffset.icalString,
				 daylight: daylightOffset.icalString } ;
		}
		else {
			return { standard: biasOffset.icalString,
				 daylight: null } ;
		}
	},

	convertDurationToSeconds: function _convertDurationToSeconds(aDuration)
	{
		var tmpStr = aDuration;
		var multiplier = 1;
		if (tmpStr.substr(0,1) == "-") {
			multiplier = -1;
			tmpStr = tmpStr.substr(1);
		}

		var total = 0;
		var subtotal = 0;

		if (tmpStr.substr(0,2) == "PT") {
			tmpStr = tmpStr.substr(2);

			var counter = 0;
			while (counter < tmpStr.length) {
				if (isNaN(tmpStr.substr(counter, 1))) {
					switch (tmpStr.substr(counter, 1).toUpperCase()) {
						case "D":
							subtotal = subtotal * 3600 * 24;
							break;
						case "H":
							subtotal = subtotal * 3600;
							break;
						case "M":
							subtotal = subtotal * 60;
							break;
						case "S":
							subtotal = subtotal;
							break;
					}
					total = total + subtotal;
					subtotal = 0;
					//this.logInfo(" ++ total:"+total);
				}
				else {
					subtotal = (subtotal * 10) + Number(tmpStr.substr(counter, 1));
					//this.logInfo(" ++ subtotal:"+subtotal);
				}
				counter = counter + 1;
			}

		}


		return total * multiplier;
		
	},

	getEWSTimeZoneId: function _getEWSTimeZoneId(aCalTimeZone)
	{
		this.logInfo("getEWSTimeZoneId:"+aCalTimeZone.tzid);
		if (this.EWSTimeZones) {
			
			if (aCalTimeZone.isFloating) {
				var tmpZone = ews.ecDefaultTimeZone();
			}
			else {
				var tmpZone = aCalTimeZone;
			}

			var weHaveAMatch = null;
			var tmpPlaceName = null;
			if (tmpZone.tzid.indexOf("/") > -1) {
				// Get City/Place name from tzid.
				tmpPlaceName = tmpZone.tzid.substr(tmpZone.tzid.indexOf("/")+1);
			}


			var tmpBiasValues = this.calculateBiasOffsets(tmpZone);
			if (!tmpBiasValues.standard) {
				return "UTC";
			}

			//if (tmpBiasValues.standard.indexOf("PT0") == 0) {
			//	this.logInfo("Changing tmpBiasValues.standard="+tmpBiasValues.standard+ " -> PT0H");
			//	tmpBiasValues.standard = "PT0H";
			//}
			this.logInfo("tmpBiasValues.standard="+tmpBiasValues.standard);
			if (tmpBiasValues.daylight) {
				this.logInfo("tmpBiasValues.daylight="+tmpBiasValues.daylight);
			}

			for each(var timeZoneDefinition in this.EWSTimeZones) {
				//this.logInfo("timeZoneDefinition.@Name="+timeZoneDefinition.@Name);
				var placeNameMatch = false;
				if ((tmpPlaceName) && (timeZoneDefinition.@Name.indexOf(tmpPlaceName) > -1)) {
					// We found our placename in the name of the timezonedefinition
					placeNameMatch = true;
				}

				var standardMatch = null;
				var periods = timeZoneDefinition.nsTypes::Periods..nsTypes::Period.(@Name == "Standard");
				if (periods) {
					for each (var period in periods) {
						//this.logInfo("xx period.@Bias="+period.@Bias.toString());
						if (this.convertDurationToSeconds(period.@Bias.toString()) == this.convertDurationToSeconds(tmpBiasValues.standard)) {
							standardMatch = period.@Bias.toString();
							break;
						}
					}
				}
		
				if (standardMatch) {
					var daylightMatch = null;
					if (tmpBiasValues.daylight) {
						var periods = timeZoneDefinition.nsTypes::Periods..nsTypes::Period.(@Name == "Daylight");
						if (periods) {
							for each (var period in periods) {
								//this.logInfo("yy period.@Bias="+period.@Bias.toString());
								if (this.convertDurationToSeconds(period.@Bias.toString()) == this.convertDurationToSeconds(tmpBiasValues.daylight)) {
									daylightMatch = period.@Bias.toString();
									break;
								}
							}
						}
					}
	
					if ((standardMatch) && ((!tmpBiasValues.daylight) || (daylightMatch))) {
						this.logInfo("WE HAVE A TIMEZONE MATCH BETWEEN LIGHTNING AND EWS. Cal:"+aCalTimeZone.tzid+", EWS:"+timeZoneDefinition.@Name);
	
						// If we also found the place name this will overrule everything else.
						if ((placeNameMatch) || (!weHaveAMatch)) {
							weHaveAMatch = timeZoneDefinition.@Id;
	
							if (placeNameMatch) {
								this.logInfo("We have a timzonematch on place name");
								break;
							}
						}
					}
				}

			}

			return weHaveAMatch;
		}

		return "UTC";
	},

	doDeleteCalendar: function _doDeleteCalendar()
	{
		this.doShutdown();
	},

	doShutdown: function _doShutdown()
	{
		if (this.shutdown) {
			return;
		}

		this.shutdown = true;
		this.inboxPoller.cancel();

		for each(var timer in this.timers) {
			if (timer) {
				timer.cancel();
			}
		}

		if (this.calendarPoller) {
			this.calendarPoller.cancel();
		}

		for each(var tmpJob in this.tmpJobs) {
			if (tmpJob) {
				if ((tmpJob.isRunning) && (tmpJob.parent)) {
					tmpJob.parent.stopRequest();
				}
			}
		}

		for each(var tmpQueue in this.queue) {
			this.observerService.notifyObservers(this, "onExchangeProgressChange", -1*tmpQueue.length); 
		}

		this.queue = new Array;

		// Now we can initialize.
		this.syncState = null;
		this.weAreSyncing = false;
		
		// Remove all items in cache from calendar.
/*		for (var index in this.itemCache) {
			if (this.itemCache[index]) {
				//this.itemCache[index].clearAlarms();
				this.deleteAlarms(this.itemCache[index]);
				this.observers.notify("onDeleteItem", [this.itemCache[index]]);
			}
		} */ // This is removed to speed up shutdown

		// Reset caches.
		this.itemCache = [];
		this.recurringMasterCache = [];

		this.meetingRequestsCache = [];
		this.meetingCancelationsCache = [];
		this.meetingrequestAnswered = [];
		this.meetingResponsesCache = [];
	},

	removeFromMeetingRequestCache: function _removeFromMeetingRequestCache(aID) {
		//this.logInfo("removeFromMeetingRequestCache:"+aUID);
//		this.meetingRequestsCache[aUID] == null;
		delete this.meetingRequestsCache[aID];
	},

	deleteAlarms: function _deleteAlarms(aItem)
	{
		return;
		//this.logInfo("deleteAlarms using alarmService");
		if (! aItem) {
			return;
		}

		var alarmService = Cc["@mozilla.org/calendar/alarm-service;1"]
					.getService(Ci.calIAlarmService);

		var alarms = aItem.getAlarms({});

		for each(var alarm in alarms) {
			//this.logInfo("Removing alarm for:"+aItem.title);
			alarmService.dismissAlarm(aItem,alarm);
		}
		//this.logInfo("deleteAlarms done.");

	},

	doTimezoneChanged: function _doTimezoneChanged()
	{
		this.logInfo("doTimeZoneChanged 1");
		return;

		var prefTzid = cal.getPrefSafe("calendar.timezone.local", null);
		this.logInfo("-- New timezone id:"+prefTzid);
		var newTimezone = ews.ecTZService().getTimezone(prefTzid);
		this.logInfo("doTimeZoneChanged 2");

		// Get all cached items into new timezone
		for (var index in this.itemCache) {
			if (this.itemCache[index]) {
				//var oldItem = this.itemCache[index].clone();
				var oldItem = this.cloneItem(this.itemCache[index]);
				if (! this.itemCache[index].startDate.isDate) {
					this.logInfo("mod startDate 1:"+this.itemCache[index].startDate.toString());
					this.itemCache[index].startDate = this.itemCache[index].startDate.getInTimezone(newTimezone);
					this.logInfo("mod startDate 2:"+this.itemCache[index].startDate.toString());
				}
				if (! this.itemCache[index].endDate.isDate) {
					this.logInfo("mod endDate");
					this.itemCache[index].endDate = this.itemCache[index].endDate.getInTimezone(newTimezone);
				}
				//this.itemCache[index].clearAlarms();
				this.observers.notify("onModifyItem", [this.itemCache[index],oldItem]);
			}
		}
		this.logInfo("doTimeZoneChanged 3");

	},

	findItemInListByDatesAndID: function _findItemInListByDates(aList, aItem)
	{
		var result = null;
		for each(var listItem in aList) {
			if ((aItem.id == listItem.getProperty("X-UID")) &&
				(listItem.startDate.compare(aItem.startDate) == 0) &&
				(listItem.endDate.compare(aItem.endDate) == 0)) {
				this.logInfo("Found matching item in list.");
				result = listItem;
			}
		}

		return result;
	},

    /**
     * Internal logging function that should be called on any database error,
     * it will log as much info as possible about the database context and
     * last statement so the problem can be investigated more easilly.
     *
     * @param message           Error message to log.
     * @param exception         Exception that caused the error.
     */
	logError: function _logError(message,exception) {
		let logMessage = "("+this.name+") " + message;

		if (exception) {
			logMessage += "\nException: " + exception;
		}

		ews.ERROR(logMessage + "\n" + ews.STACK(10));
	},

	logInfo: function(message) {
		ews.LOG("["+this.name+"] "+message + " ("+ews.STACKshort()+")");
	},
};

function ecObserver(inCalendar)  
{  
	this.calendar = inCalendar;

	var self = this;
	this.ecInvitationsCalendarManagerObserver = {
		onCalendarRegistered: function cMO_onCalendarRegistered(aCalendar) {
//			self.calendar.logInfo("onCalendarRegistered name="+aCalendar.name+", id="+aCalendar.id);
/*			if (aCalendar.id == self.calendar.id) {
				self.calendar.newCalendarCheck();
			}*/
		},

		onCalendarUnregistering: function cMO_onCalendarUnregistering(aCalendar) {
			self.calendar.logInfo("onCalendarUnregistering name="+aCalendar.name+", id="+aCalendar.id);
			if (aCalendar.id == self.calendar.id) {

				self.calendar.doDeleteCalendar();
				self.calendar.logInfo("Removing calendar preference settings.");
	
				var rmPrefs = Cc["@mozilla.org/preferences-service;1"]
			                    .getService(Ci.nsIPrefService)
					    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl.");
				try {
					rmPrefs.deleteBranch(aCalendar.id);
				} catch(err) {}

				self.unregister();
			}
		},

		onCalendarDeleting: function cMO_onCalendarDeleting(aCalendar) {
			self.calendar.logInfo("onCalendarDeleting name="+aCalendar.name+", id="+aCalendar.id);

		}
	};


	this.register();  
}  

ecObserver.prototype = {  

	observe: function(subject, topic, data) 
	{  
		// Do your stuff here.
		//LOG("ecObserver.observe. topic="+topic+",data="+data+"\n"); 
		switch (topic) {
			case "onCalReset":
				if (data == this.calendar.id) {
					this.calendar.resetCalendar();
				}
				break;
			case "quit-application":
				this.unregister();
				break;
			case "nsPref:changed":
				if (data == "calendar.timezone.local") {
					// TODO: For now this will not work because it can happen that 
					// the timezoneservice does not know about the pref update.
					// The order of calling observers is random.
					this.calendar.doTimezoneChanged();
				}
				break;
		} 
	},  

	register: function()
	{  
		var observerService = Cc["@mozilla.org/observer-service;1"]  
		                          .getService(Ci.nsIObserverService);  
		observerService.addObserver(this, "onCalReset", false);  
		observerService.addObserver(this, "quit-application", false); 

		Services.prefs.addObserver("calendar.timezone.local", this, false);

		getCalendarManager().addObserver(this.ecInvitationsCalendarManagerObserver);
	},  

	unregister: function()
	{  
		this.calendar.doShutdown();

		var observerService = Cc["@mozilla.org/observer-service;1"]  
		                            .getService(Ci.nsIObserverService);  
		observerService.removeObserver(this, "onCalReset");  
		observerService.removeObserver(this, "quit-application");  
		Services.prefs.removeObserver("calendar.timezone.local", this);

		getCalendarManager().removeObserver(this.ecInvitationsCalendarManagerObserver);
	}  
}

function fixPrefBug()
{
	var createPrefs = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService)
		    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl.createcalendar.");

	createPrefs.clearUserPref("autodiscover");
	createPrefs.clearUserPref("server");
	createPrefs.clearUserPref("mailbox");
	createPrefs.clearUserPref("displayname");
	createPrefs.clearUserPref("user");
	createPrefs.clearUserPref("domain");
	createPrefs.clearUserPref("folderbase");
	createPrefs.clearUserPref("folderpath");
	createPrefs.clearUserPref("folderID");
	createPrefs.clearUserPref("changeKey");

	var children = createPrefs.getChildList("");
	if (children.length > 0) {
		// Move prefs from old location to new location.
		for (var index in children) {
			ews.LOG("Going to move pref key:"+children[index]);
			var newPrefs = Cc["@mozilla.org/preferences-service;1"]
		                    .getService(Ci.nsIPrefService)
				    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl.");
			switch (createPrefs.getPrefType(children[index])) {
			case createPrefs.PREF_STRING:
				ews.LOG("STRING pref");
				newPrefs.setCharPref(children[index], createPrefs.getCharPref(children[index]));
				break;
			case createPrefs.PREF_INT:
				ews.LOG("INT pref");
				newPrefs.setIntPref(children[index], createPrefs.getIntPref(children[index]));
				break;
			case createPrefs.PREF_BOOL:
				ews.LOG("BOOL pref");
				newPrefs.setBoolPref(children[index], createPrefs.getBoolPref(children[index]));
				break;
			}
			createPrefs.clearUserPref(children[index]);
		}
		
	}

}

function removeOldPrefs()
{
	var currentPrefs = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService)
		    .getBranch("extensions.exchangecalendar@extensions.1st-setup.nl.");

	var children = currentPrefs.getChildList("");
	if (children.length > 0) {
		var oldUUID = "";
		children.sort();
		for (var index in children) {
			if (children[index].indexOf("createcalendar.") == -1) {
				var uuid = children[index].substr(0, children[index].indexOf("."));
				if (uuid != oldUUID) {
					ews.LOG("removeOldPrefs Checking uuid:"+uuid);
					oldUUID = uuid;
					var calPrefs = Cc["@mozilla.org/preferences-service;1"]
				                    .getService(Ci.nsIPrefService)
						    .getBranch("calendar.registry."+uuid+".");
					try {
						var type = calPrefs.getCharPref("type");
					}
					catch(err) {
						ews.LOG("   -- Removing uuid:"+uuid);
						currentPrefs.deleteBranch(uuid);
					}

				}
			}
		}
		
	}

}


/** Module Registration */
const scriptLoadOrder = [
	"calUtils.js"
];

function NSGetFactory(cid) {

	ews.LOG("--XX- calExchangeCalendar--1");
	try {
		if (!NSGetFactory.mainEC) {
			// Load main script from lightning that we need.
			cal.loadScripts(scriptLoadOrder, Cu.getGlobalForObject(this));
			NSGetFactory.mainEC = XPCOMUtils.generateNSGetFactory([calExchangeCalendar]);
			ews.LOG("---XX- huppeltje");
	}

	} catch(e) {
		Components.utils.reportError(e);
		dump(e);
		throw e;
	}

	fixPrefBug();
	removeOldPrefs();

	ews.LOG("---XX- calExchangeCalendar-miv");
	return NSGetFactory.mainEC(cid);
} 


